package ru.ispras.telecomtesk.eap.EAPPEAP;

import ru.ispras.telecomtesk.eap.ConvertTypes;
import ru.ispras.telecomtesk.eap.DebugInfo;
import ru.ispras.telecomtesk.eap.EAPTLS.EAPtypePldTls;

import java.security.MessageDigest;
import java.util.ArrayList;

public class EAPtypePldPEAP extends EAPtypePldTls {    
	public ArrayList<EAPtypePldPEAPtlv> avpList = null; 

	public EAPtypePldPEAP() {
		super();
	}

	public EAPtypePldPEAP(byte[] data) {
		super(data);
	}

	public void setEapTlsFlagVersion(byte ver) {
		this.flag = (byte)((this.flag & 0xf8) | (ver & 0x03));
	}

	public byte getEapTlsFlagVersion() {
		return (byte)(this.flag & 0x03);
	}
	public byte getEapTlsFlagVersion(byte bt) {
		return (byte)(bt & 0x03);
	}

	public int getSize() {
		int res = 1;

		if(isLengthEnable)
			res += 4;
		if(tlsData != null) 
			res += tlsData.length;

		if(avpList != null) {
			DebugInfo.printMessage(1,"EAPtypePldPEAP::getSize: " + avpList.size());
			int ext_len = 0;
			for (int j = 0; j < avpList.size(); j++) {
				ext_len += (avpList.get(j)).getSize();
			}
			res += ext_len;
		}
		return res;
	}

	public int parseNetMessage() {
		int res = 0;
		byte[] data = this.net_msg;
		if(data == null) 
			res = 1;
		else if(data.length == 0) 
			res = 2;
		else if(getEapTlsSBit(data[0]) && (data.length != 1)) 
			res = 3;
		else if( (data.length < 5) && (getEapTlsLBit(data[0]) || getEapTlsMBit(data[0])) ) 
			res = 4;
		else {
			this.flag = data[0];
			int pos = 1;
			
			if(getEapTlsLBit(data[0])) {
				tlsMessageLenght = ConvertTypes.ntoh(data, 1, 4);
				isLengthEnable = true;
				isLengthSet = true;
				pos += 4;
			}
			
			tlsData = new byte[data.length - pos];
			System.arraycopy(data, pos, tlsData, 0, tlsData.length);
		}
		
		return res;
	}


	public int toByteArray() {
		int i = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		ba[i++] = this.flag;

		if(isLengthEnable) {
			byte[] msgln = ConvertTypes.hton(tlsMessageLenght);
			System.arraycopy(msgln, 0, ba, i, msgln.length);
			i += msgln.length;
		}

		if(this.tlsData != null)
			System.arraycopy(this.tlsData, 0, ba, i, this.tlsData.length);

		//write attributes
		if(avpList != null) {
			byte[] avpArray;
			for (int j = 0; j < avpList.size(); j++) {
				if(avpList.get(j).net_msg == null)
					avpList.get(j).toByteArray();
				avpArray = avpList.get(j).net_msg;
				System.arraycopy(avpArray, 0, ba, i, avpArray.length);
				i += avpArray.length;
			}
		}

		this.net_msg = ba;
		return 0;
	}

	public String eapTlsFlag2Str() {
		return String.format("%8s", Integer.toBinaryString(this.flag & 0xFF)).replace(' ', '0');
	}
	
	public String eapTlsMsgLength2Str() {
		if(isLengthEnable)
			return "" + tlsMessageLenght;
		else
			return "not presented";
	}
	
	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "EAP-PEAP payload:");
		DebugInfo.printMessage(1,pref + "  flag: " + eapTlsFlag2Str());
		DebugInfo.printMessage(1,pref + "  TLS length: " + eapTlsMsgLength2Str());
		DebugInfo.printDumpWithLength(1, this.tlsData, pref + "  TLS Data:");
	}
}