package ru.ispras.telecomtesk.eap.EAPPEAP;

import java.security.MessageDigest;
import javax.crypto.*;
import javax.crypto.spec.SecretKeySpec;
import ru.ispras.telecomtesk.eap.BasicPayloadObject;
import ru.ispras.telecomtesk.eap.ConvertTypes;
import ru.ispras.telecomtesk.eap.DebugInfo;


public class EAPtypePldPEAPavp extends BasicPayloadObject {
	public int avpCode;
    public byte flag;         
    public int vendorID;         
    public int length;         
	public byte[] value = null; 
	protected boolean isLengthSet = false;

	public EAPtypePldPEAPavp(int code) {
		this.avpCode = code;
		this.flag = 0;
		this.vendorID = 0;
	}
	
	public EAPtypePldPEAPavp(int code, byte[] data) {
		this.avpCode = code;
		this.flag = 0;
		this.vendorID = 0;
		if(data != null) {
			value = new byte[data.length];
			System.arraycopy(data, 0, value, 0, data.length);
			setLength();
		}
	}
	
	public EAPtypePldPEAPavp(int code, byte flag, int vendorID, byte[] data) {
		this.avpCode = code;
		this.flag = flag;
		this.vendorID = vendorID;
		if(data != null) {
			value = new byte[data.length];
			System.arraycopy(data, 0, value, 0, data.length);
			setLength();
		}
	}
	
	public EAPtypePldPEAPavp(int code, byte[] data, int len) {
		this.avpCode = code;
		this.flag = 0;
		this.vendorID = 0;
		if(data != null) {
			int curlen = data.length >= len ? len : data.length;
			value = new byte[curlen];
			System.arraycopy(data, 0, value, 0, curlen);
			setLength();
		}
	}
	
	public EAPtypePldPEAPavp(byte[] data, int code, int len, int pos) {
		this.avpCode = code;
		this.flag = 0;
		this.vendorID = 0;
		if((data != null)&&(len > 0)&&(pos > 0)) {
			int curlen = 0;
			if(pos <= data.length - 1) {
				curlen = data.length >= len + pos ? len : data.length - pos;
				value = new byte[curlen];
				System.arraycopy(data, pos, value, 0, curlen);
			}
			setLength();
		}
	}
	
	public void setAvpFlag(boolean vBit, boolean mBit) {
		if(vBit) this.flag |= (byte)0x80;
		if(mBit) this.flag |= (byte)0x40;
	}

	public boolean getAvpVBit() {
		return ((this.flag & 0x80) > 0);
	}
	public boolean getAvpMBit() {
		return ((this.flag & 0x40) > 0);
	}
	public boolean getAvpVBit(byte bt) {
		return ((bt & 0x80) > 0);
	}
	public boolean getAvpMBit(byte bt) {
		return ((bt & 0x40) > 0);
	}

	public static EAPtypePldPEAPavp cloneAttribute(EAPtypePldPEAPavp avp) {
		EAPtypePldPEAPavp newAvp = null;
		if(avp != null)
			newAvp = new EAPtypePldPEAPavp(avp.avpCode, avp.flag, avp.vendorID, avp.value);
		return newAvp;
	}
	
	public int getSize() {
		int res = 8;
		if(this.getAvpVBit()) 
			res += 4;
		if(value != null) 
			res += value.length;
		return res;
	}

	public void setLength() {
		length = (byte)getSize();
		isLengthSet = true;
	}

	public void setLength(int len) {
		length = (byte)len;
		isLengthSet = true;
	}


	public int toByteArray() {
		int i = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		byte[] tmpVal = ConvertTypes.hton(avpCode); // 4 bytes Code
		System.arraycopy(tmpVal, 0, ba, i, tmpVal.length);
		i += tmpVal.length;
		
		tmpVal = ConvertTypes.hton(length); // 4 bytes Length but need 3 bytes
		System.arraycopy(tmpVal, 0, ba, i, tmpVal.length);
		ba[i] = flag;  // replace first byte by Flag
		i += tmpVal.length;
		
		if(this.getAvpVBit()) {
			tmpVal = ConvertTypes.hton(vendorID); // 4 bytes vendorID
			System.arraycopy(tmpVal, 0, ba, i, tmpVal.length);
			i += tmpVal.length;
		}
		
		if(value != null) {
			System.arraycopy(value, 0, ba, i, value.length);
		}
		
		net_msg = ba;
		return 0;
	}

	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "  Code: " + this.avpCode);
		DebugInfo.printMessage(1,pref + "  Flag: " + this.flag);
		DebugInfo.printMessage(1,pref + "  vendorID: " + this.vendorID);
		DebugInfo.printMessage(1,pref + "  length: " + this.length);
		DebugInfo.printDump(1, this.value, pref + "  value:");
	}
}    
