package ru.ispras.telecomtesk.eap.EAPPEAP;

import ru.ispras.telecomtesk.eap.BasicPayloadObject;
import ru.ispras.telecomtesk.eap.ConvertTypes;
import ru.ispras.telecomtesk.eap.DebugInfo;


public class EAPtypePldPEAPtlv extends BasicPayloadObject {
	public int tlvType;
    public byte flag;         
    public int length;         
	public byte[] value = null; 
	protected boolean isLengthSet = false;

	public EAPtypePldPEAPtlv(int code) {
		this.tlvType = code;
		this.flag = 0;
	}
	
	public EAPtypePldPEAPtlv(int code, byte[] data) {
		this.tlvType = code;
		this.flag = 0;
		if(data != null) {
			value = new byte[data.length];
			System.arraycopy(data, 0, value, 0, data.length);
			setLength();
		}
	}
	
	public EAPtypePldPEAPtlv(int code, byte flag, byte[] data) {
		this.tlvType = code;
		this.flag = flag;
		if(data != null) {
			value = new byte[data.length];
			System.arraycopy(data, 0, value, 0, data.length);
			setLength();
		}
	}
	
	public EAPtypePldPEAPtlv(int code, byte[] data, int len) {
		this.tlvType = code;
		this.flag = 0;
		if(data != null) {
			int curlen = data.length >= len ? len : data.length;
			value = new byte[curlen];
			System.arraycopy(data, 0, value, 0, curlen);
			setLength();
		}
	}
	
	public EAPtypePldPEAPtlv(byte[] data, int code, int len, int pos) {
		this.tlvType = code;
		this.flag = 0;
		if((data != null)&&(len > 0)&&(pos > 0)) {
			int curlen = 0;
			if(pos <= data.length - 1) {
				curlen = data.length >= len + pos ? len : data.length - pos;
				value = new byte[curlen];
				System.arraycopy(data, pos, value, 0, curlen);
			}
			setLength();
		}
	}
	
	public void setAvpFlag(boolean vBit, boolean mBit) {
		if(vBit) this.flag |= (byte)0x80;
		if(mBit) this.flag |= (byte)0x40;
	}

	public void setTlvMBit() {
		this.flag |= (byte)0x80;
	}
	public void setTlvRBit() {
		this.flag |= (byte)0x40;
	}
	public boolean getTlvMBit() {
		return ((this.flag & 0x80) > 0);
	}
	public boolean getTlvRBit() {
		return ((this.flag & 0x40) > 0);
	}
	public boolean getTlvMBit(byte bt) {
		return ((bt & 0x80) > 0);
	}
	public boolean getTlvRBit(byte bt) {
		return ((bt & 0x40) > 0);
	}

	public static EAPtypePldPEAPtlv cloneAttribute(EAPtypePldPEAPtlv tlv) {
		EAPtypePldPEAPtlv newTlv = null;
		if(tlv != null)
			newTlv = new EAPtypePldPEAPtlv(tlv.tlvType, tlv.flag, tlv.value);
		return newTlv;
	}
	
	public int getSize() {
		int res = 4;
		if(value != null) 
			res += value.length;
		return res;
	}

	public void setLength() {
		length = getSize();
		isLengthSet = true;
	}

	public void setLength(int len) {
		length = len;
		isLengthSet = true;
	}


	public int toByteArray() {
		int i = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		byte[] tmpVal = ConvertTypes.hton((short)tlvType); // 2 bytes Code
		System.arraycopy(tmpVal, 0, ba, i, tmpVal.length);
		ba[i] |= flag;  // set Flag
		i += tmpVal.length;
		
		tmpVal = ConvertTypes.hton((short)length); // 2 bytes Length but need 3 bytes
		System.arraycopy(tmpVal, 0, ba, i, tmpVal.length);
		i += tmpVal.length;
		
		if(value != null) {
			System.arraycopy(value, 0, ba, i, value.length);
		}
		
		net_msg = ba;
		return 0;
	}

	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "  Type: " + this.tlvType);
		DebugInfo.printMessage(1,pref + "  Flag: " + this.flag);
		DebugInfo.printMessage(1,pref + "  length: " + this.length);
		DebugInfo.printDump(1, this.value, pref + "  value:");
	}
}    
