//
// check first message: EAP-TLS-ClientHello with M bit set for Microsoft Server 12
//

package ru.ispras.telecomtesk.eap.EAPPEAP;

import java.io.*;
import java.math.BigInteger;
import java.net.*;
import java.util.ArrayList;
import java.security.*;

import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.*;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.TLSExtensions.*;
import ru.ispras.telecomtesk.tlsv12.RecordProtocol.*;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.*;
import ru.ispras.telecomtesk.tlsv12.TLSState.*;

import javax.crypto.interfaces.DHPublicKey;
import ru.ispras.telecomtesk.eap.*;
import ru.ispras.telecomtesk.eap.EAPTLS.EAPTlsFragmentation;
import ru.ispras.telecomtesk.eap.EAPoverRadius.*;





public class testEAPPeap_1_MsftSrv12
{
    public static void main(String args[]) throws Exception {
    	testEAPPeap_1_MsftSrv12_Run test1 = new testEAPPeap_1_MsftSrv12_Run();
    	test1.runtest();
    }
}

class testEAPPeap_1_MsftSrv12_Run
{
	public static byte[] remoteHostRaw = {(byte)(192),(byte)(168),(byte)(56),(byte)(101)};
	public static byte[] localHostRaw = {(byte)(10),(byte)(10),(byte)(9),(byte)(53)};

	EAPtype testType = EAPtype.EAP_TYPE_PEAP;

	String SharedSecretStr = "testing123";
//	String SharedSecretStr = "secret";
	String userNameStr = "user1-radius";
	String UserPwdStr = "wdcgy159A"; // "123456";
//	String UserPwdStr = "efqcltpm+>y]";

	RecordLayer tlsmsg = null;
	int tlsFullMsgLen = 0;
	ArrayList<byte[]> tlsMsgList = null;
	EAPRadiusSocket sock = null;
	EAPpldBasic ttlsEapPktCH = null;
	EAPtypePldPEAP ttlsPld = null;
	byte[] tlsClHello = null;
	
	public void runtest() throws Exception {
		DebugInfo.debugLevel = 6;
		DebugInfo.maxDumpSize = 2000;
		ru.ispras.telecomtesk.tlsv12.DebugInfo.debugLevel = 3;

		sock = new EAPRadiusSocket(SharedSecretStr, userNameStr, remoteHostRaw); 
		sock.setSocketTimeout(3000);
		tlsMsgList = new ArrayList<byte[]>();
		tlsClHello = createFirstTlsMsg();
		TestPEAPSequence1_MsftSrv12 testSeq1 = new TestPEAPSequence1_MsftSrv12();

		int res;
		int eapTestInd, eapSubInd;

	    for (eapSubInd = 1; eapSubInd <= testSeq1.numberTests[0]; eapSubInd++) {
//		    for (eapSubInd = 1; eapSubInd <= 1; eapSubInd++) {
		    	ttlsPld = new EAPtypePldPEAP(tlsClHello);
		    	ttlsPld = testSeq1.setTestPld(eapSubInd, ttlsPld);
		    	if(ttlsPld == null) {
		    		sock.closeSocket();
		    		return;
		    	}
		    	
		    	res = sendPacketsSequence();
		    	if(res > 1) {
//		    		sock.closeSocket();
//		    		return;
		    	}
		    }
		
	    
		DebugInfo.printMessage(1,"run: counterEAPPackets " + sock.counterEAPPackets + ", counterEAPTerminatePackets " + sock.counterEAPTerminatePackets);

// close socket		
		sock.closeSocket();
	}

	int sendPacketsSequence() {
		int res = 0;
		
		// send init ID packet
		DebugInfo.printMessage(1,"\n================================================");
		
		res = sock.sendInitEAPpacket();
		if((res != 0) || (!sock.receivedData)) { 
			DebugInfo.printMessage(1,"!!! sendInitEAPpacket failed");
			sock.sendTerminatePacket();
			return 1;
		}

		
// send second message		
		DebugInfo.printMessage(1,"-------------------------------");
		DebugInfo.printMessage(1,"create TLS ClientHello");
		if(tlsClHello == null) {
			sock.closeSocket();
			return 2;
		}

		ttlsEapPktCH = new EAPpldBasic();
		ttlsEapPktCH.code = EAPcode.EAP_CODE_RESPONSE;
		ttlsEapPktCH.identifier = sock.getReceivedEAPPacket().identifier;
		ttlsEapPktCH.type = testType;
		ttlsEapPktCH.EAPmethodPld = ttlsPld;
		ttlsEapPktCH.isLengthSet = false;
		ttlsEapPktCH.toByteArray();
		
		sock.setEapMessage(ttlsEapPktCH);

		// get all EAP fragments
/*		res = EAPTlsFragmentation.getAllTlsFragments(sock,tlsMsgList);
		if(res != 0) { 
			sock.sendTerminatePacket();
			return;
		}
*/
		tlsMsgList.clear();
		boolean cicleFlag;
		EAPheader recvEAP;
		tlsFullMsgLen = 0;
		cicleFlag = true;
		int counterEmptyRecv = 0;
		int counterNoAnsw = 0;
		
		while(cicleFlag) {
			DebugInfo.printMessage(1,"---Cicle---");
			res = sock.sendEAPpacket();
			recvEAP = sock.getReceivedEAPPacket();
			if(!sock.receivedData && (counterNoAnsw == 0)) {
				DebugInfo.printMessage(1,"^^^^^^^^^^^^^^^^^^^");
				DebugInfo.printMessage(1,"!!! no answer 1");
				ttlsPld = new EAPtypePldPEAP();
				ttlsEapPktCH.EAPmethodPld = ttlsPld;
				ttlsEapPktCH.isLengthSet = false;
				ttlsEapPktCH.toByteArray();
				
				sock.setEapMessage(ttlsEapPktCH);
				counterNoAnsw++;
				continue;
			} else if(!sock.receivedData && (counterNoAnsw == 1)) {
				DebugInfo.printMessage(1,"^^^^^^^^^^^^^^^^^^^");
				DebugInfo.printMessage(1,"!!! no answer 2");
				ttlsPld = new EAPtypePldPEAP();
				ttlsPld.setEapTlsFlag(true,false,false); // set L bit
				ttlsEapPktCH.EAPmethodPld = ttlsPld;
				ttlsEapPktCH.isLengthSet = false;
				ttlsEapPktCH.toByteArray();
				
				sock.setEapMessage(ttlsEapPktCH);
				counterNoAnsw++;
				continue;
			} else if(!sock.receivedData) {
				DebugInfo.printMessage(1,"^^^^^^^^^^^^^^^^^^^");
				DebugInfo.printMessage(1,"!!! no answer 3");
				break;
			}
			cicleFlag = false;
			DebugInfo.printMessage(1,"---Cicle 2---");

			if(recvEAP instanceof EAPpldBasic) {
				if(((EAPpldBasic) recvEAP).type == testType) {
					EAPtypePldPEAP curpld = (EAPtypePldPEAP)((EAPpldBasic)recvEAP).EAPmethodPld;
					if(curpld.getEapTlsLBit()) {
						if(tlsFullMsgLen == 0)
							tlsFullMsgLen = curpld.tlsMessageLenght;
						else {
							DebugInfo.printMessage(1,"!!! Warning! tlsMessageLenght dublicate");
						}
					}
					tlsMsgList.add(curpld.tlsData);
					if(curpld.getEapTlsMBit()) {
						cicleFlag = true;
						ttlsEapPktCH.identifier = sock.getReceivedEAPPacket().identifier;
						ttlsEapPktCH.EAPmethodPld = new EAPtypePldPEAP();
						ttlsEapPktCH.isLengthSet = false;
						ttlsEapPktCH.toByteArray();
						sock.setEapMessage(ttlsEapPktCH);
					} else if((curpld.tlsData == null) || (curpld.tlsData.length == 0)) {
						DebugInfo.printMessage(1,"*******************");
						cicleFlag = true;
						ttlsEapPktCH.identifier = sock.getReceivedEAPPacket().identifier;
						ttlsEapPktCH.EAPmethodPld = new EAPtypePldPEAP();
						if(counterEmptyRecv > 0)
							((EAPtypePldPEAP)(ttlsEapPktCH.EAPmethodPld)).setEapTlsFlag(true,false,false);
						ttlsEapPktCH.isLengthSet = false;
						ttlsEapPktCH.toByteArray();
						sock.setEapMessage(ttlsEapPktCH);
						counterEmptyRecv++;
					}
				}
			}
		}
///////
		byte[] tlsRaw = EAPTlsFragmentation.combineTlsMessages(tlsMsgList);
		if(tlsRaw == null) {
			DebugInfo.printMessage(1,"!!! Error! tlsRaw Null pointer");
		} else if(tlsRaw.length != tlsFullMsgLen) {
			DebugInfo.printMessage(1,"!!! Error! tlsRaw length failed: " + tlsRaw.length + " must " + tlsFullMsgLen);
		} else {
			DebugInfo.printDumpWithLength(2, tlsRaw, "Get full tls messages");
		}

		// send third message		
		sock.sendTerminatePacket();
		return res;
	}


	byte[] createFirstTlsMsg() {
		DebugInfo.printMessage(3, "testEAPTtlsv0_1::createFirstTlsMsg");
		int res = 0;
		byte[] resTls = null;
		ArrayList<RecordLayer> response;
		short[] cs = {TLSCipherSuite.TLS_RSA_WITH_AES_128_CBC_SHA.getValue(),
					  TLSCipherSuite.TLS_RSA_WITH_AES_256_CBC_SHA256.getValue()
					 };
		Hello msg = new Hello(cs); 
		SignatureAndHashAlgorithm[] sah = {
				new SignatureAndHashAlgorithm(HashAlgorithm.SHA1, SignatureAlgorithm.RSA),
				new SignatureAndHashAlgorithm(HashAlgorithm.MD5, SignatureAlgorithm.RSA),
				new SignatureAndHashAlgorithm(HashAlgorithm.SHA256, SignatureAlgorithm.RSA)
				};
		SignatureAndHashAlgorithmExt hl_ext_sah = new SignatureAndHashAlgorithmExt(sah);
		msg.addExtension(hl_ext_sah);

		RecordLayer tlsmsg = new RecordLayer();
		tlsmsg.msglist = new ArrayList();
		tlsmsg.msglist.add(msg);

		TLSConnectionSelector sel = new TLSConnectionSelector(null,null,0,0);
		TLSConnection current_connection = new TLSConnection(sel,true);
		current_connection.init();
		
		try {
			msg.toByteArray();
			DebugInfo.printDump(5, msg.net_msg, "testEAPTtlsv0_1::createFirstTlsMsg: ClientHello msg: ");
		} catch (Exception err) {
			DebugInfo.printMessage(1,"===Error! testEAPTtlsv0_1::createFirstTlsMsg: ClientHello Exception: " + err);                   res = 1;
		}

		try {
			tlsmsg.setContent(msg,null,current_connection);
		} catch (Exception err) {
			DebugInfo.printMessage(1,"===Error! testEAPTtlsv0_1::createFirstTlsMsg:Record layer Exception: " + err);
			res = 2;
		}

		if(res == 0)
			resTls = tlsmsg.toByteArray();

		return resTls;
	}
}




class TestPEAPSequence1_MsftSrv12
{
	int numberTestsMbit = 10;

	int[] numberTests = {numberTestsMbit};
    
	TestPEAPSequence1_MsftSrv12() {
	}
	
	public EAPtypePldPEAP setTestPld(int eapSubInd, EAPtypePldPEAP pld) {
		DebugInfo.printMessage(1,"setTestPld");
		EAPtypePldPEAP tnpPld = null;
		
		if(pld == null) {
			DebugInfo.printMessage(1,"!!! ERROR: setTestPld: null pointer");
			return tnpPld;
		}
		
		tnpPld = setMbit(eapSubInd, pld);
		
		DebugInfo.printMessage(1,"setTestPld return");
		return tnpPld;
	}
	
	public EAPtypePldPEAP setMbit(int ind, EAPtypePldPEAP pld) {
		int i;
		byte[] tmpba;
		pld.setEapTlsFlag(false,true,false); // set M bit

		switch(ind) {
		case 1: pld.setEapTlsFlag(false,false,true); break; // set Start bit
		case 2: break; // 
		case 3: pld.tlsData = null; break; // 
		case 4: pld.setEapTlsFlag(true,false,false); // set L  
			pld.setLength(0); break; // Length = 0
		case 5: pld.setEapTlsFlag(true,false,false); // set L  
			pld.setLength(pld.tlsData.length - 5); break; // Length < tlsData.length
		case 6: pld.setEapTlsFlag(true,false,false); // set L  
			pld.setLength(pld.tlsData.length + 5); break; // Length > tlsData.length
		case 7: pld.setEapTlsFlag(true,false,false); // set L  
			pld.setLength();
			i = pld.tlsData.length; 
			tmpba = new byte[i + 8];
			System.arraycopy(pld.tlsData, 0, tmpba, 0, i);
			for(;i<tmpba.length;i++) {
				tmpba[i] = 0;
			}
			pld.tlsData = tmpba;
			break; // Length = 0
		case 8: pld.setEapTlsFlag(true,false,false); // set L 
			pld.setLength();
			i = pld.tlsData.length; 
			tmpba = new byte[i + 8];
			System.arraycopy(pld.tlsData, 0, tmpba, 0, i);
			for(;i<tmpba.length;i++) {
				tmpba[i] = (byte)i;
			}
			pld.tlsData = tmpba;
			break; // Length = 0
		case 9: pld.setEapTlsFlag(true,false,false); // set L  
			pld.setLength();
			i = pld.tlsData.length; 
			tmpba = new byte[i + 8];
			System.arraycopy(pld.tlsData, 0, tmpba, 0, i);
			for(;i<tmpba.length;i++) {
				tmpba[i] = 0;
			}
			pld.tlsData = tmpba;
			pld.setLength();
			break; // Length = 0
		case 10: pld.setEapTlsFlag(true,false,false); // set L 
			pld.setLength();
			i = pld.tlsData.length; 
			tmpba = new byte[i + 8];
			System.arraycopy(pld.tlsData, 0, tmpba, 0, i);
			for(;i<tmpba.length;i++) {
				tmpba[i] = (byte)i;
			}
			pld.tlsData = tmpba;
			pld.setLength();
			break; // Length = 0
		}

		return pld;
	}

}
