//
// check second message: EAP-TLS-...-Finished --- ACK message
//

package ru.ispras.telecomtesk.eap.EAPPEAP;

import java.io.*;
import java.math.BigInteger;
import java.net.*;
import java.util.ArrayList;
import java.security.*;

import ru.ispras.telecomtesk.tlsv12.Tlsv12Client;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.*;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.TLSExtensions.*;
import ru.ispras.telecomtesk.tlsv12.RecordProtocol.*;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.*;
import ru.ispras.telecomtesk.tlsv12.TLSSocket.TLSResponse;
import ru.ispras.telecomtesk.tlsv12.TLSState.*;

import javax.crypto.interfaces.DHPublicKey;
import ru.ispras.telecomtesk.eap.*;
import ru.ispras.telecomtesk.eap.EAPTLS.EAPTlsFragmentation;
import ru.ispras.telecomtesk.eap.EAPTTLSv0.EAPtypePldTtlsv0;
import ru.ispras.telecomtesk.eap.EAPTTLSv0.EAPtypePldTtlsv0AVP;
import ru.ispras.telecomtesk.eap.EAPoverRadius.*;





public class testEAPPeap_tls_1
{
    public static void main(String args[]) throws Exception {
    	testEAPPeap_tls_1_Run test1 = new testEAPPeap_tls_1_Run();
    	test1.runtest();
    }
}

class testEAPPeap_tls_1_Run
{
	public static byte[] remoteHostRaw = {(byte)(127),(byte)(0),(byte)(0),(byte)(1)};
//	public static byte[] remoteHostRaw = {(byte)(192),(byte)(168),(byte)(56),(byte)(101)};
	public static byte[] localHostRaw = {(byte)(10),(byte)(10),(byte)(9),(byte)(53)};

	PrintWriter log1;

	EAPtype testType = EAPtype.EAP_TYPE_PEAP;

	String SharedSecretStr = "testing123";
//	String SharedSecretStr = "secret";
	String userNameStr = "tester";
//	String userNameStr = "user1-radius";
	String UserPwdStr = "tester"; 
//	String UserPwdStr = "wdcgy159A"; // "123456";
//	String UserPwdStr = "efqcltpm+>y]";

	RecordLayer tlsmsg = null;
	int tlsFullMsgLen = 0;
	ArrayList<byte[]> tlsMsgList = null;
	EAPRadiusSocket sock = null;
	int eapTestInd = 0, eapSubInd = 0;

	Tlsv12Client tlsClnt;
	TestPeapTLSSequence1 testSeq1;
	
	public void runtest() throws Exception {
		DebugInfo.debugLevel = 3;
		DebugInfo.maxDumpSize = 200;
		ru.ispras.telecomtesk.tlsv12.DebugInfo.debugLevel = 3;

		sock = new EAPRadiusSocket(SharedSecretStr, userNameStr, remoteHostRaw); 
		sock.setSocketTimeout(1000);
		tlsMsgList = new ArrayList<byte[]>();
		testSeq1 = new TestPeapTLSSequence1();
		tlsClnt = new Tlsv12Client();

		int res;
	    for (eapTestInd = 0; eapTestInd < testSeq1.numberTests; eapTestInd++) {
	    	DebugInfo.printMessageWithSeparator(1,"eapTestInd: " + eapTestInd);
	    		res = sendPacketsSequence();
				sock.sendTerminatePacket();
	    }
		DebugInfo.printMessage(1,"run: counterEAPPackets " + sock.counterEAPPackets + ", counterEAPTerminatePackets " + sock.counterEAPTerminatePackets);

// close socket		
		sock.closeSocket();
	}

	
	int sendPacketsSequence() {
		int res = 0;
		
		// send init ID packet
		DebugInfo.printMessage(1,"\n================================================");
		
		res = sock.sendInitEAPpacket();
		if((res != 0) || (!sock.receivedData)) { 
			DebugInfo.printMessage(1,"!!! sendInitEAPpacket failed");
			return 1;
		}
		
		// send second message		
		DebugInfo.printMessageWithSeparator(1,"create TLS ClientHello");
		byte[] tlsClHello = tlsClnt.createClientFirstTlsMsg();
		if(tlsClHello == null) {
			DebugInfo.printMessage(1,"!!! create Tls Client Hello failed");
			return 2;
		}
		EAPpldBasic ttlsEapPktCH = createEapPkt(new EAPtypePldPEAP(tlsClHello));
		if(0 != recvAndParseEAPmsg(ttlsEapPktCH)) return 3;

    	// send third message		
		DebugInfo.printMessageWithSeparator(1,"try create pkt3");
		byte[] msgSF = tlsClnt.createClientSecondTlsFlow();
		if(msgSF == null) {
			DebugInfo.printMessage(1,"!!! create Tls Client second flow failed");
			return 4;
		}
		EAPpldBasic ttlsEapPktThird = createEapPkt(new EAPtypePldPEAP(msgSF));
		if(0 != recvAndParseEAPmsg(ttlsEapPktThird)) return 5;
    	
		// TLS Handshake complete

	   	// send fourth message		
		DebugInfo.printMessageWithSeparator(1,"try create pkt4 under TLS");
		EAPpldBasic ttlsEapPktApp = createEapPkt(testSeq1.setTestPld(eapTestInd, tlsClnt, userNameStr));
		DebugInfo.printMessage(1,"**********************************************************");
		if(0 != recvAndParseEAPmsg(ttlsEapPktApp)) return 7;

		return res;
	}

	EAPpldBasic createEapPkt(EAPtypePldPEAP ttlspld) {
		EAPpldBasic testEapPkt = new EAPpldBasic();
		testEapPkt.code = EAPcode.EAP_CODE_RESPONSE;
		testEapPkt.identifier = sock.getReceivedEAPPacket().identifier;
		testEapPkt.type = testType;
		testEapPkt.EAPmethodPld = ttlspld;
		testEapPkt.isLengthSet = false;
		testEapPkt.toByteArray();
		return testEapPkt;
	}


	int recvFullEAPmsg() {
		int res = 0;
		tlsMsgList.clear();
		boolean cicleFlag;
		EAPheader recvEAP;
		tlsFullMsgLen = 0;
		cicleFlag = true;
		int counterEmptyRecv = 0;
		EAPpldBasic ttlsEapACKpkt = new EAPpldBasic();
		
		ttlsEapACKpkt.code = EAPcode.EAP_CODE_RESPONSE;
		ttlsEapACKpkt.type = testType;

		while(cicleFlag) {
			res = sock.sendEAPpacket();
			if((res != 0) || (!sock.receivedData)) { 
				DebugInfo.printMessage(1,"!!! sendEAPpacket failed");
				return 1;
			}
			recvEAP = sock.getReceivedEAPPacket();
			cicleFlag = false;

			if(recvEAP instanceof EAPpldBasic) {
				if(((EAPpldBasic) recvEAP).type == testType) {
					EAPtypePldPEAP curpld = (EAPtypePldPEAP)((EAPpldBasic)recvEAP).EAPmethodPld;
					if(curpld.getEapTlsLBit()) {
						if(tlsFullMsgLen == 0)
							tlsFullMsgLen = curpld.tlsMessageLenght;
						else {
							DebugInfo.printMessage(1,"!!! Warning! tlsMessageLenght dublicate");
						}
					}
					tlsMsgList.add(curpld.tlsData);
					if(curpld.getEapTlsMBit()) {
						cicleFlag = true;
						ttlsEapACKpkt.identifier = sock.getReceivedEAPPacket().identifier;
						ttlsEapACKpkt.EAPmethodPld = new EAPtypePldPEAP();
						ttlsEapACKpkt.isLengthSet = false;
						ttlsEapACKpkt.toByteArray();
						sock.setEapMessage(ttlsEapACKpkt);
					} else if((curpld.tlsData == null) || (curpld.tlsData.length == 0)) {
						cicleFlag = true;
						ttlsEapACKpkt.identifier = sock.getReceivedEAPPacket().identifier;
						ttlsEapACKpkt.EAPmethodPld = new EAPtypePldPEAP();
						ttlsEapACKpkt.isLengthSet = false;
						if(counterEmptyRecv > 0)
							((EAPtypePldPEAP)(ttlsEapACKpkt.EAPmethodPld)).setEapTlsFlag(true,false,false);
						ttlsEapACKpkt.toByteArray();
						sock.setEapMessage(ttlsEapACKpkt);
						counterEmptyRecv++;
					}
				}
			}
		}
		return res;
	}

	
	int parseTLSresp(byte[] tlsmsg) {
		DebugInfo.printMessageWithSeparator(1,"parse tls response");
		int res = 0;
		
		TLSResponse resp = new TLSResponse(tlsmsg);
   		resp.printResponse();
    	if(resp.err != 0) {
			DebugInfo.printMessage(1,"!!! TLSResponse return error");
			res = 1;
    	} else {
    		try {
    			ArrayList<RecordLayer> tlsResp = tlsClnt.parseTlsResponse(resp);	
    			if(null == tlsResp  ||  tlsClnt.conSpec.disconnectWithSendAlert != -1) {
    				DebugInfo.printMessage(1,"!!! TLSResponse return error");
    				tlsClnt.conSpec.disconnect();
    				res = 2;
    			}
    		} catch (Exception err) {
    			DebugInfo.printMessage(1,"===Error! parseTlsResponse Exception: " + err);
    			res = 3;
    		}
    	}
    	return res;
	}

	int recvAndParseEAPmsg(EAPpldBasic ttlsEapPkt) {
		int res = 0;
		
		sock.setEapMessage(ttlsEapPkt);

		if(0 != recvFullEAPmsg()) return 1;

		byte[] tlsRaw = EAPTlsFragmentation.combineTlsMessages(tlsMsgList);
		if(tlsRaw == null) {
			DebugInfo.printMessage(1,"!!! Error! tlsRaw Null pointer");
		} else if(tlsRaw.length != tlsFullMsgLen) {
			DebugInfo.printMessage(1,"!!! Error! tlsRaw length failed: " + tlsRaw.length + " must " + tlsFullMsgLen);
		} else {
			DebugInfo.printDumpWithLength(2, tlsRaw, "Get full tls messages");
		}

		// parse tls response		
		if(0 != parseTLSresp(tlsRaw)) return 2;

		return res;
	}

}




class TestPeapTLSSequence1
{
	int numberTests = 4;
    
	TestPeapTLSSequence1() {
	}
	
	public EAPtypePldPEAP setTestPld(int eapTestInd, Tlsv12Client clnt, String username) {
		DebugInfo.printMessage(1,"setTestPld");
		EAPtypePldPEAP tnpPld = null;

		if(clnt == null) {
			DebugInfo.printMessage(1,"!!! ERROR: setTestPld: null pointer");
			return tnpPld;
		}

		String str = "Hello";
		byte[] usrNm = username.getBytes();
		byte[] eapId = new byte[1 + usrNm.length]; // EAP-Identity without header
		eapId[0] = 1;
		System.arraycopy(usrNm, 0, eapId, 1, usrNm.length); // AVP with EAP-Identity
		
		byte[][] testdata = {str.getBytes(),{0,0,0,0,0,0,0,0},eapId};

		switch(eapTestInd) {
		case 0: 
		case 1: 
		case 2: 
			byte[] tlsAppl = clnt.createApplicationData(testdata[eapTestInd]);
			if(tlsAppl == null) {
				DebugInfo.printMessage(1,"!!! create Tls Application message failed");
				return null;
			}
			tnpPld = new EAPtypePldPEAP(tlsAppl); break;
		case 3: tnpPld = new EAPtypePldPEAP(); break;
		}
		
		DebugInfo.printMessage(1,"setTestPld return");
		return tnpPld;
	}
	
}
