//
// check second message: EAP-TLS-...-Finished --- ACK message
//

package ru.ispras.telecomtesk.eap.EAPPEAP;

import java.io.*;
import java.math.BigInteger;
import java.net.*;
import java.util.ArrayList;
import java.security.*;

import ru.ispras.telecomtesk.tlsv12.Tlsv12Client;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.*;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.TLSExtensions.*;
import ru.ispras.telecomtesk.tlsv12.RecordProtocol.*;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.*;
import ru.ispras.telecomtesk.tlsv12.TLSSocket.TLSResponse;
import ru.ispras.telecomtesk.tlsv12.TLSState.*;

import javax.crypto.interfaces.DHPublicKey;
import ru.ispras.telecomtesk.eap.*;
import ru.ispras.telecomtesk.eap.EAPTLS.EAPTlsFragmentation;
import ru.ispras.telecomtesk.eap.EAPTTLSv0.EAPtypePldTtlsv0;
import ru.ispras.telecomtesk.eap.EAPTTLSv0.EAPtypePldTtlsv0AVP;
import ru.ispras.telecomtesk.eap.EAPTTLSv0.IteratorRandomAVP;
import ru.ispras.telecomtesk.eap.EAPoverRadius.*;





public class testEAPPeap_tls_2
{
    public static void main(String args[]) throws Exception {
    	testEAPPeap_tls_2_Run test1 = new testEAPPeap_tls_2_Run();
    	test1.runtest();
    }
}

class testEAPPeap_tls_2_Run
{
	public static byte[] remoteHostRaw = {(byte)(127),(byte)(0),(byte)(0),(byte)(1)};
//	public static byte[] remoteHostRaw = {(byte)(192),(byte)(168),(byte)(56),(byte)(101)};
	public static byte[] localHostRaw = {(byte)(10),(byte)(10),(byte)(9),(byte)(53)};

	PrintWriter log1;

	EAPtype testType = EAPtype.EAP_TYPE_PEAP;

	String SharedSecretStr = "testing123";
//	String SharedSecretStr = "secret";
	String userNameStr = "tester";
//	String userNameStr = "user1-radius";
	String UserPwdStr = "tester"; 
//	String UserPwdStr = "wdcgy159A"; // "123456";
//	String UserPwdStr = "efqcltpm+>y]";

	RecordLayer tlsmsg = null;
	int tlsFullMsgLen = 0;
	ArrayList<byte[]> tlsMsgList = null;
	EAPRadiusSocket sock = null;
	int eapReject = 0, counterSeq = 0, eapNoAnsw = 0;
	boolean tstEap = false;

	Tlsv12Client tlsClnt;
	TestPeapTLSSequence2 testSeq1;

	ArrayList<RecordLayer> tlsResp;
	
	public void runtest() throws Exception {
		DebugInfo.debugLevel = 3;
		DebugInfo.maxDumpSize = 200;
		ru.ispras.telecomtesk.tlsv12.DebugInfo.debugLevel = 3;

		sock = new EAPRadiusSocket(SharedSecretStr, userNameStr, remoteHostRaw); 
		sock.setSocketTimeout(1000);
		tlsMsgList = new ArrayList<byte[]>();
		testSeq1 = new TestPeapTLSSequence2(userNameStr);
		tlsClnt = new Tlsv12Client();

		int res;
	    do {
    		res = sendPacketsSequence(); 
			sock.sendTerminatePacket();
			counterSeq++;
			DebugInfo.printMessage(1,"number of tests " + counterSeq);
			DebugInfo.printMessage(1,"rejected EAP Packets " + eapReject);
			DebugInfo.printMessage(1,"no answer EAP Packets " + eapNoAnsw);
			if(res > 0) {
				try {
					Thread.sleep(5000);
				} catch (Exception err2) {
					DebugInfo.printMessage(1,"sendPacketsSequence: thread: " + err2);
				}
			}

	    } while(!testSeq1.isEnd());

		DebugInfo.printMessage(1,"run: counterEAPPackets " + sock.counterEAPPackets + ", counterEAPTerminatePackets " + sock.counterEAPTerminatePackets);

// close socket		
		sock.closeSocket();
	}

	
	int sendPacketsSequence() {
		int res = 0;
		
		// send init ID packet
		DebugInfo.printMessage(1,"\n================================================");
		
		res = sock.sendInitEAPpacket();
		if((res != 0) || (!sock.receivedData)) { 
			DebugInfo.printMessage(1,"!!! sendInitEAPpacket failed");
			return 1;
		}
		
		// send second message		
		DebugInfo.printMessageWithSeparator(1,"create TLS ClientHello");
		byte[] tlsClHello = tlsClnt.createClientFirstTlsMsg();
		if(tlsClHello == null) {
			DebugInfo.printMessage(1,"!!! create Tls Client Hello failed");
			return 2;
		}
		EAPpldBasic ttlsEapPktCH = createEapPkt(new EAPtypePldPEAP(tlsClHello));
		if(0 != recvAndParseEAPmsg(ttlsEapPktCH)) return 3;

    	// send third message		
		DebugInfo.printMessageWithSeparator(1,"try create pkt3");
		byte[] msgSF = tlsClnt.createClientSecondTlsFlow();
		if(msgSF == null) {
			DebugInfo.printMessage(1,"!!! create Tls Client second flow failed");
			return 4;
		}
		EAPpldBasic ttlsEapPktThird = createEapPkt(new EAPtypePldPEAP(msgSF));
		if(0 != recvAndParseEAPmsg(ttlsEapPktThird)) return 5;
    	
		// TLS Handshake complete

	   	// send ACK message		
		DebugInfo.printMessageWithSeparator(1,"try create ack");
		EAPpldBasic ttlsEapPktAck = createEapPkt(new EAPtypePldPEAP());
		DebugInfo.printMessage(1,"**********************************************************");
		recvAndParseEAPmsg(ttlsEapPktAck);

	   	// send fourth message		
		DebugInfo.printMessageWithSeparator(1,"try create pkt4 under TLS");
		byte[] innerEapFstMsg = testSeq1.setTestPld();
		byte[] tlsAppl = tlsClnt.createApplicationData(innerEapFstMsg);
		EAPpldBasic ttlsEapPktApp = createEapPkt(new EAPtypePldPEAP(tlsAppl));
		DebugInfo.printMessage(1,"**********************************************************");
		tstEap = true;
		if(0 != recvAndParseEAPmsg(ttlsEapPktApp)) return 7;
		tstEap = false;

		return res;
	}

	EAPpldBasic createEapPkt(EAPtypePldPEAP ttlspld) {
		EAPpldBasic testEapPkt = new EAPpldBasic();
		testEapPkt.code = EAPcode.EAP_CODE_RESPONSE;
		testEapPkt.identifier = sock.getReceivedEAPPacket().identifier;
		testEapPkt.type = testType;
		testEapPkt.EAPmethodPld = ttlspld;
		testEapPkt.isLengthSet = false;
		testEapPkt.toByteArray();
		return testEapPkt;
	}


	int recvFullEAPmsg() {
		int res = 0;
		tlsMsgList.clear();
		boolean cicleFlag;
		EAPheader recvEAP;
		tlsFullMsgLen = 0;
		cicleFlag = true;
		int counterEmptyRecv = 0;
		EAPpldBasic ttlsEapACKpkt = new EAPpldBasic();
		
		ttlsEapACKpkt.code = EAPcode.EAP_CODE_RESPONSE;
		ttlsEapACKpkt.type = testType;

		while(cicleFlag) {
			res = sock.sendEAPpacket();
			if((res != 0) || (!sock.receivedData)) { 
				DebugInfo.printMessage(1,"!!! sendEAPpacket failed");
				return 1;
			}
			recvEAP = sock.getReceivedEAPPacket();
			if(tstEap) {
				if(!sock.receivedData)
					eapNoAnsw ++;
				if(recvEAP.code == EAPcode.EAP_CODE_FAILURE)
					eapReject ++;
			}
			cicleFlag = false;

			if(recvEAP instanceof EAPpldBasic) {
				if(((EAPpldBasic) recvEAP).type == testType) {
					EAPtypePldPEAP curpld = (EAPtypePldPEAP)((EAPpldBasic)recvEAP).EAPmethodPld;
					if(curpld.getEapTlsLBit()) {
						if(tlsFullMsgLen == 0)
							tlsFullMsgLen = curpld.tlsMessageLenght;
						else {
							DebugInfo.printMessage(1,"!!! Warning! tlsMessageLenght dublicate");
						}
					}
					tlsMsgList.add(curpld.tlsData);
					if(curpld.getEapTlsMBit()) {
						cicleFlag = true;
						ttlsEapACKpkt.identifier = sock.getReceivedEAPPacket().identifier;
						ttlsEapACKpkt.EAPmethodPld = new EAPtypePldPEAP();
						ttlsEapACKpkt.isLengthSet = false;
						ttlsEapACKpkt.toByteArray();
						sock.setEapMessage(ttlsEapACKpkt);
					} else if((curpld.tlsData == null) || (curpld.tlsData.length == 0)) {
						cicleFlag = true;
						ttlsEapACKpkt.identifier = sock.getReceivedEAPPacket().identifier;
						ttlsEapACKpkt.EAPmethodPld = new EAPtypePldPEAP();
						ttlsEapACKpkt.isLengthSet = false;
						if(counterEmptyRecv > 0)
							((EAPtypePldPEAP)(ttlsEapACKpkt.EAPmethodPld)).setEapTlsFlag(true,false,false);
						ttlsEapACKpkt.toByteArray();
						sock.setEapMessage(ttlsEapACKpkt);
						counterEmptyRecv++;
					}
				}
			}
		}
		return res;
	}

	
	int parseTLSresp(byte[] tlsmsg) {
		DebugInfo.printMessageWithSeparator(1,"parse tls response");
		int res = 0;
		
		TLSResponse resp = new TLSResponse(tlsmsg);
   		resp.printResponse();
    	if(resp.err != 0) {
			DebugInfo.printMessage(1,"!!! TLSResponse return error");
			res = 1;
    	} else {
    		try {
    			ArrayList<RecordLayer> tlsResp = tlsClnt.parseTlsResponse(resp);	
    			if(null == tlsResp  ||  tlsClnt.conSpec.disconnectWithSendAlert != -1) {
    				DebugInfo.printMessage(1,"!!! TLSResponse return error");
    				tlsClnt.conSpec.disconnect();
    				res = 2;
    			}
    		} catch (Exception err) {
    			DebugInfo.printMessage(1,"===Error! parseTlsResponse Exception: " + err);
    			res = 3;
    		}
    	}
    	return res;
	}

	int recvAndParseEAPmsg(EAPpldBasic ttlsEapPkt) {
		int res = 0;
		
		sock.setEapMessage(ttlsEapPkt);

		if(0 != recvFullEAPmsg()) return 1;

		byte[] tlsRaw = EAPTlsFragmentation.combineTlsMessages(tlsMsgList);
		if(tlsRaw == null) {
			DebugInfo.printMessage(1,"!!! Error! tlsRaw Null pointer");
		} else if(tlsRaw.length != tlsFullMsgLen) {
			DebugInfo.printMessage(1,"!!! Error! tlsRaw length failed: " + tlsRaw.length + " must " + tlsFullMsgLen);
		} else {
			DebugInfo.printDumpWithLength(2, tlsRaw, "Get full tls messages");
		}

		// parse tls response		
		DebugInfo.printMessageWithSeparator(1,"parse tls response");

   		try {
   			tlsResp = tlsClnt.parseTlsResponse(tlsRaw);	
   			if(null == tlsResp  ||  tlsClnt.conSpec.disconnectWithSendAlert != -1) {
   				DebugInfo.printMessage(1,"!!! TLSResponse return error");
   				tlsClnt.conSpec.disconnect();
   				res = 2;
   			}
   		} catch (Exception err) {
   			DebugInfo.printMessage(1,"===Error! parseTlsResponse Exception: " + err);
   			res = 3;
   		}

		return res;
	}

}




class TestPeapTLSSequence2
{
	byte[] testRandomData = {1,2,3,4,5,6,7,8};
	byte[] testRandomMD5Data = {16,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16};
	byte[] testRandomMSCHAPData = null;
	byte[] testRandomExpandedData = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
	byte[] testRandomSimData = {10,0,0,7,5,0,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,16,1,0,1};
	byte[] testRandomTLSData = {0,22,3,3,0,57,1,0,0,53,3,3,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,0,0,4,0,47,0,61,1,0,0,8,0,13,0,4,0,2,2,1};


	String userName = null;
	
	TestIterator itrInnerEapType;
	TestIterator itrInnerEapData;
	TestIteratorComlex itrMain;
	boolean endFlag;

	TestPeapTLSSequence2(String userName) {
		this.userName = userName;
		itrInnerEapType = new TestIterator();
		itrInnerEapData = new TestIterator();
		TestIterator[] itrArray = {itrInnerEapType,itrInnerEapData};
		itrMain = new TestIteratorComlex(itrArray);
		endFlag = false;
		
		EAPtypePldMSCHAP msPld = new EAPtypePldMSCHAP();
		msPld.setRandomNTvalue();
		testRandomMSCHAPData = msPld.value;
	}
	
    public boolean isEnd() {
    	return this.endFlag;
    }

    public void printIteratorIndexes() {
		DebugInfo.printMessage(1,"itrInnerEapType itrInnerEapData " + itrInnerEapType.index + " "  + itrInnerEapData.index);
    }
    
    public void clear() {
		itrInnerEapType.clear();
		itrInnerEapData.clear();
		itrMain.clear();
    }

	public byte[] setTestPld() {
		DebugInfo.printMessage(1,"setTestPld");
		EAPtype curTp;
		byte[] dt;
		byte[] pld = null;

		printIteratorIndexes();
		curTp = getInnerEapType(itrInnerEapType);
		dt = getInnerEapData(itrInnerEapData,curTp);
		if(!itrMain.next()) this.endFlag = true;

		if(dt != null) {
			pld = new byte[1 + dt.length];
			System.arraycopy(dt, 0, pld, 1, dt.length);
		}
		else {
			pld = new byte[1];
		}
		pld[0] = (byte)curTp.getValue();
		
		DebugInfo.printMessage(1,"setTestPld return");
		return pld;
	}
	
	EAPtype getInnerEapType(TestIterator itr) {
		switch (itr.index) {
		case 1: return EAPtype.EAP_TYPE_UNKNOWN;
		case 2: return EAPtype.EAP_TYPE_NOTIFICATION;
		case 3:	return EAPtype.EAP_TYPE_NAK;
		case 4:	return EAPtype.EAP_TYPE_TEST;
		case 5:	return EAPtype.EAP_TYPE_EXPANDED_TYPES;
		case 6:	return EAPtype.EAP_TYPE_EXPERIMENTAL;
		case 7:	return EAPtype.EAP_TYPE_MD5CHALLENGE;
		case 8:	return EAPtype.EAP_TYPE_MSCHAP;
		case 9:	return EAPtype.EAP_TYPE_SIM;
		case 10: return EAPtype.EAP_TYPE_TLS;
		case 11: return EAPtype.EAP_TYPE_TTLSv0;
		case 12: return EAPtype.EAP_TYPE_PEAP;
		case 13: itr.end(); return EAPtype.EAP_TYPE_IDENTITY;
		default: itr.end(); return EAPtype.EAP_TYPE_UNKNOWN;
		}
	}

	
	byte[] getInnerEapData(TestIterator itr, EAPtype tp) {
		byte[] pldData = null;
		
		switch (itr.index) {
		case 1:	break;
		case 2:	pldData = testRandomData; break;
		case 3:	
			switch (tp) {
			case EAP_TYPE_IDENTITY: pldData = userName.getBytes(); break;
			case EAP_TYPE_NAK: pldData = new byte[1]; pldData[0] = 0; break;
			case EAP_TYPE_EXPANDED_TYPES: pldData = testRandomExpandedData;  break;
			case EAP_TYPE_MD5CHALLENGE:	pldData = testRandomMD5Data;  break;
			case EAP_TYPE_MSCHAP:	pldData = testRandomMSCHAPData;  break;
			case EAP_TYPE_SIM:	pldData = testRandomSimData;  break;
			case EAP_TYPE_TLS:
			case EAP_TYPE_PEAP:	
			case EAP_TYPE_TTLSv0:	pldData = testRandomTLSData;;  break;
			}
			itr.end(); break;
		default: itr.end(); 
		}
		return pldData;
	}
	
}
