package ru.ispras.telecomtesk.eap.EAPSIM.Attributes;

import ru.ispras.telecomtesk.eap.ConvertTypes;
import ru.ispras.telecomtesk.eap.DebugInfo;
import ru.ispras.telecomtesk.eap.EAPSIM.EAPsimAttribute;
import ru.ispras.telecomtesk.eap.EAPSIM.EAPsimAttrType;


public class EAPsimAttrClientErrorCode extends EAPsimAttribute {
    public short errcode = 1; // default value
	protected boolean isReservedField = false;

	public EAPsimAttrClientErrorCode(short err) {
		this.attrType = EAPsimAttrType.EAPSIM_AT_CLIENT_ERROR_CODE.getValue();
		this.errcode = err;
		this.value = ConvertTypes.hton(err);
		isValueSet = true;
		setLength(1);
	}

	public EAPsimAttrClientErrorCode(int err) {
		this((short)1);
	}

	// parse message constructor 
	public EAPsimAttrClientErrorCode(byte[] data, int len, int pos) {
		this.attrType = EAPsimAttrType.EAPSIM_AT_CLIENT_ERROR_CODE.getValue();
		parseData(data, len, pos);
		if(this.value != null) {
			if(this.value.length == 2) {
				this.errcode = ConvertTypes.ntohs(this.value,0);
			} else {
				DebugInfo.printMessage(1,"===Error! EAPsimAttrClientErrorCode: bad code length");
				DebugInfo.printDump(1,this.value, "  counter:");
			}
		} else
			DebugInfo.printMessage(1,"===Error! EAPsimAttrClientErrorCode: code value not found");
	}

	public String getCodeReview(int code) {
		String rev = "" + code + ": ";
		switch(code) {
		case 0: rev += "Unable to process packet"; break; 
		case 1: rev += "Unsupported version"; break; 
		case 2: rev += "Insufficient number of challenges"; break; 
		case 3: rev += "RANDs are not fresh"; break; 
		default: rev += "undefined error";
		}
		return rev;
	}

	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "  attrType: " + this.attrType);
		DebugInfo.printMessage(1,pref + "  length: " + this.length);
		DebugInfo.printMessage(1,pref + "  Error code " + getCodeReview(this.errcode));
	}
}    
