package ru.ispras.telecomtesk.eap.EAPSIM.Attributes;

import ru.ispras.telecomtesk.eap.ConvertTypes;
import ru.ispras.telecomtesk.eap.DebugInfo;
import ru.ispras.telecomtesk.eap.EAPSIM.EAPsimAttribute;
import ru.ispras.telecomtesk.eap.EAPSIM.EAPsimAttrType;


public class EAPsimAttrNotification extends EAPsimAttribute {

    public short notifCode = (short)32800; // default value
	protected boolean isReservedField = false;

	public EAPsimAttrNotification() {
		this.attrType = EAPsimAttrType.EAPSIM_AT_NOTIFICATION.getValue();
		this.value = ConvertTypes.hton(notifCode);
		isValueSet = true;
		setLength(1);
	}

	public EAPsimAttrNotification(short code) {
		this.attrType = EAPsimAttrType.EAPSIM_AT_NOTIFICATION.getValue();
		this.notifCode = code;
		this.value = ConvertTypes.hton(code);
		isValueSet = true;
		setLength(1);
	}

	public EAPsimAttrNotification(int code) {
		this((short)code);
	}

	// parse message constructor 
	public EAPsimAttrNotification(byte[] data, int len, int pos) {
		this.attrType = EAPsimAttrType.EAPSIM_AT_NOTIFICATION.getValue();
		parseData(data, len, pos);
		if(this.value != null) {
			if(this.value.length == 2) {
				this.notifCode = ConvertTypes.ntohs(this.value,0);
			} else {
				DebugInfo.printMessage(1,"===Error! EAPsimAttrNotification: bad data length");
				DebugInfo.printDump(1,this.value, "  code:");
			}
		} else
			DebugInfo.printMessage(1,"===Error! EAPsimAttrNotification: code value not found");
	}

	public byte getSbit() { // Success bit (S bit)
		if((0x8000 & this.notifCode) != 0) // 0x8000 = 32768
			return 1;
		else
			return 0;
	}

	public byte getPbit() { // Phase bit (P bit)
		if((0x4000 & this.notifCode) != 0) // 0x4000 = 16384
			return 1;
		else
			return 0;
	}

	public String Sbit2Str() { // Phase bit (P bit)
		if(this.getSbit() > 0)
			return "1 - Successful cases";
		else
			return "0 - Unsuccessful cases";
	}

	public String Pbit2Str() { // Phase bit (P bit)
		if(this.getPbit() > 0)
			return "1 - Before Challenge/Re-authentication round";
		else
			return "0 - After Challenge/Re-authentication round";
	}

	// if the P bit is set to one, then the S bit MUST be set to zero.
	public boolean checkSPbits() { // Phase bit (P bit)
		return !((this.getPbit() > 0) && (this.getSbit() > 0));
	}

	public String getCodeReview(int code) {
		String rev = "" + code + ": ";
		switch(code) {
		case 0: rev += "General failure after authentication"; break; 
		// 1026 Implies failure, used after successful authentication
		case 1026: rev += "User has been temporarily denied access to the requested service"; break; 
		// 1031 Implies failure, used after successful authentication.
		case 1031: rev += "User has not subscribed to the requested service"; break; 
		case 16384: rev += "General failure"; break; 
		case 32768: rev += "Success"; break;
		default: rev += "undefined code";
		}
		return rev;
	}

	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "  attrType: " + this.attrType);
		DebugInfo.printMessage(1,pref + "  length: " + this.length);
		DebugInfo.printMessage(1,pref + "  S bit: " + this.Sbit2Str());
		DebugInfo.printMessage(1,pref + "  P bit: " + this.Pbit2Str());
		DebugInfo.printMessage(1,pref + "  Notification code " + getCodeReview(this.notifCode));
	}
}    
