package ru.ispras.telecomtesk.eap.EAPSIM.Attributes;

import ru.ispras.telecomtesk.eap.DebugInfo;
import java.util.Random;

import ru.ispras.telecomtesk.eap.EAPSIM.EAPsimAttribute;
import ru.ispras.telecomtesk.eap.EAPSIM.EAPsimAttrType;

/*
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|     Type      |    Length     |           Reserved            |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|                                                               |
|                           n * 16 Bytes                        |
|                                                               |
|                                                               |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
*/

public class EAPsimAttrRand extends EAPsimAttribute {
//	public byte[] value;  Used for the field "Reserved"
	public byte[][] attrData = null;  // n * 16 bytes

	public EAPsimAttrRand() {
		this.attrType = EAPsimAttrType.EAPSIM_AT_RAND.getValue();
		setValue();
		attrData = new byte[3][16];
		// set random field
		Random rndObj = new Random(System.currentTimeMillis());
		rndObj.nextBytes(attrData[0]);
		rndObj.nextBytes(attrData[1]);
		rndObj.nextBytes(attrData[2]);
	}
	
	public EAPsimAttrRand(int lenData) {
		this.attrType = EAPsimAttrType.EAPSIM_AT_RAND.getValue();
		setValue();
        int ind = 0, lastLen = 0, i = 0;
        if(lenData > 0) {
           ind = lenData / 16;
           lastLen = lenData % 16;
           if(lastLen > 0) ind ++;
           else lastLen = 16;
        }
		attrData = new byte[ind][];
        if(ind > 0) {
		  // set random fields
		  Random rndObj = new Random(System.currentTimeMillis());

          for(i=0; i< attrData.length - 1; i++) {
            attrData[i] = new byte[16];
		    rndObj.nextBytes(attrData[i]);
          }

          attrData[i] = new byte[lastLen];   //  last Rand - 16 bytes or less
		  rndObj.nextBytes(attrData[i]);
        }
	}
	
	public EAPsimAttrRand(byte[] data) {
		this.attrType = EAPsimAttrType.EAPSIM_AT_RAND.getValue();
		setValue();
		if(data != null) {
                  cutRANDSfromArray(data, data.length, 0);
                }
	}
	
	// parse message constructor 
	public EAPsimAttrRand(byte[] data, int len, int pos) {
		this.attrType = EAPsimAttrType.EAPSIM_AT_RAND.getValue();
		parseData(data, len, pos);
	}
	
	public void parseData(byte[] data, int len, int pos) {
		if((data != null)&&(len >= 2)&&(pos > 0)) {
			if(len <= data.length - pos) {
				setValue(data, 2, pos);
				pos += 2;
				len -= 2;
                                cutRANDSfromArray(data,len,pos);
				setLength();
			} else {
				DebugInfo.printMessage(1,"===Error! EAPsimAttrRand::parseData: input array too short");
			}
		} else {
			DebugInfo.printMessage(1,"===Error! EAPsimAttrRand::parseData: bad input value");
		}
	}

	
	public void cutRANDSfromArray(byte[] data, int datalen, int startpos) {
		if((data != null)&&(datalen > 0)&&(startpos > 0)&&(datalen <= data.length - startpos)) {
                  int ind = 0, pos = 0, curlen = 0;
                  ind = datalen / 16;
                  if(datalen % 16 > 0) ind ++;

		  attrData = new byte[ind][];
                  if(ind > 0) {
                    for(ind = 0; pos < datalen; ind++) {
                      curlen = (datalen - pos >= 16) ? 16 : datalen - pos;
                      attrData[ind] = new byte[curlen];
                      System.arraycopy(data, pos + startpos, attrData[ind], 0, curlen);

		      if(curlen != 16)
			DebugInfo.printMessage(1,"===Error! EAPsimAttrRand::cutRANDSfromArray RAND[" + ind + "] bad length " + curlen);

                      pos += curlen;
                    }
                  }
		} else {
			DebugInfo.printMessage(1,"===Error! EAPsimAttrRand::cutRANDSfromArray bad input data");
		}
	}

	
	public int getSize() {
		int res = 2;
		if(value != null) 
			res += value.length;
		if(attrData != null) {
                        for(int i=0; i<attrData.length; i++) {
				if(attrData[i] != null) 
					res += attrData[i].length;
                        }
                }
		return res;
	}


	public int toByteArray() {
		int i = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		ba[i++] = (byte)attrType;
		ba[i++] = (byte)length;
		if(value != null) {
			System.arraycopy(value, 0, ba, i, value.length);
			i += value.length;
		}
		if(attrData != null) {
                        for(int j=0; j<attrData.length; j++) {
				if(attrData[j] != null) 
					System.arraycopy(attrData[j], 0, ba, i, attrData[j].length);
					i += attrData[j].length;
                        }
		}
		net_msg = ba;
		return 0;
	}

	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "  attrType: " + EAPsimAttrType.getString(this.attrType));
		DebugInfo.printMessage(1,pref + "  length: " + this.length + " // multiples 4 bytes");
		DebugInfo.printDump(1, this.value, pref + "  reserved:");
		if(attrData != null) {
                        for(int i=0; i<attrData.length; i++) {
				if(attrData[i] != null) 
					DebugInfo.printDump(1, this.attrData[i], pref + "  RAND " + i + ":");
                        }
                }
	}
}    
