package ru.ispras.telecomtesk.eap.EAPSIM.Attributes;

import ru.ispras.telecomtesk.eap.DebugInfo;
import ru.ispras.telecomtesk.eap.ConvertTypes;
import ru.ispras.telecomtesk.eap.EAPSIM.EAPsimAttribute;
import ru.ispras.telecomtesk.eap.EAPSIM.EAPsimAttrType;


public class EAPsimAttrVersionList extends EAPsimAttribute {
    public int verListLength;
    public short[] versionList = null;
	public byte[] padding = null; 
	protected boolean isverListLengthSet = false;
	protected boolean isPaddingSet = false;

	public EAPsimAttrVersionList(short[] verList) {
		this.attrType = EAPsimAttrType.EAPSIM_AT_VERSION_LIST.getValue();
		if(verList != null) {
			verListLength = 2 * verList.length;
			isverListLengthSet = true;
			versionList = new short[verList.length];
			System.arraycopy(verList, 0, versionList, 0, verList.length);
			setpadding();
			setLength();
		} else {
			DebugInfo.printMessage(1,"===Error! EAPsimAttrVersionList: verList null pointer");
		}
	}
	
	// parse message constructor 
	public EAPsimAttrVersionList(byte[] data, int len, int pos) {
		this.attrType = EAPsimAttrType.EAPSIM_AT_VERSION_LIST.getValue();
		parseData(data, len, pos);
	}
	
	public void parseData(byte[] data, int len, int pos) {
		if((data != null)&&(len >= 2)&&(pos > 0)) {
			if(len <= data.length - pos) {
				verListLength = ConvertTypes.ntohs(data,pos);
				isverListLengthSet = true;
				pos += 2;
				len -= 2;
				int curlen = verListLength;
				if(verListLength > len) {
					curlen = len;
	    			DebugInfo.printMessage(1,"===Error! EAPsimAttrVersionList::parseData: verListLength out of array: " + verListLength + " > " + len);
				}
				versionList = new short[curlen/2];
	    		if(curlen % 2 != 0) {
	    			DebugInfo.printMessage(1,"===Error! EAPsimAttrVersionList::parseData: versionList length must be even");
	    		}
	    		for(int i=0; i<versionList.length; i++) {
	    			versionList[i] = ConvertTypes.ntohs(data,pos);
	    			pos += 2;
	    		}
	    		padding = new byte[len - 2 * versionList.length];
				System.arraycopy(data, pos, padding, 0, padding.length);
				isPaddingSet = true;
				setLength();
			} else {
				DebugInfo.printMessage(1,"===Error! EAPsimAttrVersionList::parseData: input array too short");
			}
		} else {
			DebugInfo.printMessage(1,"===Error! EAPsimAttrVersionList::parseData: bad input value");
		}
	}

	public int getSize() {
			int res = 4;
			if(versionList != null) 
				res += 2 * versionList.length;
			if(padding != null) 
				res += padding.length;
			return res;
	}

	public void setpadding() {
		int i = (2 * versionList.length) % 4;
		if(i > 0) 
			i = 4 - i;
		padding = new byte[i];
		for(int j = 0; j<i; j++)
			padding[j] = 0;
		
		isPaddingSet = true;
	}

	public int toByteArray() {
		int i = 0;
		if(isPaddingSet == false) setpadding();
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		ba[i++] = (byte)attrType;
		ba[i++] = (byte)length;
		// write versionList
		if(isverListLengthSet == false) 
			verListLength = (versionList != null) ? (2 * versionList.length) : 0;
		
		ba[i++] = (byte)(verListLength >>> 8);
		ba[i++] = (byte)verListLength;
		
		if(versionList != null) {
			for (int j = 0; j < versionList.length; j++) {
				ba[i++] = (byte)(versionList[j] >>> 8);
				ba[i++] = (byte)versionList[j];
			}
		}
		if(padding != null) {
			System.arraycopy(padding, 0, ba, i, padding.length);
		}
		net_msg = ba;
		return 0;
	}

	public byte[] versionListToByteArray() {
		int i = 0;
		byte[] vl = null;
		if(versionList != null) {
			vl = new byte[2 * versionList.length];
			for (int j = 0; j < versionList.length; j++) {
				vl[i++] = (byte)(versionList[j] >>> 8);
				vl[i++] = (byte)versionList[j];
			}
		}
		return vl;
	}

	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "  attrType: " + EAPsimAttrType.getString(this.attrType));
		DebugInfo.printMessage(1,pref + "  length: " + this.length + " // multiples 4 bytes");
		DebugInfo.printMessage(1,pref + "  version list length: " + this.verListLength);
		String vl = "  version list:";
		for(int i=0; i<versionList.length; i++) {
			vl += " " + versionList[i] + ";";
		}
		DebugInfo.printMessage(1,pref + vl);
		DebugInfo.printDump(1, this.padding, pref + "  padding:");
	}
}    
