package ru.ispras.telecomtesk.eap.EAPSIM;

import java.security.MessageDigest;
import javax.crypto.*;
import javax.crypto.spec.SecretKeySpec;
import ru.ispras.telecomtesk.eap.BasicPayloadObject;
import ru.ispras.telecomtesk.eap.DebugInfo;


public class EAPsimAttribute extends BasicPayloadObject {
	public int attrType;
    public int length;         
	public byte[] value = null; 
	protected boolean isLengthSet = false;
	protected boolean isValueSet = false;
	protected boolean isReservedField = true;

	public EAPsimAttribute() {
		this.attrType = EAPsimAttrType.EAPSIM_AT_UNKNOWN.getValue();
	}
	
	public EAPsimAttribute(int attr) {
		this.attrType = attr;
	}
	
	public EAPsimAttribute(EAPsimAttrType attr) {
		this.attrType = attr.getValue();
	}
	
	public EAPsimAttribute(int attr, byte[] data) {
		this.attrType = attr;
		parseData(data, data.length, 0);
	}
	
	public EAPsimAttribute(int attr, byte[] data, int len) {
		this.attrType = attr;
		parseData(data, len, 0);
	}
	
	public EAPsimAttribute(byte[] data, int attr, int len, int pos) {
		this.attrType = attr;
		parseData(data, len, pos);
	}
	
	public void parseData(byte[] data, int len, int pos) {
		if((data != null)&&(len >= 0)&&(pos >= 0)) {
			int curlen = 0;
			if(pos <= data.length - 1) {
				curlen = data.length >= len + pos ? len : data.length - pos;
				setValue(data, curlen, pos);
			}
			setLength();
		}
	}
	
	public static EAPsimAttribute cloneAttribute(EAPsimAttribute attr) {
		EAPsimAttribute newAttr = null;
		if(attr != null)
			newAttr = new EAPsimAttribute(attr.attrType, attr.value);
		return newAttr;
	}
	
//	length of an attribute multiples of four
    public int getSize() {
		if(isValueSet == false) setValue();
		if(value != null) {
			int res = 2 + value.length;
			if(res % 4 > 0)
				res += 4 - res % 4;
			return res;
		}
		else
			return 4;
	}

	public void setLength() {
		if(getSize() % 4 != 0)
			DebugInfo.printMessage(1,"===Warning! EAPsimAttribute::setLength: bad value " + getSize() + ", " + EAPsimAttrType.valueOf(attrType));
		length = (byte)getSize() / 4;
		isLengthSet = true;
	}

	public void setLength(int len) {
		length = (byte)len;
		isLengthSet = true;
	}


	public void setValue() {
		value = new byte[2];
		value[0] = 0;
		value[1] = 0;
		isValueSet = true;
	}


	public void setValue(byte[] data, int length, int pos) {
		value = new byte[length];
		System.arraycopy(data, pos, value, 0, length);
		isValueSet = true;
		if(isReservedField && ((value[0] != 0) || (value[1] != 0)) )
			DebugInfo.printMessage(1,"===Warning! EAPsimAttribute::setValue: wrong Reserved bytes: " + value[0] + " " + value[1]);
	}


	public int toByteArray() {
		int i = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		ba[i++] = (byte)attrType;
		ba[i++] = (byte)length;
		if(value != null) {
			System.arraycopy(value, 0, ba, i, value.length);
			i += value.length;
		}

		while(i < ba.length) ba[i++] = 0; // set padding to Zero bytes

		net_msg = ba;
		return 0;
	}

	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "  attrType: " + EAPsimAttrType.getString(this.attrType));
		DebugInfo.printMessage(1,pref + "  length: " + this.length + " // multiples 4 bytes");
		DebugInfo.printDump(1, this.value, pref + "  value:");
	}
}    
