package ru.ispras.telecomtesk.eap.EAPSIM;

import ru.ispras.telecomtesk.eap.ConvertTypes;
import ru.ispras.telecomtesk.eap.DebugInfo;

/*
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|     Type      |    Length     |      Actual Value Length      |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
.                             Value                             .
.                            Padding                            .
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
*/

public class EAPsimAttribute2 extends EAPsimAttribute {
    public int valueLength;
//	public byte[] value;  Used for the field "Actual Value Length"
	public byte[] padding = null; 
	protected boolean isValueLengthSet = false;
	protected boolean isPaddingSet = false;

	public EAPsimAttribute2() {
		this.attrType = EAPsimAttrType.EAPSIM_AT_UNKNOWN.getValue();
	}
	
	public EAPsimAttribute2(byte[] data) {
		initValue(data);
	}
	
	// parse message constructor 
	public EAPsimAttribute2(byte[] data, int len, int pos) {
		parseData(data, len, pos);
	}

	public void initValue(byte[] data) {
		if(data != null) {
			valueLength = data.length;
			isValueLengthSet = true;
			value = new byte[data.length];
			System.arraycopy(data, 0, value, 0, data.length);
			setpadding();
			setLength();
		} else {
			DebugInfo.printMessage(1,"===Error! EAPsimAttribute2::initValue: data null pointer");
		}
	}
	
	public void parseData(byte[] data, int len, int pos) {
		if((data != null)&&(len >= 2)&&(pos > 0)) {
			if(len <= data.length - pos) {
				valueLength = ConvertTypes.ntohs(data,pos);
				isValueLengthSet = true;
				pos += 2;
				len -= 2;
				int curlen = valueLength;
				if(valueLength > len) {
					curlen = len;
	    			DebugInfo.printMessage(1,"===Error! EAPsimAttribute2::parseData: valueLength out of array: " + valueLength + " > " + len);
				}
				value = new byte[curlen];
				System.arraycopy(data, pos, value, 0, value.length);
				pos += value.length;
	    		padding = new byte[len - value.length];
				System.arraycopy(data, pos, padding, 0, padding.length);
				isPaddingSet = true;
				setLength();
			} else {
				DebugInfo.printMessage(1,"===Error! EAPsimAttribute2::parseData: input array too short");
			}
		} else {
			DebugInfo.printMessage(1,"===Error! EAPsimAttribute2::parseData: bad input value");
		}
	}

	public int getSize() {
		int res = 4;
		if(value != null) 
			res += value.length;
		if(padding != null) 
			res += padding.length;
		return res;
	}

	public void setpadding() {
		int i = value.length % 4;
		if(i > 0) 
			i = 4 - i;
		padding = new byte[i];
		for(int j = 0; j<i; j++)
			padding[j] = 0;
		
		isPaddingSet = true;
	}

	public int toByteArray() {
		int i = 0;
		if(isPaddingSet == false) setpadding();
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		ba[i++] = (byte)attrType;
		ba[i++] = (byte)length;
		if(isValueLengthSet == false) 
			valueLength = (value != null) ? value.length : 0;
		
		ba[i++] = (byte)(valueLength >>> 8);
		ba[i++] = (byte)valueLength;
		
		if(value != null) {
			System.arraycopy(value, 0, ba, i, value.length);
			i += value.length;
		}
		if(padding != null) {
			System.arraycopy(padding, 0, ba, i, padding.length);
		}
		net_msg = ba;
		return 0;
	}

	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "  attrType: " + EAPsimAttrType.getString(this.attrType));
		DebugInfo.printMessage(1,pref + "  length: " + this.length + " // multiples 4 bytes");
		DebugInfo.printMessage(1,pref + "  data length: " + this.valueLength);
		DebugInfo.printDump(1, this.value, pref + "  Data:");
		DebugInfo.printDump(1, this.padding, pref + "  padding:");
	}
}    
