package ru.ispras.telecomtesk.eap.EAPSIM;

import ru.ispras.telecomtesk.eap.DebugInfo;
import java.util.Random;

/*
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|     Type      | Length = 5    |           Reserved            |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|                                                               |
|                              Data                             |
|                                                               |
|                                                               |
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
*/

public class EAPsimAttribute3 extends EAPsimAttribute {
//	public byte[] value;  Used for the field "Reserved"
	public byte[] attrData = null;  // 16 bytes

	public EAPsimAttribute3() {
		setValue();
		attrData = new byte[16];
		// set random field
		Random rndObj = new Random(System.currentTimeMillis());
		rndObj.nextBytes(attrData);
	}
	
	public EAPsimAttribute3(int lenData) {
		setValue();
		attrData = new byte[lenData];
		// set random field
		Random rndObj = new Random(System.currentTimeMillis());
		rndObj.nextBytes(attrData);
	}
	
	public EAPsimAttribute3(byte[] data) {
		setValue();
		if(data != null) {
			this.attrData = new byte[data.length];
			System.arraycopy(data, 0, this.attrData, 0, data.length);
			if(data.length != 16)
				DebugInfo.printMessage(1,"===Error! EAPsimAttribute3: data bad length " + data.length);
			setLength();
		} else {
			DebugInfo.printMessage(1,"===Error! EAPsimAttribute3: data null pointer");
		}
	}
	
	// parse message constructor 
	public EAPsimAttribute3(byte[] data, int len, int pos) {
		parseData(data, len, pos);
	}
	
	public void parseData(byte[] data, int len, int pos) {
		if((data != null)&&(len >= 2)&&(pos > 0)) {
			if(len <= data.length - pos) {
				setValue(data, 2, pos);
				pos += 2;
				len -= 2;
	    		if(len != 16) {
	    			DebugInfo.printMessage(1,"===Error! EAPsimAttribute3::parseData: data too short " + len);
	    		}
	    		this.attrData = new byte[len];
				System.arraycopy(data, pos, this.attrData, 0, len);
				setLength();
			} else {
				DebugInfo.printMessage(1,"===Error! EAPsimAttribute3::parseData: input array too short");
			}
		} else {
			DebugInfo.printMessage(1,"===Error! EAPsimAttribute3::parseData: bad input value");
		}
	}

	
	public int getSize() {
		int res = 2;
		if(value != null) 
			res += value.length;
		if(attrData != null) 
			res += attrData.length;
		return res;
	}


	public int toByteArray() {
		int i = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		ba[i++] = (byte)attrType;
		ba[i++] = (byte)length;
		if(value != null) {
			System.arraycopy(value, 0, ba, i, value.length);
			i += value.length;
		}
		if(attrData != null) {
			System.arraycopy(attrData, 0, ba, i, attrData.length);
		}
		net_msg = ba;
		return 0;
	}

	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "  attrType: " + EAPsimAttrType.getString(this.attrType));
		DebugInfo.printMessage(1,pref + "  length: " + this.length + " // multiples 4 bytes");
		DebugInfo.printDump(1, this.value, pref + "  reserved:");
		DebugInfo.printDump(1, this.attrData, pref + "  Data:");
	}
}    
