package ru.ispras.telecomtesk.eap.EAPSIM;


import java.math.BigInteger;
import java.security.MessageDigest;
import ru.ispras.telecomtesk.eap.DebugInfo;
import ru.ispras.telecomtesk.eap.ConvertTypes;
import java.security.NoSuchAlgorithmException;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.xml.bind.DatatypeConverter;

public class EAPsimSecurity {    

    static String tString = "67452301EFCDAB8998BADCFE10325476C3D2E1F0";
    static byte[] tArray = DatatypeConverter.parseHexBinary(tString);

    public static byte[] createMasterKey(byte[] identity, byte[] nKc, byte[] nonceMT, byte[] versionList, byte[] selectedVersion) {
		MessageDigest mobj;
		byte[] data = null;
		byte[] mk = null;
		int pos = 0;

		if((identity != null) && (nKc != null) && (nonceMT != null) && (versionList != null) && (selectedVersion != null)) {
			data = new byte[identity.length + nKc.length + nonceMT.length + versionList.length + selectedVersion.length];

			System.arraycopy(identity, 0, data, pos, identity.length);
			pos += identity.length;
			System.arraycopy(nKc, 0, data, pos, nKc.length);
			pos += nKc.length;
			System.arraycopy(nonceMT, 0, data, pos, nonceMT.length);
			pos += nonceMT.length;
			System.arraycopy(versionList, 0, data, pos, versionList.length);
			pos += versionList.length;
			System.arraycopy(selectedVersion, 0, data, pos, selectedVersion.length);

			DebugInfo.printDump(3, data, "EAPsimSecurity::createMasterKey: input data");

			try {
			mobj = MessageDigest.getInstance("SHA1");
			mk = mobj.digest(data);
			DebugInfo.printDump(3, mk, "EAPsimSecurity::createMasterKey: ");
			} catch (NoSuchAlgorithmException e) {
				DebugInfo.printMessage(1, e.toString());
			}

		} else {
			DebugInfo.printMessage(2,"EAPsimSecurity::createMasterKey: null pointer");
		}

		DebugInfo.printDump(3, mk, "EAPsimSecurity::createMasterKey: result key:");

		return mk;
	}

	public static byte[] createNewXKey(byte[] identity, byte[] counter, byte[] nonceS, byte[] mk) {
		MessageDigest mobj;
		byte[] data = null;
		byte[] key = null;
		int pos = 0;

		if((identity != null) && (counter != null) && (nonceS != null) && (mk != null)) {
			data = new byte[identity.length + counter.length + nonceS.length + mk.length];

			System.arraycopy(identity, 0, data, pos, identity.length);
			pos += identity.length;
			System.arraycopy(counter, 0, data, pos, counter.length);
			pos += counter.length;
			System.arraycopy(nonceS, 0, data, pos, nonceS.length);
			pos += nonceS.length;
			System.arraycopy(mk, 0, data, pos, mk.length);

			DebugInfo.printDump(3, data, "EAPsimSecurity::createNewXKey: input data");

			try {
			mobj = MessageDigest.getInstance("SHA1");
			key = mobj.digest(data);
			DebugInfo.printDump(3, key, "EAPsimSecurity::createNewXKey: ");
			} catch (NoSuchAlgorithmException e) {
				DebugInfo.printMessage(1, e.toString());
			}

		} else {
			DebugInfo.printMessage(2,"EAPsimSecurity::createNewXKey: null pointer");
		}

		DebugInfo.printDump(3, key, "EAPsimSecurity::createNewXKey: result key:");

		return key;
	}

	public static byte[] eapSimPRF(byte[] MasterKey, int resLen) {
        byte[] res = null;

		if(resLen > 0) {
			byte[] wiByteArray;
			BigInteger xkey, wi, twoN;
            int pos=0, len;

			DebugInfo.printDump(3, MasterKey, "EAPsimSecurity::eapSimPRF input data:");
			res = new byte[resLen];
			twoN = BigInteger.ZERO.setBit(160);
//			BigInteger maxNbits = twoN.subtract(BigInteger.ONE);
			xkey = new BigInteger(MasterKey);

			while(pos < resLen) {
				wiByteArray = eapSimPRNG(xkey.toByteArray());
				if(wiByteArray == null) break;
				wi = new BigInteger(wiByteArray);
				xkey = BigInteger.ONE.add(xkey);
				xkey = xkey.add(wi);
				xkey = xkey.mod(twoN);

				len = (resLen - pos) >= wiByteArray.length ? wiByteArray.length : resLen - pos;
				System.arraycopy(wiByteArray, 0, res, pos, len);
				pos += len;
			}
		}
		DebugInfo.printDump(3, res, "\nEAPsimSecurity::eapSimPRF: output data:");
		return res;
	}

        // see FIPS180-1-SecureHashStandard
	public static byte[] eapSimPRNG(byte[] data) {
				byte[] res;
                int temp, i, sht=0;

            	DebugInfo.printMessage(5,"-----\nEAPsimSecurity::eapSimPRNG");
                if((data == null)||(data.length < 20)) {
        			DebugInfo.printDump(1, data, "!!!Error: EAPsimSecurity::eapSimPRNG: bad input data");
                	return null;
                }
                if(data.length != 20) {
        			DebugInfo.printDump(1, data, "!!!Warning: EAPsimSecurity::eapSimPRNG: bad input data length");
                }
                
    			DebugInfo.printDump(5, tArray, "EAPsimSecurity::eapSimPRNG tArray:");
                if(tArray.length != 20) {
                	DebugInfo.printMessage(1,"!!!Error: EAPsimSecurity::eapSimPRNG: bad tArray length: " + tArray.length);
                	return null;
                }

                byte[] dataArrayWithPad = new byte[64];
                int dataPos = data.length > 20 ? data.length - 20 : 0;
    			System.arraycopy(data, dataPos, dataArrayWithPad, 0, 20);
                for(i=20; i<dataArrayWithPad.length; i++)
                	dataArrayWithPad[i] = 0;

    			DebugInfo.printMessage(5, "start algorithm");
               // A, B, C, D, E
                int[] aArray = new int[5];

                // initialize H0, H1, H2, H3, H4
                int[] hArray = new int[5];
                for(i=0; i<=4; i++)
                	hArray[i] = ConvertTypes.ntoh(tArray,i*4);

                // step a: initialize W0, ..., W15
                int[] wArray = new int[80];
                if(dataArrayWithPad.length == 65) sht++;
                for(i=0; i<=15; i++)
                	wArray[i] = ConvertTypes.ntoh(dataArrayWithPad,sht + i*4);

                // step b
                for(i=16; i<=79; i++)
                	wArray[i] = eapSimPRNG_Sfunc(wArray[i-3] ^ wArray[i-8] ^ wArray[i-14] ^ wArray[i-16], 1);

                // step c
                for(i=0; i<=4; i++)
                	aArray[i] = hArray[i];

                // step d
                for(i=0; i<=79; i++) {
                	temp = eapSimPRNG_Sfunc(aArray[0],5) 
                	+ eapSimPRNG_Ffunc(aArray[1], aArray[2], aArray[3], i)
                	+ aArray[4] + wArray[i] + eapSimPRNG_Kconst(i);
                	aArray[4] = aArray[3];
                	aArray[3] = aArray[2];
                	aArray[2] = eapSimPRNG_Sfunc(aArray[1],30);
					aArray[1] = aArray[0];
					aArray[0] = temp;
                }

                // step e
                for(i=0; i<=4; i++)
                	hArray[i] += aArray[i];

                res = new byte[20];
                for(i=0; i<=4; i++)
                	System.arraycopy(ConvertTypes.hton(hArray[i]), 0, res, i*4, 4);

    			DebugInfo.printDump(5, res, "EAPsimSecurity::eapSimPRNG output:");

                return res;
	}

	public static int eapSimPRNG_Sfunc(int x, int exp) {
                if((exp >= 0)&&(exp <= 32))
			return (int)((x << exp) | (x >>> (32 - exp)));
                else {
			DebugInfo.printMessage(1,"!!!Error: EAPsimSecurity::eapSimPRNG_Sfunc: bad input");
			return 0;
                }
	}

	public static int eapSimPRNG_Ffunc(int b, int c, int d, int ind) {
                if((ind >= 0)&&(ind <= 19))
			return (int)((b & c) | ((~b) & d));
                else if((ind >= 20)&&(ind <= 39))
			return (int)(b ^ c ^ d);
                else if((ind >= 40)&&(ind <= 59))
			return (int)((b & c) |(b & d) | (c & d));
                else if((ind >= 60)&&(ind <= 79))
			return (int)(b ^ c ^ d);
                else {
			DebugInfo.printMessage(1,"!!!Error: EAPsimSecurity::eapSimPRNG_Ffunc: bad input");
			return 0;
                }
	}

	public static int eapSimPRNG_Kconst(int ind) {
                if((ind >= 0)&&(ind <= 19))
			return (int)0x5a827999;
                else if((ind >= 20)&&(ind <= 39))
			return (int)0x6ed9eba1;
                else if((ind >= 40)&&(ind <= 59))
			return (int)0x8f1bbcdc;
                else if((ind >= 60)&&(ind <= 79))
			return (int)0xca62c1d6;
                else {
			DebugInfo.printMessage(1,"!!!Error: EAPsimSecurity::eapSimPRNG_Kconst: bad input");
			return 0;
                }
	}

        // HMAC-SHA1-128 obtained from the 20-byte HMAC-SHA1 first 16 bytes
	public static byte[] mac(byte[] data, byte[] key) {
		byte[] res = null;
		byte[] tmpmac = macSHA1(data, key);
		if((tmpmac != null)&&(tmpmac.length >= 16)) {
			res = new byte[16];
			System.arraycopy(tmpmac, 0, res, 0, res.length);
		} else {
			DebugInfo.printMessage(1,"!!!Error: EAPsimSecurity::mac: null data");
		}
		return res;
	}

    // HMAC-SHA1 20 bytes output
	public static byte[] macSHA1(byte[] data, byte[] key) {
		Mac mobj;
		byte[] res = null;
		SecretKeySpec sk;
		String alg = "HmacSHA1";
		
		DebugInfo.printDump(5, data, "EAPsimSecurity::macSHA1: input data:");
		DebugInfo.printDump(5, key, "key:");

		if((data != null)&&(key != null)) {
			try {
				mobj = Mac.getInstance(alg);
				sk = new SecretKeySpec(key, alg);
				mobj.init(sk);
				res = mobj.doFinal(data);
			} catch (Exception err) {
				DebugInfo.printMessage(1,"!!!Error: EAPsimSecurity::macSHA1: " + err);
			}
		}
		
		return res;
	}

	public static boolean compareMac(byte[] mac1, byte[] mac2) {
		boolean res = false;
		
		DebugInfo.printDump(5, mac1, "EAPsimSecurity::compareMac: mac1:");
		DebugInfo.printDump(5, mac2, "mac2:");
		if((mac1 != null)&&(mac2 != null)&&(mac1.length == mac2.length)) {
			res = true;
			for(int i=0; i < mac1.length; i++) {
				if(mac1[i] != mac2[i]) {
					res = false;
					break;
				}
			}
		}
		
		return res;
	}
}




