package ru.ispras.telecomtesk.eap.EAPSIM;

import ru.ispras.telecomtesk.eap.BasicPayloadObject;
import ru.ispras.telecomtesk.eap.DebugInfo;
import ru.ispras.telecomtesk.eap.EAPSIM.Attributes.*;

import java.util.ArrayList;


public class EAPtypePldSim extends BasicPayloadObject {    
	public EAPSIMsubtype subtype; 
	public int  reserved = 0;
	public int  parsedMACposition = 0;
	public ArrayList<EAPsimAttribute> attrList = null; // EAP SIM Attributes

	public EAPtypePldSim() {
		this.subtype = EAPSIMsubtype.EAPSIM_SUBTYPE_UNKNOWN;
	}

	public EAPtypePldSim(int type) {
		this.subtype = EAPSIMsubtype.valueOf(type);
	}


	public EAPtypePldSim(EAPSIMsubtype type) {
		this.subtype = type;
	}


	public int getSize() {
		int res = 3;

		if(attrList != null) {
			DebugInfo.printMessage(1,"EAPtypePldSim::getSize: " + attrList.size());
			int ext_len = 0;
			for (int j = 0; j < attrList.size(); j++) {
				ext_len += (attrList.get(j)).getSize();
			}
			res += ext_len;
		}
		return res;
	}

	public void addAttribute(EAPsimAttribute attr) {
		if(attrList == null)
			attrList = new ArrayList<EAPsimAttribute>();
		if(attr != null)
			attrList.add(attr);
	}

	public EAPsimAttribute getAttribute(int index) {
		EAPsimAttribute attr = null;
		try {
			attr = attrList.get(index);
		}
		catch (Exception ignored) {}
		
		return attr;
	}
	
	public EAPsimAttribute getAttributeByType(EAPsimAttrType atrType) {
		return getAttributeByType(atrType.getValue());
	}
	public EAPsimAttribute getAttributeByType(int type) {
		EAPsimAttribute attr = null;
		try {
			if(attrList != null)
				for(int i = 0; i<attrList.size(); i++) {
					if(attrList.get(i).attrType == type) {
						attr = attrList.get(i);
						break;
					}
				}
		}
		catch (Exception ignored) {}
		
		return attr;
	}
	
	public void removeAttribute(EAPsimAttribute attr) {
		attrList.remove(attr);
	}
	
	public void clearAttributesList() {
		if(attrList != null)
			attrList.clear();
	}
	
	public int getAttributesListLength() {
		int attr_len = 0;
		for (int j = 0; j < attrList.size(); j++) {
			attr_len += ((EAPsimAttribute)attrList.get(j)).getSize();
		}
		return attr_len;
	}


// a stream of octets consisting of the Identifier, followed by (concatenated with) the "secret", followed by (concatenated with) the Challenge Value (RFC 1994)
/*	public int setChallengeValue(byte id, byte[] chlg_value, byte[] key) throws Exception {
		MessageDigest mobj;
		byte[] data = null;
		int res = 0;
		
		if((chlg_value == null) || (key == null)) {
			res = 1;
			DebugInfo.printMessage(3,"EAPtypePldMd5::setMD5value: null pointer");
		}

		data = new byte[1 + chlg_value.length + key.length];
		data[0] = id;

		System.arraycopy(key, 0, data, 1, key.length);
		System.arraycopy(chlg_value, 0, data, 1 + key.length, chlg_value.length);
		DebugInfo.printDump(3, data, "EAPtypePldMd5::setMD5value: input data");

		mobj = MessageDigest.getInstance("MD5");
		value = mobj.digest(data);
		valueSize = value.length;
		DebugInfo.printDump(3, value, "EAPtypePldMd5::setMD5value: MD5 hash");

		return res;
	}
*/
	public int parseNetMessage() {
		int res = 0;
		byte[] data = this.net_msg;
		if(data == null) 
			res = 1;
		else if(data.length < 3) 
			res = 2;
		else if((data[1] != 0) || (data[2] != 0)) 
			res = 3;
		else {
			this.subtype = EAPSIMsubtype.valueOf(data[0]);
			
			int len = 0;
			
			if(data.length > 3) {
				int curpos = 3;
				EAPsimAttribute newAttr = null;
				
				while(curpos < data.length - 1) {
					newAttr = parseAttributes(data,data[curpos],data[curpos + 1]*4 - 2,curpos + 2);
					if(newAttr == null) {
						res = 4;
						break;
					}
					this.addAttribute(newAttr);
					curpos += newAttr.getSize();
				}
				if(curpos != data.length) {
					DebugInfo.printMessage(1,"EAPtypePldSim::parseNetMessage: remain some bytes, length " + data.length + ", curpos " + curpos);
					res = 5;
				}
			}
		}
		
		return res;
	}

	public EAPsimAttribute parseAttributes(byte[] data, byte type, int len, int pos) {
		EAPsimAttribute attr = null;

		if(pos + len <= data.length) {
			switch(EAPsimAttrType.valueOf(type)) {
			case EAPSIM_AT_RAND: attr = new EAPsimAttrRand(data, len, pos); break;
			case EAPSIM_AT_PADDING: attr = new EAPsimAttrPadding(data, len, pos); break;
			case EAPSIM_AT_NONCE_MT: attr = new EAPsimAttrNonceMT(data, len, pos); break;
			case EAPSIM_AT_PERMANENT_ID_REQ: attr = new EAPsimAttrPermanentIdReq(data, len, pos); break;
			case EAPSIM_AT_MAC: attr = new EAPsimAttrMAC(data, len, pos); 
								parsedMACposition = pos + 7; break;
			case EAPSIM_AT_NOTIFICATION: attr = new EAPsimAttrNotification(data, len, pos); break;
			case EAPSIM_AT_ANY_ID_REQ: attr = new EAPsimAttrAnyIdReq(data, len, pos); break;
			case EAPSIM_AT_IDENTITY: attr = new EAPsimAttrIdentity(data, len, pos); break;
			case EAPSIM_AT_VERSION_LIST: attr = new EAPsimAttrVersionList(data, len, pos); break;  // parse message constructor
			case EAPSIM_AT_SELECTED_VERSION: attr = new EAPsimAttrSelectedVersion(data, len, pos); break;
			case EAPSIM_AT_FULLAUTH_ID_REQ: attr = new EAPsimAttrFullAuthIdReq(data, len, pos); break;
			case EAPSIM_AT_COUNTER: attr = new EAPsimAttrCounter(data, len, pos); break;
			case EAPSIM_AT_COUNTER_TOO_SMALL: attr = new EAPsimAttrCounterSmall(data, len, pos); break;
			case EAPSIM_AT_NONCE_S: attr = new EAPsimAttrNonceS(data, len, pos); break;
			case EAPSIM_AT_CLIENT_ERROR_CODE: attr = new EAPsimAttrClientErrorCode(data, len, pos); break;
			case EAPSIM_AT_IV: attr = new EAPsimAttrIV(data, len, pos); break;
			case EAPSIM_AT_ENCR_DATA: attr = new EAPsimAttrEncrData(data, len, pos); break;
			case EAPSIM_AT_NEXT_PSEUDONYM: attr = new EAPsimAttrNextPseudonym(data, len, pos); break;
			case EAPSIM_AT_NEXT_REAUTH_ID: attr = new EAPsimAttrNextReauthID(data, len, pos); break;
			case EAPSIM_AT_RESULT_IND: attr = new EAPsimAttrResultInd(data, len, pos); break;
			default: attr = new EAPsimAttribute(data, type, len, pos);
			}
		} else {
			DebugInfo.printMessage(1,"EAPtypePldSim::parseAttributes: length out of the array: " + data.length + ", index " + pos + ", need length " + len);
			DebugInfo.printMessage(1,"attrType: " + EAPsimAttrType.getString(type));
		}
	
		return attr;
	}

	public int toByteArray() {
		int i = 0;
		// create byte array
		byte[] ba = new byte[getSize()];
		
		ba[i++] = (byte)(this.subtype.getValue());
		ba[i++] = 0;
		ba[i++] = 0;

		//write attributes
		if(attrList != null) {
			byte[] attrArray;
			for (int j = 0; j < attrList.size(); j++) {
				if(attrList.get(j).net_msg == null)
					attrList.get(j).toByteArray();
				attrArray = attrList.get(j).net_msg;
				System.arraycopy(attrArray, 0, ba, i, attrArray.length);
				i += attrArray.length;
			}
		}

		this.net_msg = ba;
		return 0;
	}

	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "EAP-SIM payload:");
		DebugInfo.printMessage(1,pref + "  subtype: " + this.subtype);
		if(attrList != null) {
			for (int j = 0; j < attrList.size(); j++)
				attrList.get(j).print(pref + "    ");
		}
	}
}