package ru.ispras.telecomtesk.eap.EAPTLS;

import ru.ispras.telecomtesk.eap.BasicPayloadObject;
import ru.ispras.telecomtesk.eap.ConvertTypes;
import ru.ispras.telecomtesk.eap.DebugInfo;

public class EAPtypePldTls extends BasicPayloadObject {    
	public byte flag = 0; 
	public int  tlsMessageLenght = 0;
	public byte[] tlsData = null; 
	protected boolean isLengthEnable = false;
	public boolean isLengthSet = false;

	public EAPtypePldTls() {
	}

	public EAPtypePldTls(byte[] data) {
		if(data != null) {
			tlsData = new byte[data.length];
			System.arraycopy(data, 0, tlsData, 0, tlsData.length);
		}
	}

	public void setEapTlsFlag(boolean lBit, boolean mBit, boolean sBit) {
		if(lBit) { this.flag |= (byte)0x80; this.isLengthEnable = true; }
		if(mBit) this.flag |= (byte)0x40;
		if(sBit) this.flag |= (byte)0x20;
	}

	public boolean getEapTlsLBit() {
		return ((this.flag & 0x80) > 0);
	}
	public boolean getEapTlsMBit() {
		return ((this.flag & 0x40) > 0);
	}
	public boolean getEapTlsSBit() {
		return ((this.flag & 0x20) > 0);
	}
	public boolean getEapTlsLBit(byte bt) {
		return ((bt & 0x80) > 0);
	}
	public boolean getEapTlsMBit(byte bt) {
		return ((bt & 0x40) > 0);
	}
	public boolean getEapTlsSBit(byte bt) {
		return ((bt & 0x20) > 0);
	}


	public int getSize() {
		int res = 1;

		if(isLengthEnable)
			res += 4;
		if(tlsData != null) 
			res += tlsData.length;

		return res;
	}

	public void setLength() {
		if(tlsData != null) 
			tlsMessageLenght = tlsData.length;
		isLengthSet = true;
	}

	public void setLength(int len) {
		tlsMessageLenght = len;
		isLengthSet = true;
	}

	public int parseNetMessage() {
		int res = 0;
		byte[] data = this.net_msg;
		if(data == null) 
			res = 1;
		else if(data.length == 0) 
			res = 2;
		else if(getEapTlsSBit(data[0]) && (data.length != 1)) 
			res = 3;
		else if( (data.length < 5) && (getEapTlsLBit(data[0]) || getEapTlsMBit(data[0])) ) 
			res = 4;
		else {
			this.flag = data[0];
			int pos = 1;
			
			if(getEapTlsLBit(data[0]) || getEapTlsMBit(data[0])) {
				tlsMessageLenght = ConvertTypes.ntoh(data, 1, 4);
				isLengthEnable = true;
				isLengthSet = true;
				pos += 4;
			}
			
			tlsData = new byte[data.length - pos];
			System.arraycopy(data, pos, tlsData, 0, tlsData.length);
		}
		
		return res;
	}


	public int toByteArray() {
		int i = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		ba[i++] = this.flag;

		if(isLengthEnable) {
			byte[] msgln = ConvertTypes.hton(tlsMessageLenght);
			System.arraycopy(msgln, 0, ba, i, msgln.length);
			i += msgln.length;
		}

		if(this.tlsData != null)
			System.arraycopy(this.tlsData, 0, ba, i, this.tlsData.length);

		this.net_msg = ba;
		return 0;
	}

	public String eapTlsFlag2Str() {
		return String.format("%8s", Integer.toBinaryString(this.flag & 0xFF)).replace(' ', '0');
	}
	
	public String eapTlsMsgLength2Str() {
		if(isLengthEnable)
			return "" + tlsMessageLenght;
		else
			return "not presented";
	}
	
	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "EAP-TLS payload:");
		DebugInfo.printMessage(1,pref + "  flag: " + eapTlsFlag2Str());
		DebugInfo.printMessage(1,pref + "  TLS length: " + eapTlsMsgLength2Str());
		DebugInfo.printDumpWithLength(1, this.tlsData, pref + "  TLS Data:");
	}
}