package ru.ispras.telecomtesk.eap.EAPTTLSv0;

import ru.ispras.telecomtesk.eap.BasicPayloadObject;
import ru.ispras.telecomtesk.eap.ConvertTypes;
import ru.ispras.telecomtesk.eap.DebugInfo;


public class EAPtypePldTtlsv0AVP extends BasicPayloadObject {
	public int avpCode;
    public byte flag;         
    public int vendorID;         
    public int length;         
    public int paddingLength = 0;         
	public byte[] value = null; 
	protected boolean isLengthSet = false;

	public EAPtypePldTtlsv0AVP(int code) {
		this.avpCode = code;
		this.flag = 0;
		this.vendorID = 0;
	}
	
	public EAPtypePldTtlsv0AVP(int code, byte[] data) {
		this.avpCode = code;
		this.flag = 0;
		this.vendorID = 0;
		if(data != null) {
			value = new byte[data.length];
			System.arraycopy(data, 0, value, 0, data.length);
			setLength();
		}
	}
	
	public EAPtypePldTtlsv0AVP(int code, byte flag, int vendorID, byte[] data) {
		this.avpCode = code;
		this.flag = flag;
		this.vendorID = vendorID;
		if(data != null) {
			value = new byte[data.length];
			System.arraycopy(data, 0, value, 0, data.length);
			setLength();
		}
	}
	
	public EAPtypePldTtlsv0AVP(int code, byte[] data, int len) {
		this.avpCode = code;
		this.flag = 0;
		this.vendorID = 0;
		if(data != null) {
			int curlen = data.length >= len ? len : data.length;
			value = new byte[curlen];
			System.arraycopy(data, 0, value, 0, curlen);
			setLength();
		}
	}
	
	public EAPtypePldTtlsv0AVP(byte[] data, int code, int len, int pos) {
		this.avpCode = code;
		this.flag = 0;
		this.vendorID = 0;
		if((data != null)&&(len > 0)&&(pos > 0)) {
			int curlen = 0;
			if(pos <= data.length - 1) {
				curlen = data.length >= len + pos ? len : data.length - pos;
				value = new byte[curlen];
				System.arraycopy(data, pos, value, 0, curlen);
			}
			setLength();
		}
	}
	
	public void setAvpFlag(boolean vBit, boolean mBit) {
		if(vBit) this.flag |= (byte)0x80;
		if(mBit) this.flag |= (byte)0x40;
	}

	public void setAvpVBit() {
		this.flag |= (byte)0x80;
	}
	public void setAvpMBit() {
		this.flag |= (byte)0x40;
	}
	public boolean getAvpVBit() {
		return ((this.flag & 0x80) > 0);
	}
	public boolean getAvpMBit() {
		return ((this.flag & 0x40) > 0);
	}
	public boolean getAvpVBit(byte bt) {
		return ((bt & 0x80) > 0);
	}
	public boolean getAvpMBit(byte bt) {
		return ((bt & 0x40) > 0);
	}

	public static EAPtypePldTtlsv0AVP cloneAttribute(EAPtypePldTtlsv0AVP avp) {
		EAPtypePldTtlsv0AVP newAvp = null;
		if(avp != null)
			newAvp = new EAPtypePldTtlsv0AVP(avp.avpCode, avp.flag, avp.vendorID, avp.value);
		return newAvp;
	}
	
	public int getSize() {
		int res = 8;
		if(this.getAvpVBit()) 
			res += 4;
		if(value != null) 
			res += value.length;
		if(res % 4 > 0) {
			paddingLength = 4 - res % 4;
			res += paddingLength;
		}
		return res;
	}

	public void setLength() {
		length = (getSize() - paddingLength);
		isLengthSet = true;
	}

	public void setLength(int len) {
		length = len;
		isLengthSet = true;
	}


	public int parseNetMessage() {
		int res = 0,pos = 8;
		byte[] data = this.net_msg;
		DebugInfo.printDump(3, data, "EAPtypePldTtlsv0AVP::parseNetMessage: input data");
		if(data == null) 
			res = 1;
		else if(data.length < 8) {
			DebugInfo.printMessage(1, "!!!Error! EAPtypePldTtlsv0AVP::parseNetMessage: input data too short");
			res = 2;
		} else {
			this.avpCode = ConvertTypes.ntoh(data);
			this.flag = data[4];
			this.length = ConvertTypes.ntoh(data,5,3);
			if(this.getAvpVBit()) {
				if(data.length < 12) {
					DebugInfo.printMessage(1, "!!!Error! EAPtypePldTtlsv0AVP::parseNetMessage: input data with V bit too short");
				} else {
					this.vendorID = ConvertTypes.ntoh(data,8);
					pos += 4;
				}
			}
			if((this.length > data.length)&&(this.length < pos)) {
				DebugInfo.printMessage(1, "!!!Error! EAPtypePldTtlsv0AVP::parseNetMessage: length value failed");
			} else if (this.length > pos) {
				this.value = new byte[this.length - pos];
				System.arraycopy(data, pos, this.value, 0, value.length);
			}
		}
		
		return res;
	}

	
	public int toByteArray() {
		int i = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		byte[] tmpVal = ConvertTypes.hton(avpCode); // 4 bytes Code
		System.arraycopy(tmpVal, 0, ba, i, tmpVal.length);
		i += tmpVal.length;
		
		tmpVal = ConvertTypes.hton(length); // 4 bytes Length but need 3 bytes
		System.arraycopy(tmpVal, 0, ba, i, tmpVal.length);
		ba[i] = flag;  // replace first byte by Flag
		i += tmpVal.length;
		
		if(this.getAvpVBit()) {
			tmpVal = ConvertTypes.hton(vendorID); // 4 bytes vendorID
			System.arraycopy(tmpVal, 0, ba, i, tmpVal.length);
			i += tmpVal.length;
		}
		
		if(value != null) {
			System.arraycopy(value, 0, ba, i, value.length);
			i += value.length;
		}
		for(; i < ba.length; i++) {
			ba[i] = 0;
		}
		
		net_msg = ba;
		return 0;
	}

	public String avpFlag2Str() {
		return String.format("%8s", Integer.toBinaryString(this.flag & 0xFF)).replace(' ', '0');
	}
	
	public String avpVendorID2Str() {
		if(this.getAvpVBit())
			return "" + this.vendorID;
		else
			return "not presented";
	}
	
	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "  Code: " + this.avpCode + " " + EAPtypePldTtlsv0AVPCode.valueOf(this.avpCode));
		DebugInfo.printMessage(1,pref + "  Flag: " + avpFlag2Str());
		DebugInfo.printMessage(1,pref + "  vendorID: " + avpVendorID2Str());
		DebugInfo.printMessage(1,pref + "  length: " + this.length);
		DebugInfo.printDump(1, this.value, pref + "  value:");
	}
}    
