//
// check first message: EAP-TLS-ClientHello -- ACK message
//

package ru.ispras.telecomtesk.eap.EAPTTLSv0;

import java.io.*;
import java.math.BigInteger;
import java.net.*;
import java.util.ArrayList;
import java.security.*;

import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.*;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.TLSExtensions.*;
import ru.ispras.telecomtesk.tlsv12.RecordProtocol.*;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.*;
import ru.ispras.telecomtesk.tlsv12.TLSState.*;

import javax.crypto.interfaces.DHPublicKey;
import ru.ispras.telecomtesk.eap.*;
import ru.ispras.telecomtesk.eap.EAPTLS.EAPTlsFragmentation;
import ru.ispras.telecomtesk.eap.EAPoverRadius.*;





public class testEAPTtlsv0_2
{
    public static void main(String args[]) throws Exception {
    	testEAPTtlsv0_2_Run test1 = new testEAPTtlsv0_2_Run();
    	test1.runtest();
    }
}

class testEAPTtlsv0_2_Run
{
	public static byte[] remoteHostRaw = {(byte)(10),(byte)(10),(byte)(12),(byte)(12)};
	public static byte[] localHostRaw = {(byte)(10),(byte)(10),(byte)(9),(byte)(53)};

	EAPtype testType = EAPtype.EAP_TYPE_TTLSv0;

	String SharedSecretStr = "testing123";
//	String SharedSecretStr = "secret";
	String userNameStr = "tester";
	String UserPwdStr = "tester"; // "123456";
//	String UserPwdStr = "efqcltpm+>y]";

	RecordLayer tlsmsg = null;
	int tlsFullMsgLen = 0;
	ArrayList<byte[]> tlsMsgList = null;
	EAPRadiusSocket sock = null;
	EAPpldBasic ttlsEapPktCH = null;
	EAPtypePldTtlsv0 ttlsPld = null;
	byte[] tlsClHello = null;
	
	public void runtest() throws Exception {
		DebugInfo.debugLevel = 3;
		DebugInfo.maxDumpSize = 200;
		ru.ispras.telecomtesk.tlsv12.DebugInfo.debugLevel = 3;

		sock = new EAPRadiusSocket(SharedSecretStr, userNameStr, remoteHostRaw); 
//		sock.setSocketTimeout(5000);
		tlsMsgList = new ArrayList<byte[]>();
		tlsClHello = createFirstTlsMsg();
		TestTTlsv0Sequence2 testSeq2 = new TestTTlsv0Sequence2();

		int res;
		int eapTestInd, eapSubInd;

	    for (eapTestInd = 0; eapTestInd < testSeq2.numberTests.length; eapTestInd++) {
		    for (eapSubInd = 1; eapSubInd <= testSeq2.numberTests[eapTestInd]; eapSubInd++) {
		    	DebugInfo.printMessage(1,"eapTestInd, eapSubInd: " + eapTestInd + ", " + eapSubInd);
		    	ttlsPld = new EAPtypePldTtlsv0();
		    	ttlsPld = testSeq2.setTestPld(eapTestInd, eapSubInd, ttlsPld, tlsClHello);
		    	if(ttlsPld == null) {
		    		sock.closeSocket();
		    		return;
		    	}
		    	
	    		res = sendPacketsSequence();
	    		if(res > 1) {
	    			sock.closeSocket();
	    			return;
			    }
		    }
	    }
		
	    
		DebugInfo.printMessage(1,"run: counterEAPPackets " + sock.counterEAPPackets + ", counterEAPTerminatePackets " + sock.counterEAPTerminatePackets);

// close socket		
		sock.closeSocket();
	}

	int sendPacketsSequence() {
		int res = 0;
		
		// send init ID packet
		DebugInfo.printMessage(1,"\n================================================");
		
		res = sock.sendInitEAPpacket();
		if((res != 0) || (!sock.receivedData)) { 
			DebugInfo.printMessage(1,"!!! sendInitEAPpacket failed");
			sock.sendTerminatePacket();
			return 1;
		}

		
// send second message		
		DebugInfo.printMessage(1,"-------------------------------");
		DebugInfo.printMessage(1,"create TLS ClientHello");
		if(tlsClHello == null) {
			sock.closeSocket();
			return 2;
		}

		ttlsEapPktCH = new EAPpldBasic();
		ttlsEapPktCH.code = EAPcode.EAP_CODE_RESPONSE;
		ttlsEapPktCH.identifier = sock.getReceivedEAPPacket().identifier;
		ttlsEapPktCH.type = testType;
		ttlsEapPktCH.EAPmethodPld = new EAPtypePldTtlsv0(tlsClHello);
		ttlsEapPktCH.isLengthSet = false;
		ttlsEapPktCH.toByteArray();
		
		sock.setEapMessage(ttlsEapPktCH);

		// get all EAP fragments
/*		res = EAPTlsFragmentation.getAllTlsFragments(sock,tlsMsgList);
		if(res != 0) { 
			sock.sendTerminatePacket();
			return;
		}
*/
		tlsMsgList.clear();
		boolean cicleFlag;
		EAPheader recvEAP;
		tlsFullMsgLen = 0;
		cicleFlag = true;
		int counterEmptyRecv = 0;
		
		res = sock.sendEAPpacket();   // Send Client Hello
		if((res != 0) || (!sock.receivedData)) { 
			DebugInfo.printMessage(1,"!!! sendEAPpacket failed");
			sock.sendTerminatePacket();
			return 3;
		}
		recvEAP = sock.getReceivedEAPPacket();
		cicleFlag = false;

		if(recvEAP instanceof EAPpldBasic) {
			if(((EAPpldBasic) recvEAP).type == testType) {
				EAPtypePldTtlsv0 curpld = (EAPtypePldTtlsv0)((EAPpldBasic)recvEAP).EAPmethodPld;
				if(curpld.getEapTlsLBit()) {
					if(tlsFullMsgLen == 0)
						tlsFullMsgLen = curpld.tlsMessageLenght;
					else {
						DebugInfo.printMessage(1,"!!! Warning! tlsMessageLenght dublicate");
					}
				}
				tlsMsgList.add(curpld.tlsData);
				if(curpld.getEapTlsMBit()) {
					cicleFlag = true;
					ttlsEapPktCH.identifier = sock.getReceivedEAPPacket().identifier;
					ttlsEapPktCH.EAPmethodPld = ttlsPld;
					ttlsEapPktCH.isLengthSet = false;
					ttlsEapPktCH.toByteArray();
					sock.setEapMessage(ttlsEapPktCH);
				} 
			}
		}
		
		while(cicleFlag) {
			DebugInfo.printMessage(1,"*******************");
			res = sock.sendEAPpacket();
			DebugInfo.printMessage(1,"*******************");
			if((res != 0) || (!sock.receivedData)) { 
				DebugInfo.printMessage(1,"!!! sendEAPpacket failed");
				sock.sendTerminatePacket();
				return 3;
			}
			recvEAP = sock.getReceivedEAPPacket();
			cicleFlag = false;

			if(recvEAP instanceof EAPpldBasic) {
				if(((EAPpldBasic) recvEAP).type == testType) {
					EAPtypePldTtlsv0 curpld = (EAPtypePldTtlsv0)((EAPpldBasic)recvEAP).EAPmethodPld;
					if(curpld.getEapTlsLBit()) {
						if(tlsFullMsgLen == 0)
							tlsFullMsgLen = curpld.tlsMessageLenght;
						else {
							DebugInfo.printMessage(1,"!!! Warning! tlsMessageLenght dublicate");
						}
					}
					tlsMsgList.add(curpld.tlsData);
					if(curpld.getEapTlsMBit()) {
						cicleFlag = true;
						ttlsEapPktCH.identifier = sock.getReceivedEAPPacket().identifier;
						ttlsEapPktCH.EAPmethodPld = new EAPtypePldTtlsv0();
						ttlsEapPktCH.isLengthSet = false;
						ttlsEapPktCH.toByteArray();
						sock.setEapMessage(ttlsEapPktCH);
					} else if((curpld.tlsData == null) || (curpld.tlsData.length == 0)) {
						cicleFlag = true;
						ttlsEapPktCH.identifier = sock.getReceivedEAPPacket().identifier;
						ttlsEapPktCH.EAPmethodPld = new EAPtypePldTtlsv0();
						if(counterEmptyRecv > 0)
							((EAPtypePldTtlsv0)(ttlsEapPktCH.EAPmethodPld)).setEapTlsFlag(true,false,false);
						ttlsEapPktCH.isLengthSet = false;
						ttlsEapPktCH.toByteArray();
						sock.setEapMessage(ttlsEapPktCH);
						counterEmptyRecv++;
					}
				}
			}
		}
///////
		byte[] tlsRaw = EAPTlsFragmentation.combineTlsMessages(tlsMsgList);
		if(tlsRaw == null) {
			DebugInfo.printMessage(1,"!!! Error! tlsRaw Null pointer");
		} else if(tlsRaw.length != tlsFullMsgLen) {
			DebugInfo.printMessage(1,"!!! Error! tlsRaw length failed: " + tlsRaw.length + " must " + tlsFullMsgLen);
		} else {
			DebugInfo.printDumpWithLength(2, tlsRaw, "Get full tls messages");
		}

		// send third message		
		sock.sendTerminatePacket();
		return res;
	}


	byte[] createFirstTlsMsg() {
		DebugInfo.printMessage(3, "testEAPTtlsv0_1::createFirstTlsMsg");
		int res = 0;
		byte[] resTls = null;
		ArrayList<RecordLayer> response;
		short[] cs = {TLSCipherSuite.TLS_RSA_WITH_AES_128_CBC_SHA.getValue(),
					  TLSCipherSuite.TLS_RSA_WITH_AES_256_CBC_SHA256.getValue()
					 };
		Hello msg = new Hello(cs); 
		SignatureAndHashAlgorithm[] sah = {
				new SignatureAndHashAlgorithm(HashAlgorithm.SHA1, SignatureAlgorithm.RSA),
				new SignatureAndHashAlgorithm(HashAlgorithm.MD5, SignatureAlgorithm.RSA),
				new SignatureAndHashAlgorithm(HashAlgorithm.SHA256, SignatureAlgorithm.RSA)
				};
		SignatureAndHashAlgorithmExt hl_ext_sah = new SignatureAndHashAlgorithmExt(sah);
		msg.addExtension(hl_ext_sah);

		RecordLayer tlsmsg = new RecordLayer();
		tlsmsg.msglist = new ArrayList();
		tlsmsg.msglist.add(msg);

		TLSConnectionSelector sel = new TLSConnectionSelector(null,null,0,0);
		TLSConnection current_connection = new TLSConnection(sel,true);
		current_connection.init();
		
		try {
			msg.toByteArray();
			DebugInfo.printDump(5, msg.net_msg, "testEAPTtlsv0_1::createFirstTlsMsg: ClientHello msg: ");
		} catch (Exception err) {
			DebugInfo.printMessage(1,"===Error! testEAPTtlsv0_1::createFirstTlsMsg: ClientHello Exception: " + err);                   res = 1;
		}

		try {
			tlsmsg.setContent(msg,null,current_connection);
		} catch (Exception err) {
			DebugInfo.printMessage(1,"===Error! testEAPTtlsv0_1::createFirstTlsMsg:Record layer Exception: " + err);
			res = 2;
		}

		if(res == 0)
			resTls = tlsmsg.toByteArray();

		return resTls;
	}
}




class TestTTlsv0Sequence2
{
	int numberTestsLbit = 7;
	int numberTestsMbit = 8;
	int numberTestsRbits = 3;        
	int numberTestsVbits = 6;
	int numberTestsData = 1;

	int[] numberTests = {numberTestsLbit, numberTestsMbit, numberTestsRbits, numberTestsVbits, numberTestsData};
    
	TestTTlsv0Sequence2() {
	}
	
	public EAPtypePldTtlsv0 setTestPld(int eapTestInd, int eapSubInd, EAPtypePldTtlsv0 pld, byte[] chl) {
		DebugInfo.printMessage(1,"setTestPld");
		EAPtypePldTtlsv0 tnpPld = null;

		if(pld == null) {
			DebugInfo.printMessage(1,"!!! ERROR: setTestPld: null pointer");
			return tnpPld;
		}

		switch(eapTestInd) {
		case 0: tnpPld = setLbit(eapSubInd, pld, chl); break;
		case 1: tnpPld = setMbit(eapSubInd, pld, chl); break;
		case 2: tnpPld = setRbits(eapSubInd, pld); break;
		case 3: tnpPld = setVbits(eapSubInd, pld); break;
		case 4: tnpPld = setData(eapSubInd, pld); break;
		}
		
		DebugInfo.printMessage(1,"setTestPld return");
		return tnpPld;
	}
	
	public EAPtypePldTtlsv0 setLbit(int ind, EAPtypePldTtlsv0 pld, byte[] chl) {
		int i;
		byte[] tmpba;
		pld.setEapTlsFlag(true,false,false); // set L bit
		
		switch(ind) {
		case 1: pld.setEapTlsFlag(false,false,true); break; // set Start bit
		case 2: pld.setLength(0); break; // Length = 0
		case 3: pld.setLength(5); break; // 
		case 4: pld.setLength();
			tmpba = new byte[8];
			for(i=0;i<tmpba.length;i++) {
				tmpba[i] = 0;
			}
			pld.tlsData = tmpba;
			break; // Length = 0
		case 5: pld.setLength();
			pld.tlsData = chl;
			break; // Length = 0
		case 6: 
			tmpba = new byte[8];
			for(i=0;i<tmpba.length;i++) {
				tmpba[i] = 0;
			}
			pld.tlsData = tmpba;
			pld.setLength();
			break; // Length = 0
		case 7: 
			pld.tlsData = chl;
			pld.setLength();
			break; // Length = 0
		}

		return pld;
	}

	public EAPtypePldTtlsv0 setMbit(int ind, EAPtypePldTtlsv0 pld, byte[] chl) {
		int i;
		byte[] tmpba;
		pld.setEapTlsFlag(false,true,false); // set M bit

		switch(ind) {
		case 1: pld.setEapTlsFlag(false,false,true); break; // set Start bit
		case 2: break; // 
		case 3: pld.setEapTlsFlag(true,false,false); // set L  
			pld.setLength(0); break; // Length = 0
		case 4: pld.setEapTlsFlag(true,false,false); // set L  
			pld.setLength(5); break; // Length > tlsData.length
		case 5: pld.setEapTlsFlag(true,false,false); // set L  
			pld.setLength();
			tmpba = new byte[8];
			for(i=0;i<tmpba.length;i++) {
				tmpba[i] = 0;
			}
			pld.tlsData = tmpba;
			break; 
		case 6: pld.setEapTlsFlag(true,false,false); // set L 
			pld.setLength();
			pld.tlsData = chl;
			break; 
		case 7: pld.setEapTlsFlag(true,false,false); // set L  
			tmpba = new byte[8];
			for(i=0;i<tmpba.length;i++) {
				tmpba[i] = 0;
			}
			pld.tlsData = tmpba;
			pld.setLength();
			break; 
		case 8: pld.setEapTlsFlag(true,false,false); // set L 
			pld.tlsData = chl;
			pld.setLength();
			break; 
		}

		return pld;
	}

	public EAPtypePldTtlsv0 setRbits(int ind, EAPtypePldTtlsv0 pld) {
		switch(ind) {
		case 1: pld.flag = (byte)0x20; break; // set Start bit
		case 2: pld.flag = (byte)0x18; break; // set Reserved bits
		case 3: pld.flag = (byte)0x18; 
				pld.setEapTlsFlag(true,false,false); // set Reserved bits
				break; 
		}
		return pld;
	}

	public EAPtypePldTtlsv0 setVbits(int ind, EAPtypePldTtlsv0 pld) {
		switch(ind) {
		case 1: pld.flag = (byte)0x0; break; 
		case 2: pld.flag = (byte)0x01; break; 
		case 3: pld.flag = (byte)0x02; break; 
		case 4: pld.flag = (byte)0x04; break; 
		case 5: pld.flag = (byte)0x07; break; 
		case 6: pld.flag = (byte)0x02; 
				pld.setEapTlsFlag(true,false,false); // set Reserved bits
				break; 
		}
		return pld;
	}

	public EAPtypePldTtlsv0 setData(int ind, EAPtypePldTtlsv0 pld) {
		switch(ind) {
		case 1: byte[] tmpba = {0,0,0,0,0,0,0,0,0};
			pld = new EAPtypePldTtlsv0(tmpba); break; 
		}
		return pld;
	}
}
