//
// After EAP-TLS-Handshake finished send: 
//    - ACK message
//    - under TLS: AVP "Reply-Message" with string "Hello"
//    - under TLS: AVP "Reply-Message" with bytes array {0,0,0,0,0,0,0,0}
//

package ru.ispras.telecomtesk.eap.EAPTTLSv0;

import java.util.ArrayList;

import ru.ispras.telecomtesk.tlsv12.Tlsv12Client;
import ru.ispras.telecomtesk.tlsv12.RecordProtocol.*;
import ru.ispras.telecomtesk.tlsv12.TLSSocket.TLSResponse;

import ru.ispras.telecomtesk.eap.*;
import ru.ispras.telecomtesk.eap.EAPTLS.EAPTlsFragmentation;
import ru.ispras.telecomtesk.eap.EAPoverRadius.*;





public class testEAPTtlsv0_tls_1
{
    public static void main(String args[]) throws Exception {
    	testEAPTtlsv0_tls_1_Run test1 = new testEAPTtlsv0_tls_1_Run();
    	test1.runtest();
    }
}

class testEAPTtlsv0_tls_1_Run
{
	public static byte[] remoteHostRaw = {(byte)(10),(byte)(10),(byte)(12),(byte)(12)};
	public static byte[] localHostRaw = {(byte)(10),(byte)(10),(byte)(9),(byte)(53)};

	EAPtype testType = EAPtype.EAP_TYPE_TTLSv0;

	String SharedSecretStr = "testing123";
//	String SharedSecretStr = "secret";
	String userNameStr = "tester";
	String UserPwdStr = "tester"; // "123456";
//	String UserPwdStr = "efqcltpm+>y]";

	RecordLayer tlsmsg = null;
	int tlsFullMsgLen = 0;
	ArrayList<byte[]> tlsMsgList = null;
	EAPRadiusSocket sock = null;
	int eapTestInd = 0, eapSubInd = 0;

	Tlsv12Client tlsClnt;
	TestTTlsv0TLSSequence1 testSeq1;
	
	public void runtest() throws Exception {
		DebugInfo.debugLevel = 3;
		DebugInfo.maxDumpSize = 200;
		ru.ispras.telecomtesk.tlsv12.DebugInfo.debugLevel = 3;

		sock = new EAPRadiusSocket(SharedSecretStr, userNameStr, remoteHostRaw); 
//		sock.setSocketTimeout(5000);
		tlsMsgList = new ArrayList<byte[]>();
		testSeq1 = new TestTTlsv0TLSSequence1();
		tlsClnt = new Tlsv12Client();

		int res;
	    for (eapTestInd = 0; eapTestInd < testSeq1.numberTests; eapTestInd++) {
		    	DebugInfo.printMessageWithSeparator(1,"eapTestInd: " + eapTestInd);
	    		res = sendPacketsSequence();
				sock.sendTerminatePacket();
	    }
		DebugInfo.printMessage(1,"run: counterEAPPackets " + sock.counterEAPPackets + ", counterEAPTerminatePackets " + sock.counterEAPTerminatePackets);

// close socket		
		sock.closeSocket();
	}

	
	int sendPacketsSequence() {
		int res = 0;
		
		// send init ID packet
		DebugInfo.printMessage(1,"\n================================================");
		
		res = sock.sendInitEAPpacket();
		if((res != 0) || (!sock.receivedData)) { 
			DebugInfo.printMessage(1,"!!! sendInitEAPpacket failed");
			return 1;
		}
		
		// send second message		
		DebugInfo.printMessageWithSeparator(1,"create TLS ClientHello");
		byte[] tlsClHello = tlsClnt.createClientFirstTlsMsg();
		if(tlsClHello == null) {
			DebugInfo.printMessage(1,"!!! create Tls Client Hello failed");
			return 2;
		}
		EAPpldBasic ttlsEapPktCH = createEapPkt(new EAPtypePldTtlsv0(tlsClHello));
		if(0 != recvAndParseEAPmsg(ttlsEapPktCH)) return 3;

    	// send third message		
		DebugInfo.printMessageWithSeparator(1,"try create pkt3");
		byte[] msgSF = tlsClnt.createClientSecondTlsFlow();
		if(msgSF == null) {
			DebugInfo.printMessage(1,"!!! create Tls Client second flow failed");
			return 4;
		}
		EAPpldBasic ttlsEapPktThird = createEapPkt(new EAPtypePldTtlsv0(msgSF));
		if(0 != recvAndParseEAPmsg(ttlsEapPktThird)) return 5;
    	
	   	// send fourth message		
		DebugInfo.printMessageWithSeparator(1,"try create pkt4 under TLS");
		EAPpldBasic ttlsEapPktApp = createEapPkt(testSeq1.setTestPld(eapTestInd, tlsClnt));
		DebugInfo.printMessage(1,"**********************************************************");
		if(0 != recvAndParseEAPmsg(ttlsEapPktApp)) return 7;

		return res;
	}

	EAPpldBasic createEapPkt(EAPtypePldTtlsv0 ttlspld) {
		EAPpldBasic testEapPkt = new EAPpldBasic();
		testEapPkt.code = EAPcode.EAP_CODE_RESPONSE;
		testEapPkt.identifier = sock.getReceivedEAPPacket().identifier;
		testEapPkt.type = testType;
		testEapPkt.EAPmethodPld = ttlspld;
		testEapPkt.isLengthSet = false;
		testEapPkt.toByteArray();
		return testEapPkt;
	}


	int recvFullEAPmsg() {
		int res = 0;
		tlsMsgList.clear();
		boolean cicleFlag;
		EAPheader recvEAP;
		tlsFullMsgLen = 0;
		cicleFlag = true;
		int counterEmptyRecv = 0;
		EAPpldBasic ttlsEapACKpkt = new EAPpldBasic();
		
		ttlsEapACKpkt.code = EAPcode.EAP_CODE_RESPONSE;
		ttlsEapACKpkt.type = testType;

		while(cicleFlag) {
			res = sock.sendEAPpacket();
			if((res != 0) || (!sock.receivedData)) { 
				DebugInfo.printMessage(1,"!!! sendEAPpacket failed");
				return 1;
			}
			recvEAP = sock.getReceivedEAPPacket();
			cicleFlag = false;

			if(recvEAP instanceof EAPpldBasic) {
				if(((EAPpldBasic) recvEAP).type == testType) {
					EAPtypePldTtlsv0 curpld = (EAPtypePldTtlsv0)((EAPpldBasic)recvEAP).EAPmethodPld;
					if(curpld.getEapTlsLBit()) {
						if(tlsFullMsgLen == 0)
							tlsFullMsgLen = curpld.tlsMessageLenght;
						else {
							DebugInfo.printMessage(1,"!!! Warning! tlsMessageLenght dublicate");
						}
					}
					tlsMsgList.add(curpld.tlsData);
					if(curpld.getEapTlsMBit()) {
						cicleFlag = true;
						ttlsEapACKpkt.identifier = sock.getReceivedEAPPacket().identifier;
						ttlsEapACKpkt.EAPmethodPld = new EAPtypePldTtlsv0();
						ttlsEapACKpkt.isLengthSet = false;
						ttlsEapACKpkt.toByteArray();
						sock.setEapMessage(ttlsEapACKpkt);
					} else if((curpld.tlsData == null) || (curpld.tlsData.length == 0)) {
						cicleFlag = true;
						ttlsEapACKpkt.identifier = sock.getReceivedEAPPacket().identifier;
						ttlsEapACKpkt.EAPmethodPld = new EAPtypePldTtlsv0();
						ttlsEapACKpkt.isLengthSet = false;
						if(counterEmptyRecv > 0)
							((EAPtypePldTtlsv0)(ttlsEapACKpkt.EAPmethodPld)).setEapTlsFlag(true,false,false);
						ttlsEapACKpkt.toByteArray();
						sock.setEapMessage(ttlsEapACKpkt);
						counterEmptyRecv++;
					}
				}
			}
		}
		return res;
	}

	
	int parseTLSresp(byte[] tlsmsg) {
		DebugInfo.printMessageWithSeparator(1,"parse tls response");
		int res = 0;
		
		TLSResponse resp = new TLSResponse(tlsmsg);
   		resp.printResponse();
    	if(resp.err != 0) {
			DebugInfo.printMessage(1,"!!! TLSResponse return error");
			res = 1;
    	} else {
    		try {
    			ArrayList<RecordLayer> tlsResp = tlsClnt.parseTlsResponse(resp);	
    			if(null == tlsResp  ||  tlsClnt.conSpec.disconnectWithSendAlert != -1) {
    				DebugInfo.printMessage(1,"!!! TLSResponse return error");
    				tlsClnt.conSpec.disconnect();
    				res = 2;
    			}
    		} catch (Exception err) {
    			DebugInfo.printMessage(1,"===Error! parseTlsResponse Exception: " + err);
    			res = 3;
    		}
    	}
    	return res;
	}

	int recvAndParseEAPmsg(EAPpldBasic ttlsEapPkt) {
		int res = 0;
		
		sock.setEapMessage(ttlsEapPkt);

		if(0 != recvFullEAPmsg()) return 1;

		byte[] tlsRaw = EAPTlsFragmentation.combineTlsMessages(tlsMsgList);
		if(tlsRaw == null) {
			DebugInfo.printMessage(1,"!!! Error! tlsRaw Null pointer");
		} else if(tlsRaw.length != tlsFullMsgLen) {
			DebugInfo.printMessage(1,"!!! Error! tlsRaw length failed: " + tlsRaw.length + " must " + tlsFullMsgLen);
		} else {
			DebugInfo.printDumpWithLength(2, tlsRaw, "Get full tls messages");
		}

		// parse tls response		
		if(0 != parseTLSresp(tlsRaw)) return 2;

		return res;
	}

}




class TestTTlsv0TLSSequence1
{
	int numberTests = 3;
   
	TestTTlsv0TLSSequence1() {
	}
	
	public EAPtypePldTtlsv0 setTestPld(int eapTestInd, Tlsv12Client clnt) {
		DebugInfo.printMessage(1,"setTestPld");
		EAPtypePldTtlsv0 tnpPld = null;

		if(clnt == null) {
			DebugInfo.printMessage(1,"!!! ERROR: setTestPld: null pointer");
			return tnpPld;
		}
		String str = "Hello";
		byte[][] testdata = {str.getBytes(),{0,0,0,0,0,0,0,0}};

		switch(eapTestInd) {
		case 0: 
		case 1: 
			EAPtypePldTtlsv0AVP testAVPdata = new EAPtypePldTtlsv0AVP(18,testdata[eapTestInd]); // Reply-Message Attribute
			testAVPdata.toByteArray();

			byte[] tlsAppl = clnt.createApplicationData(testAVPdata.net_msg);
			if(tlsAppl == null) {
				DebugInfo.printMessage(1,"!!! create Tls Application message failed");
				return null;
			}
			tnpPld = new EAPtypePldTtlsv0(tlsAppl); break;
		case 2: tnpPld = new EAPtypePldTtlsv0(); break;
		}

		
		DebugInfo.printMessage(1,"setTestPld return");
		return tnpPld;
	}
	
}
