package ru.ispras.telecomtesk.eap.EAPTTLSv0;

import java.io.*;
import java.math.BigInteger;
import java.net.*;
import java.util.ArrayList;
import java.security.*;

import ru.ispras.telecomtesk.eap.DebugInfo;
import ru.ispras.telecomtesk.eap.*;
import ru.ispras.telecomtesk.eap.EAPTLS.EAPTlsFragmentation;
import ru.ispras.telecomtesk.eap.EAPoverRadius.*;

import ru.ispras.telecomtesk.tlsv12.Tlsv12Client;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.*;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.TLSExtensions.*;
import ru.ispras.telecomtesk.tlsv12.RecordProtocol.*;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.*;
import ru.ispras.telecomtesk.tlsv12.TLSSocket.TLSResponse;
import ru.ispras.telecomtesk.tlsv12.TLSState.*;

import javax.crypto.interfaces.DHPublicKey;





public class tryEAPTtlsv0init
{
    public static void main(String args[]) throws Exception {
    	tryEAPTtlsv0initRun test1 = new tryEAPTtlsv0initRun();
    	test1.runtest();
    }
}

class tryEAPTtlsv0initRun
{
	public static byte[] remoteHostRaw = {(byte)(10),(byte)(10),(byte)(12),(byte)(12)};
	public static byte[] localHostRaw = {(byte)(10),(byte)(10),(byte)(9),(byte)(53)};

	
	public void runtest() throws Exception {
		DebugInfo.debugLevel = 6;
		DebugInfo.maxDumpSize = 2000;
		ru.ispras.telecomtesk.tlsv12.DebugInfo.debugLevel = 6;
		ru.ispras.telecomtesk.tlsv12.DebugInfo.maxDumpSize = 600;
		
		EAPtype testType = EAPtype.EAP_TYPE_TTLSv0;

		String SharedSecretStr = "testing123";
//		String SharedSecretStr = "secret";
		String userNameStr = "tester";
		String UserPwdStr = "tester"; // "123456";
//		String UserPwdStr = "efqcltpm+>y]";

		System.out.println("sendPacket:" + DebugInfo.debugLevel + " " + DebugInfo.maxDumpSize);
		EAPRadiusSocket sock = new EAPRadiusSocket(SharedSecretStr, userNameStr, remoteHostRaw); 
//		sock.setSocketTimeout(5000);

		Tlsv12Client tlsClnt = new Tlsv12Client();
		
		int res;
		boolean cicleFlag;
		EAPheader recvEAP;
		int tlsFullMsgLen = 0;
		ArrayList<byte[]> tlsMsgList = new ArrayList<byte[]>();
		System.out.println("sendPacket:" + DebugInfo.debugLevel + " " + DebugInfo.maxDumpSize);
		
// send first message		
		DebugInfo.printMessage(1,"try create pkt1");
		res = sock.sendInitEAPpacket();
		if((res != 0) || (!sock.receivedData)) { 
			DebugInfo.printMessage(1,"!!! sendInitEAPpacket failed");
			sock.sendTerminatePacket();
			return;
		}

		System.out.println("sendPacket:" + DebugInfo.debugLevel + " " + DebugInfo.maxDumpSize);
		
// send second message		
		DebugInfo.printMessageWithSeparator(1,"try create pkt2");
		DebugInfo.printMessage(1,"create TLS ClientHello");
		
		byte[] msgCH = tlsClnt.createClientFirstTlsMsg();
		if(msgCH == null) {
			DebugInfo.printMessage(1,"!!! create Tls Client Hello failed");
			sock.sendTerminatePacket();
			return;
		}

		EAPpldBasic ttlsEapPktCH = new EAPpldBasic();
		ttlsEapPktCH.code = EAPcode.EAP_CODE_RESPONSE;
		ttlsEapPktCH.identifier = sock.getReceivedEAPPacket().identifier;
		ttlsEapPktCH.type = testType;
		ttlsEapPktCH.EAPmethodPld = new EAPtypePldTtlsv0(msgCH);
		ttlsEapPktCH.isLengthSet = false;
		ttlsEapPktCH.toByteArray();
		
		sock.setEapMessage(ttlsEapPktCH);

		// get all EAP fragments
		tlsFullMsgLen = 0;
		cicleFlag = true;
		tlsMsgList.clear();
		while(cicleFlag) {
			res = sock.sendEAPpacket();
			if((res != 0) || (!sock.receivedData)) { 
				DebugInfo.printMessage(1,"!!! sendInitEAPpacket failed");
				sock.sendTerminatePacket();
				return;
			}
			recvEAP = sock.getReceivedEAPPacket();
			cicleFlag = false;

			if(recvEAP instanceof EAPpldBasic) {
				if(((EAPpldBasic) recvEAP).type == testType) {
					EAPtypePldTtlsv0 curpld = (EAPtypePldTtlsv0)((EAPpldBasic)recvEAP).EAPmethodPld;
					if(curpld.getEapTlsLBit()) {
						if(tlsFullMsgLen == 0)
							tlsFullMsgLen = curpld.tlsMessageLenght;
						else {
							DebugInfo.printMessage(1,"!!! Warning! tlsMessageLenght dublicate");
						}
					}
					tlsMsgList.add(curpld.tlsData);
					if(curpld.getEapTlsMBit()) {
						cicleFlag = true;
						ttlsEapPktCH.identifier = sock.getReceivedEAPPacket().identifier;
						ttlsEapPktCH.EAPmethodPld = new EAPtypePldTtlsv0();
						ttlsEapPktCH.isLengthSet = false;
						ttlsEapPktCH.toByteArray();
						sock.setEapMessage(ttlsEapPktCH);
					}
				}
			}
		}

		byte[] tlsRaw = EAPTlsFragmentation.combineTlsMessages(tlsMsgList);
		if(tlsRaw == null) {
			DebugInfo.printMessage(1,"!!! Error! tlsRaw Null pointer");
		} else if(tlsRaw.length != tlsFullMsgLen) {
			DebugInfo.printMessage(1,"!!! Error! tlsRaw length failed: " + tlsRaw.length + " must " + tlsFullMsgLen);
		} else {
			DebugInfo.printDumpWithLength(2, tlsRaw, "Get full tls messages");
		}

// parse tls response		
		DebugInfo.printMessageWithSeparator(1,"parse tls response");
		
		TLSResponse resp = new TLSResponse(tlsRaw);
   		resp.printResponse();
    	if(resp.err != 0) {
			DebugInfo.printMessage(1,"!!! TLSResponse return error");
			sock.sendTerminatePacket();
			return;
    	}

    	try {
    		ArrayList<RecordLayer> tlsResp = tlsClnt.parseTlsResponse(resp);	
    		if(null == tlsResp  ||  tlsClnt.conSpec.disconnectWithSendAlert != -1) {
    			DebugInfo.printMessage(1,"!!! TLSResponse return error");
    			tlsClnt.conSpec.disconnect();
    			sock.sendTerminatePacket();
    			return;
        	}
    	} catch (Exception err) {
			DebugInfo.printMessage(1,"===Error! parseTlsResponse Exception: " + err);
			sock.sendTerminatePacket();
			return;
		}

		System.out.println("sendPacket:" + DebugInfo.debugLevel + " " + DebugInfo.maxDumpSize);

// send third message		
			DebugInfo.printMessageWithSeparator(1,"try create pkt3");

			byte[] msgSF = tlsClnt.createClientSecondTlsFlow();
			if(msgSF == null) {
				DebugInfo.printMessage(1,"!!! create Tls Client second flow failed");
				sock.sendTerminatePacket();
				return;
			}

			EAPpldBasic ttlsEapPktThird = new EAPpldBasic();
			ttlsEapPktThird.code = EAPcode.EAP_CODE_RESPONSE;
			ttlsEapPktThird.identifier = sock.getReceivedEAPPacket().identifier;
			ttlsEapPktThird.type = testType;
			ttlsEapPktThird.EAPmethodPld = new EAPtypePldTtlsv0(msgSF);
			ttlsEapPktThird.isLengthSet = false;
			ttlsEapPktThird.toByteArray();

			sock.setEapMessage(ttlsEapPktThird);

			// get all EAP fragments
			tlsFullMsgLen = 0;
			cicleFlag = true;
			tlsMsgList.clear();
			while(cicleFlag) {
				res = sock.sendEAPpacket();
				if((res != 0) || (!sock.receivedData)) { 
					DebugInfo.printMessage(1,"!!! sendInitEAPpacket failed");
					sock.sendTerminatePacket();
					return;
				}
				recvEAP = sock.getReceivedEAPPacket();
				cicleFlag = false;

				if(recvEAP instanceof EAPpldBasic) {
					if(((EAPpldBasic) recvEAP).type == testType) {
						EAPtypePldTtlsv0 curpld = (EAPtypePldTtlsv0)((EAPpldBasic)recvEAP).EAPmethodPld;
						if(curpld.getEapTlsLBit()) {
							if(tlsFullMsgLen == 0)
								tlsFullMsgLen = curpld.tlsMessageLenght;
							else {
								DebugInfo.printMessage(1,"!!! Warning! tlsMessageLenght dublicate");
							}
						}
						tlsMsgList.add(curpld.tlsData);
						if(curpld.getEapTlsMBit()) {
							cicleFlag = true;
							ttlsEapPktThird.identifier = sock.getReceivedEAPPacket().identifier;
							ttlsEapPktThird.EAPmethodPld = new EAPtypePldTtlsv0();
							ttlsEapPktThird.isLengthSet = false;
							ttlsEapPktThird.toByteArray();
							sock.setEapMessage(ttlsEapPktThird);
						}
					}
				}
			}

			tlsRaw = EAPTlsFragmentation.combineTlsMessages(tlsMsgList);
			if(tlsRaw == null) {
				DebugInfo.printMessage(1,"!!! Error! tlsRaw Null pointer");
			} else if(tlsRaw.length != tlsFullMsgLen) {
				DebugInfo.printMessage(1,"!!! Error! tlsRaw length failed: " + tlsRaw.length + " must " + tlsFullMsgLen);
			} else {
				DebugInfo.printDumpWithLength(2, tlsRaw, "Get full tls messages");
			}
		
		
			// parse tls response		
			DebugInfo.printMessageWithSeparator(1,"parse tls response");
			
			resp = new TLSResponse(tlsRaw);
	   		resp.printResponse();
	    	if(resp.err != 0) {
				DebugInfo.printMessage(1,"!!! TLSResponse return error");
				sock.sendTerminatePacket();
				return;
	    	}

	    	try {
	    		ArrayList<RecordLayer> tlsResp = tlsClnt.parseTlsResponse(resp);	
	    		if(null == tlsResp  ||  tlsClnt.conSpec.disconnectWithSendAlert != -1) {
	    			DebugInfo.printMessage(1,"!!! TLSResponse return error");
	    			tlsClnt.conSpec.disconnect();
	    			sock.sendTerminatePacket();
	    			return;
	        	}
	    	} catch (Exception err) {
				DebugInfo.printMessage(1,"===Error! parseTlsResponse Exception: " + err);
				sock.sendTerminatePacket();
				return;
			}

// close socket		
		sock.sendTerminatePacket();
		sock.closeSocket();
	}

}

