package ru.ispras.telecomtesk.eap.EAPoverLAN;

import ru.ispras.telecomtesk.eap.BasicPayloadObject;
import ru.ispras.telecomtesk.eap.ConvertTypes;
import ru.ispras.telecomtesk.eap.DebugInfo;

public class EAPOLEthernetHeader extends BasicPayloadObject {
    public short type;               // 2 bytes
	public byte[] srcAddr = null; 
	public byte[] dstAddr = null;
	public BasicPayloadObject data = null;


	public EAPOLEthernetHeader() {
		srcAddr = new byte[6];
		dstAddr = new byte[6];
	}

	public EAPOLEthernetHeader(byte[] srcAddr, byte[] dstAddr, int type) {
		this.srcAddr = new byte[6];
		this.dstAddr = new byte[6];
		if((srcAddr != null)&&(srcAddr.length >= this.srcAddr.length)) 
			System.arraycopy(srcAddr, 0, this.srcAddr, 0, this.srcAddr.length);
		if((dstAddr != null)&&(dstAddr.length >= this.dstAddr.length)) 
			System.arraycopy(dstAddr, 0, this.dstAddr, 0, this.dstAddr.length);
		this.type = (short)type;
	}

	public int getSize() {
		int res = 2;

		if(srcAddr != null) 
			res += srcAddr.length;
		if(dstAddr != null) 
			res += dstAddr.length;
		if(data != null)
			res += data.getSize();

		return res;
	}

	public void setData(BasicPayloadObject data) {
		this.data = data;
	}

	public short getType() {
		return type;
	}

	public byte[] getSrcAddr() {
		return srcAddr;
	}

	public byte[] getDstAddr() {
		return dstAddr;
	}

	
	public int parseNetMessage() {
		int res = 0;
		byte[] data = this.net_msg;
		if(data == null) 
			res = 1;
		else if(data.length < 14) 
			res = 2;
		else {
			System.arraycopy(data, 0, this.dstAddr, 0, 6);
			System.arraycopy(data, 6, this.srcAddr, 0, 6);
			this.type = ConvertTypes.ntohs(data, 12);
			
			if(data.length > 14) {
				BasicPayloadObject pld;
				
				if(type == EAPOLEthernetType.ETHERNET_TYPE_EAP_OVER_LAN.getValue()) {
					pld = new EAPOLHeader();
				} else {
					pld = new BasicPayloadObject();
				}

				pld.net_msg = new byte[data.length - 14];
				System.arraycopy(data, 14, pld.net_msg, 0, pld.net_msg.length);
				int err = pld.parseNetMessage();
				DebugInfo.printMessage(2,"EAPOLEthernetHeader::parseNetMessage return " + res);
				this.setData(pld);
			}
		}
		
		return res;
	}

	public int toByteArray() {
		int pos = 0;
		// create byte array
		byte[] ba = new byte[getSize()];
		
		System.arraycopy(this.dstAddr, 0, ba, pos, 6);
		pos += 6;
		System.arraycopy(this.srcAddr, 0, ba, pos, 6);
		pos += 6;
		ba[pos++] = (byte)(type >>> 8);
		ba[pos++] = (byte)type;
		
		if(data != null) {
			if(data.net_msg == null) 
				data.toByteArray();
			System.arraycopy(data.net_msg, 0, ba, pos, data.net_msg.length);
		}

		this.net_msg = ba;
		return 0;
	}

	public void print() {
		DebugInfo.printMessage(1,"EthernetHeader:");
		DebugInfo.printDump(1, this.dstAddr, "  dstAddr:");
		DebugInfo.printDump(1, this.srcAddr, "  srcAddr:");
		DebugInfo.printMessage(1,"  type: " + EAPOLEthernetType.toString(type));
		if(data != null)
			this.data.print();
	}
}    
