package ru.ispras.telecomtesk.eap.EAPoverLAN;

import ru.ispras.telecomtesk.eap.BasicPayloadObject;
import ru.ispras.telecomtesk.eap.ConvertTypes;
import ru.ispras.telecomtesk.eap.DebugInfo;
import ru.ispras.telecomtesk.eap.EAPcode;
import ru.ispras.telecomtesk.eap.EAPheader;

public class EAPOLHeader extends BasicPayloadObject {
    public byte version;           
    public byte type;               
	public short length;    // length of EAP payload
	public BasicPayloadObject data = null;
	protected boolean isLengthSet = false;


	public EAPOLHeader() {
		version = 1;
	}

	public EAPOLHeader(EAPheader pld) {
		version = 1;
		type = EAPOLtype.EAPOL_TYPE_PACKET.getValue();
		this.data = pld;
	}

	public void setStartMessage() {
		type = EAPOLtype.EAPOL_TYPE_START.getValue();
		length = 0;
		isLengthSet = true;
		data = null;
	}

	public void setLength() {
		if(data != null)
			length = (short)data.getSize();
		else
			length = 0;
		isLengthSet = true;
	}

	public void setLength(int len) {
		length = (short)len;
		isLengthSet = true;
	}

	public int getSize() {
		int res = 4;

		if(data != null)
			res += (short)data.getSize();

		return res;
	}

	public void setData(BasicPayloadObject data) {
		this.data = data;
	}

	public int parseNetMessage() {
		int res = 0;
		byte[] data = this.net_msg;
		if(data == null) 
			res = 1;
		else if(data.length < 4) 
			res = 2;
		else {
			this.version = data[0];
			this.type = data[1];
			this.length = ConvertTypes.ntohs(data, 2);
			
			if(data.length > 4) {
				BasicPayloadObject pld;
				
				if(type == EAPOLtype.EAPOL_TYPE_PACKET.getValue()) {
					pld = EAPheader.createEAPheader(EAPcode.valueOf(data[4]));
				} else {
					pld = new BasicPayloadObject();
				}

				pld.net_msg = new byte[data.length - 4];
				System.arraycopy(data, 4, pld.net_msg, 0, pld.net_msg.length);
				int err = pld.parseNetMessage();
				DebugInfo.printMessage(2,"EAPOLHeader::parseNetMessage return " + res);
				this.setData(pld);
			}
		}
		
		return res;
	}

	public int toByteArray() {
		int pos = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		ba[pos++] = (byte)version;
		ba[pos++] = (byte)type;
		ba[pos++] = (byte)(length >>> 8);
		ba[pos++] = (byte)length;
		
		if(data != null) {
			if(data.net_msg == null) 
				data.toByteArray();
			System.arraycopy(data.net_msg, 0, ba, pos, data.net_msg.length);
		}

		this.net_msg = ba;
		return 0;
	}

	public void print() {
		DebugInfo.printMessage(1,"EAP over LAN Header:");
		DebugInfo.printMessage(1,"  version: " + this.version);
		DebugInfo.printMessage(1,"  type: " + EAPOLtype.toString(this.type));
		DebugInfo.printMessage(1,"  length: " + this.length);
		if(data != null)
			this.data.print();
	}
}    
