package ru.ispras.telecomtesk.eap.EAPoverRadius;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import ru.ispras.telecomtesk.eap.*;
import ru.ispras.telecomtesk.eap.DebugInfo;

public class EAPRadiusSocket extends BasicPayloadObject {
	public int destination_port; 

	public static byte[] sendData = new byte[65000];
	public static int sendDataLength = 0;
	public static byte[] receiveData = new byte[65000];
	public static int receiveDataLength = 0;

	public InetAddress remoteIPAddress;
	public byte[] remoteHostRaw;

	private DatagramSocket clientSocket;
	private DatagramPacket sendPacket;
	private DatagramPacket receivePacket;
	private int socketTimeout;

	public String radiusSharedSecret;
	private byte[] SharedSecret;
	public String userName;

	public int radiusPktId;
	public int EAPPktId;
	public int counterEAPPackets; // without EAP init messages
	public int counterEAPTerminatePackets;
	public boolean receivedData;

	EAPRadiusHeader radiusPkt;
	EAPRadiusAttributes radAttr_name;
	EAPRadiusAttributes radAttr_auth;

	EAPRadiusHeader	receivedRadPkt;
	EAPheader receivedEAPPkt;
    
	EAPpldBasic eapInitMsg;
	EAPRadiusAttributes radAttrEAPinit;
	EAPheader eapMsg;
	EAPRadiusAttributes radAttrEAP;
	EAPRadiusAttributes radAttr_state;


	public EAPRadiusSocket(String sharedSecret, String userName, byte[] remoteHostAddr) {
		// default destination port = 1812
		this(1812, sharedSecret, userName, remoteHostAddr);
	}

	public EAPRadiusSocket(int port, String sharedSecret, String userName, byte[] remoteHostAddr) {
		this.destination_port = port;
		this.radiusSharedSecret = sharedSecret;
		this.userName = userName;
		this.remoteHostRaw = remoteHostAddr;
		EAPRadiusSocketInit();
	}

	public int EAPRadiusSocketInit() {
		int res = 0;
		try {
			socketTimeout = 3000;
			clientSocket = new DatagramSocket();
			clientSocket.setSoTimeout(socketTimeout);
			remoteIPAddress = InetAddress.getByAddress(remoteHostRaw);
			receivePacket = new DatagramPacket(receiveData, receiveData.length);
		} catch (Exception err) {
			DebugInfo.printMessage(1,"!!! ERROR: EAPRadiusSocketInit failed: " + err);
			res = 1;
		}
		
		radiusPktId = 10;
		EAPPktId = 10;
		counterEAPPackets = 1;
		counterEAPTerminatePackets = 1;
		receivedData = false;

		SharedSecret = radiusSharedSecret.getBytes();
		
	    radAttr_name = new EAPRadiusAttributes(1);
		radAttr_name.value = userName.getBytes();

	    radAttr_auth = new EAPRadiusAttributes(80);

		radiusPkt = new EAPRadiusHeader(1,radiusPktId);
		
	    eapInitMsg = new EAPpldResponse(EAPPktId, EAPtype.EAP_TYPE_IDENTITY);
	    eapInitMsg.EAPmethodPld = new EAPtypePldID(userName.getBytes());
	    eapInitMsg.toByteArray();
	    radAttrEAPinit = new EAPRadiusAttributes(79,eapInitMsg.net_msg);

		return res;
	}

	public void setSocketTimeout(int tm) {
		this.socketTimeout = tm;
		try {
			clientSocket.setSoTimeout(socketTimeout);
		} catch (Exception err) {
			DebugInfo.printMessage(1,"!!! ERROR: EAPRadiusSocket::setSocketTimeout: " + err);
		}
	}

	public void setFirstPacketsID(int radiusPktId, int eapPktId) {
		this.radiusPktId = radiusPktId;
		this.EAPPktId = eapPktId;
	}

	public void setEapMessage(EAPheader eapMsg) {
		this.eapMsg = eapMsg;
	}

	public EAPRadiusHeader getReceivedRadiusPacket() {
		return this.receivedRadPkt;
	}

	public EAPheader getReceivedEAPPacket() {
		return this.receivedEAPPkt;
	}

	public int parseRecvMessage() {
		int res1, res2 = 0;
		
		DebugInfo.printDump(2, receivePacket.getData(), 0, receivePacket.getLength() , "EAPRadiusSocket::parseRecvMessage: received packet");

		// parse received message
		receivedRadPkt = new EAPRadiusHeader();
		receivedRadPkt.net_msg = new byte[receivePacket.getLength()];
		System.arraycopy(receivePacket.getData(), 0, receivedRadPkt.net_msg, 0, receivePacket.getLength());
		res1 = receivedRadPkt.parseNetMessage();
		DebugInfo.printMessage(1,"EAPRadiusSocket::parseRecvMessage: packet was parsed, res: " + res1);
		receivedRadPkt.print();
		if(radiusPkt.identifier != receivedRadPkt.identifier) {
			DebugInfo.printMessage(1,"!!! Warning: EAPRadiusSocket::parseRecvMessage: received identifier not equal sended one");
			res1 += 2;
		}
		radAttr_state = EAPRadiusAttributes.cloneAttribute(receivedRadPkt.getAttributeByType(24));

		// parse EAP received message
		receivedEAPPkt = null;
		byte[] eapRawMsg = receivedRadPkt.getEAPMessage();
		if(eapRawMsg != null) {
			DebugInfo.printMessage(1,"EAPRadiusSocket::parseRecvMessage: EAP message found, try parse");
			if(eapRawMsg.length > 4)
				receivedEAPPkt = new EAPpldBasic();
			else
				receivedEAPPkt = new EAPheader();
			
			receivedEAPPkt.net_msg = eapRawMsg;
			res2 = receivedEAPPkt.parseNetMessage(); 
			DebugInfo.printMessage(1,"EAPRadiusSocket::parseRecvMessage: EAP packet was parsed, res: " + res2);
			receivedEAPPkt.print("   ");
		}
		
		return (res1 + res2);
	}
	

	// max radius attribute size = 255 byte: type (1 byte) + length (1 byte) + data (253 bytes)
	public void cutLargeEAPpacket() throws Exception {
		int len = eapMsg.net_msg.length;
		int curpos = 0;
		int MAXRADIUSATTRLENGTH = 253;
		byte[] data = null;
		
		DebugInfo.printMessage(1,"EAPRadiusSocket::cutLargeEAPpacket");

		while(curpos < len) {
			if(len - curpos >= MAXRADIUSATTRLENGTH) {
				data = new byte[MAXRADIUSATTRLENGTH];
				System.arraycopy(eapMsg.net_msg, curpos, data, 0, MAXRADIUSATTRLENGTH);
				curpos += MAXRADIUSATTRLENGTH;
			} else {
				data = new byte[len - curpos];
				System.arraycopy(eapMsg.net_msg, curpos, data, 0, data.length);
				curpos = len;
			}
			radAttrEAP = new EAPRadiusAttributes(79,data);
			radiusPkt.addAttribute(radAttrEAP);
		}
	}

	public int sendInitEAPpacket() {
		int res = 0;
		
		DebugInfo.printMessage(1,"EAPRadiusSocket::sendInitEAPpacket: try create init EAP packet");

		if(radAttrEAPinit.net_msg != null)
			radAttrEAPinit.net_msg[3] = (byte)(EAPPktId++); // change EAP 'identifier' field
		
		radiusPkt.ReInit(1,radiusPktId++);
		radiusPkt.addAttribute(radAttr_name);
		radiusPkt.addAttribute(radAttr_auth);
		radiusPkt.addAttribute(radAttrEAPinit);

		radAttr_auth.setMessageAuthenticator(radiusPkt,SharedSecret);
		radiusPkt.toByteArray();
		
		sendPacket();
		res = waitAnswerPacket();

		return res;
	}

	public int sendEAPpacket() {
		int res = 0;
		
		DebugInfo.printMessage(1,"EAPRadiusSocket::sendEAPpacket: try create EAP packet");
		
		radiusPkt.ReInit(1,radiusPktId++);
//		radiusPkt.addAttribute(radAttr_name);
		if((radAttr_state != null) && (radAttr_state.value != null)) {
			DebugInfo.printMessage(1,"sendEAPpacket: State Attribute found");
			radiusPkt.addAttribute(radAttr_state);
		}
		radiusPkt.addAttribute(radAttr_auth);

		try {
			cutLargeEAPpacket();
		} catch (Exception err) {
			DebugInfo.printMessage(1,"===Error! EAPRadiusSocket::cutLargeEAPpacket: " + err);
			return 1;
		}

		radAttr_auth.setMessageAuthenticator(radiusPkt,SharedSecret);
		radiusPkt.toByteArray();
		
		DebugInfo.printMessage(1,"EAPRadiusSocket::sendEAPpacket: radius packet for send: ");
		radiusPkt.print();
		
		DebugInfo.printMessage(1,"EAPRadiusSocket::sendEAPpacket: EAP packet for send: ");
		eapMsg.print("   ");

		sendPacket();
		counterEAPPackets++;
		res = waitAnswerPacket();

		return res;
	}

	public int sendTerminatePacket() {
		DebugInfo.printMessage(1,"EAPRadiusSocket::sendTerminatePacket: try create terminate packet");
		EAPRadiusAttributes stateRadiusAttr = receivedRadPkt.getAttributeByType(24);
		if((stateRadiusAttr != null) && (stateRadiusAttr.value != null)) {
			DebugInfo.printMessage(3,"EAPRadiusSocket::sendTerminatePacket: State Attribute found");
			radiusPkt.ReInit(1,radiusPktId++);
			radiusPkt.addAttribute(stateRadiusAttr);
			radiusPkt.addAttribute(radAttr_auth);
			radAttr_auth.setMessageAuthenticator(radiusPkt,SharedSecret);
			radiusPkt.toByteArray();
			sendPacket();
			counterEAPTerminatePackets++;
			waitAnswerPacket();
		}
		
		return 0;
	}
	

	void sendPacket() {
		DebugInfo.printDump(2, radiusPkt.net_msg, "EAPRadiusSocket::sendPacket: Radius packet");
		DebugInfo.printMessage(1,"EAPRadiusSocket::sendPacket: Sending Radius packet");
		sendPacket = new DatagramPacket(radiusPkt.net_msg, radiusPkt.net_msg.length, remoteIPAddress, destination_port);
		radiusPkt.isLengthSet = false;
		try {
			clientSocket.send(sendPacket);
		} catch (IOException err) {
			DebugInfo.printMessage(1,"!!!ERROR, EAPRadiusSocket::sendPacket: send packet: " + err);
		}
		DebugInfo.printMessage(1,"EAPRadiusSocket::sendPacket: packet has been sended");
	}
	
	int waitAnswerPacket() {
		int res = 0;
		receivedData = false;
		
		DebugInfo.printMessage(1,"EAPRadiusSocket::waitAnswerPacket");

		try {
			clientSocket.receive(receivePacket);
			receivedData = true;
			DebugInfo.printMessage(1,"EAPRadiusSocket::waitAnswerPacket: packet has been received, length: " + receivePacket.getLength());
			res = parseRecvMessage();
		} catch (IOException err) {
			DebugInfo.printMessage(1,"EAPRadiusSocket::waitAnswerPacket: receive packet: " + err);
			DebugInfo.printMessage(1,"!!! Warning: EAPRadiusSocket::waitAnswerPacket: not receive answer");
			res = 2;
		}
		
		return res;
	}

	public void closeSocket() {
		DebugInfo.printMessage(1,"EAPRadiusSocket::closeSocket");
		clientSocket.close();
		DebugInfo.printMessage(1,"EAPRadiusSocket::closeSocket: socket closed");
	}
	
}    
