package ru.ispras.telecomtesk.eap;

import ru.ispras.telecomtesk.eap.EAPPEAP.EAPtypePldPEAP;
import ru.ispras.telecomtesk.eap.EAPSIM.EAPtypePldSim;
import ru.ispras.telecomtesk.eap.EAPTLS.EAPtypePldTls;
import ru.ispras.telecomtesk.eap.EAPTTLSv0.EAPtypePldTtlsv0;

// EAP packet

public class EAPpldBasic extends EAPheader {    //  EAP Request and Response Type
	public EAPtype type;
	public BasicPayloadObject EAPmethodPld = null; // EAP method payload


	public int getSize() {
		int i = 5;
		if(pldData != null) 
			i += pldData.length;
		if(EAPmethodPld != null) {
			EAPmethodPld.toByteArray();
			i += EAPmethodPld.net_msg.length;
		} 
		return i;
	}

	public void setLength() {
		length = (short)getSize();
		isLengthSet = true;
	}

	public void setLength(int len) {
		length = (short)len;
		isLengthSet = true;
	}

	public int parseNetMessage() {
		int res = 0;
		byte[] data = this.net_msg;
		if(data == null) 
			res = 1;
		else if(data.length < 5) 
			res = 2;
		else {
			this.code = EAPcode.valueOf(data[0]);
			this.identifier = data[1];
			this.length = (short)(data[2] & 0xFF);
			this.length <<= 8;
			this.length |= (data[3] & 0xFF);
			this.type = EAPtype.valueOf(data[4]);
			
			if(data.length > 5) {
				switch (this.type) {
				case EAP_TYPE_MD5CHALLENGE: EAPmethodPld = new EAPtypePldMd5();break;
				case EAP_TYPE_SIM: EAPmethodPld = new EAPtypePldSim();break;
				case EAP_TYPE_TLS: EAPmethodPld = new EAPtypePldTls();break;
				case EAP_TYPE_TTLSv0: EAPmethodPld = new EAPtypePldTtlsv0();break;
				case EAP_TYPE_PEAP: EAPmethodPld = new EAPtypePldPEAP();break;
				case EAP_TYPE_MSCHAP: EAPmethodPld = new EAPtypePldMSCHAP();break;
				default: EAPmethodPld = new BasicPayloadObject();
				}

				EAPmethodPld.net_msg = new byte[data.length - 5];
				System.arraycopy(data, 5, EAPmethodPld.net_msg, 0, data.length - 5);
				int res2 = EAPmethodPld.parseNetMessage();
				DebugInfo.printMessage(1,"EAPpldBasic::parseNetMessage: " + data[4]+ " " + this.type.toString() + " parsed: " + res2);
			}
		}
		
		return res;
	}

	public int toByteArray() {
		int i = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		ba[i++] = code.getValue();
		ba[i++] = identifier;
		ba[i++] = (byte)(length >>> 8);
		ba[i++] = (byte)length;
		ba[i++] = (byte)(type.getValue());
		if(pldData != null) {
			System.arraycopy(pldData, 0, ba, i, pldData.length);
		}
		if(EAPmethodPld != null) {
			if(EAPmethodPld.net_msg == null)
				EAPmethodPld.toByteArray();
			System.arraycopy(EAPmethodPld.net_msg, 0, ba, i, EAPmethodPld.net_msg.length);
		}
		net_msg = ba;
		return 0;
	}

	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "EAP packet:");
		DebugInfo.printMessage(1,pref + "  code: " + this.code);
		DebugInfo.printMessage(1,pref + "  identifier: " + this.identifier);
		DebugInfo.printMessage(1,pref + "  length: " + this.length);
		DebugInfo.printMessage(1,pref + "  type: " + this.type);
		if(EAPmethodPld != null) {
			EAPmethodPld.print(pref + "  ");
		}
	}
}