package ru.ispras.telecomtesk.eap;

import java.security.MessageDigest;
import java.util.Random;


public class EAPtypePldMSCHAP extends BasicPayloadObject {    
	public byte  code;
	public byte  id;
	public int  length;
	public int  valueSize;
	public byte[] value = null; 
	public byte[] name = null; 
	protected boolean isLengthSet = false;


	public EAPtypePldMSCHAP() {
		code = 0; id = 0;
	}

	public EAPtypePldMSCHAP(int code, int id) {
		this.code = (byte)code; this.id = (byte)id;
	}

	public int getSize() {
		// code + id + length + valueSize = 1 + 1 + 2 + 1 = 5
        int i=5;
		if(value != null) 
			i += value.length;
		if(name != null) 
			i += name.length;
		return i;
	}

	public void setLength() {
		this.length = getSize();
		if(value != null) 
			valueSize = value.length;
		else
			valueSize = 0;
		
		isLengthSet = true;
	}

	public void setLength(int len) {
		valueSize = len;
		isLengthSet = true;
	}

	public byte[] ChallengeHash(byte[] chlg_value, String name) throws Exception {
		MessageDigest mobj;
		byte[] data = null;
		byte[] res = null;
		
		if((chlg_value != null) && (chlg_value.length >= 16)
				&& (value != null) && (value.length >= 16)
				&& (name != null)) {
			mobj = MessageDigest.getInstance("SHA1");
			mobj.update(value, 0, 16);
			mobj.update(chlg_value);
			mobj.update(name.getBytes());
			data = mobj.digest();
			DebugInfo.printDump(3, data, "EAPtypePldMSCHAP::ChallengeHash: ");

			if(data != null) {
				res = new byte[8];
				System.arraycopy(data, 0, res, 0, res.length);
			}
		}
		
		return res;
	}
		
	public void setRandomNTvalue() {
		value = new byte[49];
		valueSize = value.length;
		Random rndObj = new Random(System.currentTimeMillis());
		rndObj.nextBytes(value);
		for(int i=0;i<8;i++) // RESERVED field
			value[i+16] = 0;
		value[48] = 0;  // FLAG field
		DebugInfo.printDump(3, value, "EAPtypePldMSCHAP::setRandomNTvalue: ");
	}
	
	public int setNTvalue(String name, byte[] chlg_value, String pwd) throws Exception {
		MessageDigest mobj;
		byte[] data = null;
		int res = 0;
		
		setRandomNTvalue();
		
		if(chlg_value != null) {
			byte[] clHash = ChallengeHash(chlg_value, name);
			// NtPasswordHash()
			// ChallengeResponse()
			// DesEncrypt()
			
		} else {
			res = 1;
			DebugInfo.printMessage(3,"EAPtypePldMSCHAP::setNTvalue: null pointer");
		}

		return res;
	}

	public int parseNetMessage() {
		int res = 0;
		byte[] data = this.net_msg;
		if(data == null) 
			res = 1;
		else if(data.length < 5) 
			res = 2;
		else {
			this.code = data[0];
			this.id = data[1];
			this.length = ConvertTypes.ntoh(data,2,2);
			this.valueSize = data[4];
			int len = 0;
			int pos = 5;
			
			if(data.length > pos) {
				if(valueSize > 0) {
					len = data.length - pos > valueSize ? valueSize : data.length - pos;
					this.value = new byte[len];
					System.arraycopy(this.net_msg, pos, this.value, 0, len);
				}
				pos += len;
				if (data.length > pos) {
					this.name = new byte[data.length - pos];
					System.arraycopy(data, pos, this.name, 0, this.name.length);
				}
			}
		}
		
		return res;
	}

	public int toByteArray() {
		int i = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		
		ba[i++] = code;
		ba[i++] = id;
		byte[] ba_len = ConvertTypes.hton((short)length);
		System.arraycopy(ba_len, 0, ba, i, ba_len.length);
		i += ba_len.length;
		ba[i++] = (byte)valueSize;
		if(value != null) {
			System.arraycopy(value, 0, ba, i, value.length);
		}
		if(name != null) {
			System.arraycopy(name, 0, ba, i, name.length);
		}
		net_msg = ba;
		return 0;
	}

	public void print(String pref) {
		DebugInfo.printMessage(1,pref + "EAP MSCHAP payload:");
		DebugInfo.printMessage(1,pref + "  code: " + this.code);
		DebugInfo.printMessage(1,pref + "  id: " + this.id);
		DebugInfo.printMessage(1,pref + "  length: " + this.length);
		DebugInfo.printMessage(1,pref + "  valueSize: " + this.valueSize);
		DebugInfo.printDump(1, this.value, pref + "  value:");
//		if(this.code == 4) { // failed
			DebugInfo.printMessage(1, pref + "  value ASCII:" + String.valueOf(this.value));
//		}
		DebugInfo.printDump(1, this.name, pref + "  name:");
	}
}