package ru.ispras.telecomtesk.eap.pcap;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.pcap4j.core.PacketListener;
import org.pcap4j.core.PcapHandle;
import org.pcap4j.core.PcapNetworkInterface;
import org.pcap4j.core.RawPacketListener;
import org.pcap4j.core.PcapNetworkInterface.PromiscuousMode;
import org.pcap4j.core.Pcaps;
import org.pcap4j.packet.Packet;
import org.pcap4j.util.LinkLayerAddress;

import ru.ispras.telecomtesk.eap.DebugInfo;

public class jPcap
{
	public PcapHandle sendSocketHandle;
	public PcapHandle capSocketHandle;
	public byte[] receivedPacket = null;
  	ExecutorService pool;
  	Task t;
  	RawPacketListener listener;
	int snaplen = 65536;
	PromiscuousMode mode = PromiscuousMode.NONPROMISCUOUS;
	int timeout = 10;
	byte[] localMAC;
	
    public  void startCapture() throws Exception {
        System.out.println("jPcap::startCapture");
    	List<PcapNetworkInterface> devs = Pcaps.findAllDevs();
    	if(devs.size() != 0) {
    	   for(PcapNetworkInterface nif : devs) {
//    	      System.out.println(nif.toString());
    		   if(nif.getName().equals("\\Device\\NPF_{F6D24150-82BB-4CC7-865C-E17462202487}")) {
     	 		  System.out.println("============");
    		      System.out.println(nif.toString());
    			   
       		      List<LinkLayerAddress> lls = nif.getLinkLayerAddresses();
       		      if(lls.size() != 0) {
       		    	  localMAC = lls.get(0).getAddress();
       		    	  if(localMAC.length != 6)
	            			DebugInfo.printDump(1, localMAC, "!!!Error jPcap::startCapture bad local MAC");

       		    	  for(LinkLayerAddress lla : lls) {
       		    		  System.out.println(lla.toString());
       		    	  }
       		      }
       		      
       		      System.out.println("---openlive---");
       		      sendSocketHandle = nif.openLive(snaplen, mode, timeout);
       		      capSocketHandle = nif.openLive(snaplen, mode, timeout);
       		      pool = Executors.newSingleThreadExecutor(); 

       		      listener = new RawPacketListener() {
			            @Override
			            public void gotPacket(byte[] packet) {
			            	if((packet != null)&&(packet.length >= 14)
			            		&&(packet[12] == (byte)0x88)&&(packet[13] == (byte)0x8e)) {
			            			int i;
			            			for(i=0;i<6;i++) {
			            				if(localMAC[i] != packet[i])
			            					break;
			            			}
			            			if(i==6) {
			            				receivedPacket = new byte[packet.length];
			            				System.arraycopy(packet, 0, receivedPacket, 0, receivedPacket.length);
				            			DebugInfo.printMessage(1,"=============");			
				            			DebugInfo.printDump(1, receivedPacket, "jPcap: capture our packet");
				            			DebugInfo.printMessage(1,"=============");			
			            			} else {
				            			DebugInfo.printMessage(1,"----------");			
				            			DebugInfo.printDump(1, packet, "jPcap: captured packet");
				            			DebugInfo.printMessage(1,"----------");						            				
			            			}
			            	}
			            }
			      };

			      t = new Task(capSocketHandle, listener);
			      pool.execute(t);
    		   }
    	   }
    	}
    			    
        System.out.println("started");
    }
    
    public void sendPacket(byte[] sendingPacket) throws Exception {
    	DebugInfo.printDump(1, sendingPacket, "jPcap: sending packet");
    	sendSocketHandle.sendPacket(sendingPacket);

      	System.out.println("capt " + sendSocketHandle.getStats().getNumPacketsCaptured()
      	    		  + " recv " + sendSocketHandle.getStats().getNumPacketsReceived()
      	    		  + " drop " + sendSocketHandle.getStats().getNumPacketsDropped()
      	    		  + " dropif " + sendSocketHandle.getStats().getNumPacketsDroppedByIf());

      	System.out.println("============");
    }
    
    public  void stopCapture() {
        System.out.println("jPcap::stopCapture");
        if (sendSocketHandle != null && sendSocketHandle.isOpen()) {
        	sendSocketHandle.close();
        }
  	  	if (pool != null && !pool.isShutdown()) {
  	  		pool.shutdown();
        }
  	  	if (capSocketHandle != null && capSocketHandle.isOpen()) {
  	  		try {
  	  			capSocketHandle.breakLoop();
  	  		} catch (Exception e) {
  				DebugInfo.printMessage(1,"!!!Error. jPcap::stopCapture " + e.toString());			
  			}

  	  		capSocketHandle.close();
  	  	}
        System.out.println("stoped");
    }
    
    
    private static class Task implements Runnable {

        private PcapHandle handle;
        private RawPacketListener listener;

        public Task(PcapHandle handle, RawPacketListener listener) {
          this.handle = handle;
          this.listener = listener;
        }

        @Override
        public void run() {
          try {
            handle.loop(100, listener);
          } catch (Exception e) {
            e.printStackTrace();
          } 
        }

      }     
}

// local mac b8:86:87:c7:c5:27
// ap mac 00:3a:9a:d1:6d:e0