
// test EAPSIM first message

package ru.ispras.telecomtesk.eap;

import java.io.*;
import java.math.BigInteger;
import java.net.*;
import java.util.ArrayList;
import java.security.*;

import javax.crypto.interfaces.DHPublicKey;

import ru.ispras.telecomtesk.eap.EAPoverRadius.*;
import ru.ispras.telecomtesk.eap.EAPSIM.*;
import ru.ispras.telecomtesk.eap.EAPSIM.Attributes.*;



public class testEAPSIM1
{
    public static void main(String args[]) throws Exception {
    	testEAPSIM1Run test1 = new testEAPSIM1Run();
    	test1.runtest();
    }
}

class testEAPSIM1Run
{
	public static int destination_port = 1812; 

	public static byte[] sendData = new byte[65000];
	public static int sendDataLength = 0;
	public static byte[] receiveData = new byte[65000];
	public static int receiveDataLength = 0;

	public static InetAddress remoteIPAddress;
//	public static String remoteHost = "192.168.56.101";
	public static byte[] remoteHostRaw = {(byte)(192),(byte)(168),(byte)(56),(byte)(101)};
//	public static byte[] localHostRaw = {(byte)(192),(byte)(168),(byte)(160),(byte)(1)};
	public static byte[] localHostRaw = {(byte)(127),(byte)(0),(byte)(0),(byte)(1)};

	DatagramSocket clientSocket;
	DatagramPacket sendPacket;
	DatagramPacket receivePacket;

	byte[] SharedSecret;
	byte[] UserPwd;
	String userName = "eapsimtest";
	String SharedSecretStr = "testing123";
	String UserPwdStr = "123456";

	int numberSendWrongPkts = 1;
	int numberSendRightPkts = 1;

	int counterRadiusPkt = 1;
	int counterEAPPkt = 1;
	EAPRadiusHeader radiusPkt;
	EAPRadiusAttributes radAttr_name;
	EAPRadiusAttributes radAttr_auth;

	EAPRadiusHeader	receivedRadPkt;
	EAPheader receivedEAPPkt;
    
	EAPpldBasic eapInitMsg;
	EAPRadiusAttributes radAttr_eap_init;
	EAPpldBasic eapSecondMsg;
	EAPRadiusAttributes radAttr_eap_second;
	EAPRadiusAttributes radAttr_state;

	int counterWrongPackets1 = 0;
	int counterWrongPackets2 = 0;
    
	byte[] testRandomData = {1,2,3,4,5,6,7,8};
	byte[] testRandomMD5Data = {16,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16};
	byte[] testRandomExpandedData = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
	byte[] testExpandedNak = {(byte)254,0,0,0,0,0,0,4,(byte)254,0,0,0,0,0,0,0};
	byte[] testExpandedData = {0,0,0,0,0,0,(byte)254,0,0,0,0,0,0,1};

	TestAttributesSequence1 tstAttrLst = null;
	EAPtypePldSim eapSim = null;

	public void runtest() throws Exception {
		DebugInfo.debugLevel = 2;
		DebugInfo.maxDumpSize = 1000;

		clientSocket = new DatagramSocket();
		clientSocket.setSoTimeout(3000);
		remoteIPAddress = InetAddress.getByAddress(remoteHostRaw);
		receivePacket = new DatagramPacket(receiveData, receiveData.length);

		SharedSecret = SharedSecretStr.getBytes();
		UserPwd = UserPwdStr.getBytes();

		radiusPkt = new EAPRadiusHeader(1,counterRadiusPkt);
		
	    radAttr_name = new EAPRadiusAttributes(1);
		radAttr_name.value = userName.getBytes();

	    radAttr_auth = new EAPRadiusAttributes(80);
		
	    eapInitMsg = new EAPpldResponse(counterEAPPkt++, EAPtype.EAP_TYPE_IDENTITY);
	    eapInitMsg.EAPmethodPld = new EAPtypePldID(userName.getBytes());
	    eapInitMsg.toByteArray();
		radAttr_eap_init = new EAPRadiusAttributes(79,eapInitMsg.net_msg);

		tstAttrLst = new TestAttributesSequence1(userName);
		eapSim = new EAPtypePldSim(EAPSIMsubtype.EAPSIM_SUBTYPE_START);
		
		
		int eapSimInd;
		int eapSimAttrInd;
		int result;


	    for (eapSimInd = 1; eapSimInd <= 5; eapSimInd++) {
		    for (eapSimAttrInd = 0; eapSimAttrInd < tstAttrLst.getSize(); eapSimAttrInd++) {
			DebugInfo.printMessage(1,"eapSimInd, eapSimAttrInd: " + eapSimInd + ", " + eapSimAttrInd);
			tstAttrLst.setTestAttrList(eapSimInd, eapSimAttrInd, eapSim);
	    		result = sendPacketsSequence();
	    		if(result > 1) {
	    			clientSocket.close();
	    			DebugInfo.printMessage(1,"socket closed, stop test");
	    			return;
			    }
		    }
	    }

    
		DebugInfo.printMessage(1,"run: number of received packets: on wrong " + counterWrongPackets1 + ", on terminate " + counterWrongPackets2);
		counterWrongPackets1 = 0; counterWrongPackets2 = 0;
	

		// close socket		
		clientSocket.close();
		DebugInfo.printMessage(1,"socket closed");
	}


	
	int sendPacketsSequence() {
		int res = 0;
		
		// send init ID packet
		DebugInfo.printMessage(1,"\n================================================");
		DebugInfo.printMessage(1,"sendPacketsSequence: try create init ID packet " + numberSendRightPkts++);
		res = sendInitEAPpacket();

		if(receivedEAPPkt != null) {
			try {
				eapSecondMsg = new EAPpldResponse(receivedEAPPkt.identifier, EAPtype.EAP_TYPE_SIM);
				radAttr_state = EAPRadiusAttributes.cloneAttribute(receivedRadPkt.getAttributeByType(24));
			} catch (Exception err) {
				DebugInfo.printMessage(1,"!!! ERROR, sendPacketsSequence: create EAP packet2: " + err);
				return 2;
			}
			
		}
		
		// send second packet
		DebugInfo.printMessage(1,"-------------------------------");
		DebugInfo.printMessage(1,"sendPacketsSequence: try create second packet " + numberSendWrongPkts++);

		eapSecondMsg.EAPmethodPld = eapSim;
		eapSecondMsg.toByteArray();
		radAttr_eap_second = new EAPRadiusAttributes(79,eapSecondMsg.net_msg);
		
		DebugInfo.printMessage(1,"sendPacketsSequence: try send second packet");
		res = sendEAPpacket();

		return res;	
	}
	

	int parseRecvMessage() {
		int res1, res2 = 0;
		
		DebugInfo.printDump(2, receivePacket.getData(), 0, receivePacket.getLength() , "received packet");

		// parse received message
		receivedRadPkt = new EAPRadiusHeader();
		receivedRadPkt.net_msg = new byte[receivePacket.getLength()];
		System.arraycopy(receivePacket.getData(), 0, receivedRadPkt.net_msg, 0, receivePacket.getLength());
		res1 = receivedRadPkt.parseNetMessage();
		DebugInfo.printMessage(1,"packet was parsed, res: " + res1);
		receivedRadPkt.print();
		if(radiusPkt.identifier != receivedRadPkt.identifier) {
			DebugInfo.printMessage(1,"!!! Warning: received identifier not equal sended one");
			res1 += 2;
		}

		// parse EAP received message
		receivedEAPPkt = null;
		EAPRadiusAttributes eapRadAttr = receivedRadPkt.getAttributeByType(79);
		if((eapRadAttr != null) && (eapRadAttr.value != null)) {
			DebugInfo.printMessage(1,"EAP message found, try parse");
			if(eapRadAttr.value.length > 4)
				receivedEAPPkt = new EAPpldBasic();
			else
				receivedEAPPkt = new EAPheader();
			
			receivedEAPPkt.net_msg = new byte[eapRadAttr.value.length];
			System.arraycopy(eapRadAttr.value, 0, receivedEAPPkt.net_msg, 0, eapRadAttr.value.length);
			res2 = receivedEAPPkt.parseNetMessage(); 
			DebugInfo.printMessage(1,"EAP packet was parsed, res: " + res2);
			receivedEAPPkt.print("   ");
		}
		
		return (res1 + res2);
	}
	

	int sendInitEAPpacket() {
		int res = 0;
		
		if(radAttr_eap_init.net_msg != null)
			radAttr_eap_init.net_msg[3] = (byte)(counterEAPPkt++); // change EAP 'identifier' field
		
		radiusPkt.ReInit(1,counterRadiusPkt++);
		radiusPkt.addAttribute(radAttr_name);
		radiusPkt.addAttribute(radAttr_auth);
		radiusPkt.addAttribute(radAttr_eap_init);

		radAttr_auth.setMessageAuthenticator(radiusPkt,SharedSecret);
		radiusPkt.toByteArray();
		
		sendPacket();

		try {
			clientSocket.receive(receivePacket);
			DebugInfo.printMessage(1,"sendPacketsSequence: packet2 has been received, length: " + receivePacket.getLength());
			res = parseRecvMessage();
			if(receivedEAPPkt == null)
				res = 2;
		} catch (IOException err) {
			DebugInfo.printMessage(1,"sendPacketsSequence: receive packet2: " + err);
			DebugInfo.printMessage(1,"!!! ERROR: sendPacketsSequence: not receive answer");
			res = 2;
		}
		
		return res;
	}

	int sendEAPpacket() {
		int res = 0;
		
		radiusPkt.ReInit(1,counterRadiusPkt++);
//		radiusPkt.addAttribute(radAttr_name);
		if((radAttr_state != null) && (radAttr_state.value != null)) {
			DebugInfo.printMessage(1,"sendEAPpacket: State Attribute found");
			radiusPkt.addAttribute(radAttr_state);
		}
		radiusPkt.addAttribute(radAttr_auth);
		radiusPkt.addAttribute(radAttr_eap_second);

		radAttr_auth.setMessageAuthenticator(radiusPkt,SharedSecret);
		radiusPkt.toByteArray();
		
		DebugInfo.printMessage(1,"radius packet for send: ");
		radiusPkt.print();
		
		DebugInfo.printMessage(1,"EAP packet for send: ");
		eapSecondMsg.print("   ");

		sendPacket();

		try {
			clientSocket.receive(receivePacket);
			counterWrongPackets1++;
			DebugInfo.printMessage(1,"sendEAPpacket: packet2 has been received, length: " + receivePacket.getLength());
			res = parseRecvMessage();
			sendTerminatePacket();
		} catch (IOException err) {
			DebugInfo.printMessage(1,"sendEAPpacket: receive packet2: " + err);
			DebugInfo.printMessage(1,"!!! ERROR: sendEAPpacket: not receive answer");
			res = 2;
		}
		
		return res;
	}

	int sendTerminatePacket() {
		DebugInfo.printMessage(1,"sendTerminatePacket: try send terminate packet");
		EAPRadiusAttributes stateRadiusAttr = receivedRadPkt.getAttributeByType(24);
		if((stateRadiusAttr != null) && (stateRadiusAttr.value != null)) {
			DebugInfo.printMessage(1,"sendTerminatePacket: State Attribute found");
			radiusPkt.ReInit(1,counterRadiusPkt++);
			radiusPkt.addAttribute(stateRadiusAttr);
			radiusPkt.addAttribute(radAttr_auth);
			radAttr_auth.setMessageAuthenticator(radiusPkt,SharedSecret);
			radiusPkt.toByteArray();
			sendPacket();
			try {
				clientSocket.receive(receivePacket);
				DebugInfo.printMessage(1,"sendTerminatePacket: packet has been received, length: " + receivePacket.getLength());
				counterWrongPackets2++;
				parseRecvMessage();
				if(receivedRadPkt.code != 3) 
					DebugInfo.printMessage(1,"!!! sendPacketsSequence: Received unexpected message");
			} catch (IOException err) {
				DebugInfo.printMessage(1,"sendTerminatePacket: receive packet: " + err);
			}
		}
		
		return 0;
	}
	

	void sendPacket() {
		DebugInfo.printDump(2, radiusPkt.net_msg, "sendPacket: Radius packet");
		DebugInfo.printMessage(1,"sendPacket: Sending Radius packet");
		sendPacket = new DatagramPacket(radiusPkt.net_msg, radiusPkt.net_msg.length, remoteIPAddress, destination_port);
		radiusPkt.isLengthSet = false;
		try {
			clientSocket.send(sendPacket);
		} catch (IOException err) {
			DebugInfo.printMessage(1,"!!!ERROR, sendPacket: send packet: " + err);
		}
		DebugInfo.printMessage(1,"sendPacket: packet has been sended");
	}
	
}

class TestAttributesSequence1
{
	// EAP SIM Attributes
	EAPsimAttribute attrUNKNOWN;
	EAPsimAttrRand  attrRAND;
	EAPsimAttrPadding attrPadding;
	EAPsimAttrNonceMT attrNonceMT;
	EAPsimAttrPermanentIdReq attrPerID;
	EAPsimAttrMAC attrMAC;
	EAPsimAttrNotification attrNotf;
	EAPsimAttrAnyIdReq attrAnyID;
	EAPsimAttrIdentity attrID;
	EAPsimAttrVersionList attrVerLst;
	EAPsimAttrSelectedVersion attrSelVer;
	EAPsimAttrFullAuthIdReq attrFullID;
	EAPsimAttrCounter attrCnt;
	EAPsimAttrCounterSmall attrCntSmall;
	EAPsimAttrNonceS attrNonceS;
	EAPsimAttrClientErrorCode attrClErrCode;
	EAPsimAttrIV attrIV;
	EAPsimAttrEncrData attrENC;
	EAPsimAttrNextPseudonym attrNxtPsdnm;
	EAPsimAttrNextReauthID attrNxtID;
	EAPsimAttrResultInd attrResInd;
	EAPsimAttribute attrEXP;
    
	ArrayList<EAPsimAttribute> eapSimAttrList = null;


	TestAttributesSequence1(String userName) {
		this.attrUNKNOWN = new EAPsimAttribute();
		this.attrRAND    = new EAPsimAttrRand();
		this.attrPadding = new EAPsimAttrPadding();
		this.attrNonceMT = new EAPsimAttrNonceMT();
		this.attrPerID = new EAPsimAttrPermanentIdReq();
		this.attrMAC = new EAPsimAttrMAC();
		this.attrNotf = new EAPsimAttrNotification();
		this.attrAnyID = new EAPsimAttrAnyIdReq();
		this.attrID = new EAPsimAttrIdentity(userName.getBytes());
		short[] verList = {2,1};
		this.attrVerLst = new EAPsimAttrVersionList(verList);
		this.attrSelVer = new EAPsimAttrSelectedVersion(1);
		this.attrFullID = new EAPsimAttrFullAuthIdReq();
		this.attrCnt = new EAPsimAttrCounter(1);
		this.attrCntSmall = new EAPsimAttrCounterSmall();
		this.attrNonceS = new EAPsimAttrNonceS();
		this.attrClErrCode = new EAPsimAttrClientErrorCode(0);
		this.attrIV = new EAPsimAttrIV();
		this.attrENC = new EAPsimAttrEncrData();
		String pseudonym1 = "eapSimTestPseudonym"; 
		this.attrNxtPsdnm = new EAPsimAttrNextPseudonym(pseudonym1.getBytes());
		String reauthID1 = "eapSimTestReauthID"; 
		this.attrNxtID = new EAPsimAttrNextReauthID(reauthID1.getBytes());
		this.attrResInd = new EAPsimAttrResultInd();
		this.attrEXP = new EAPsimAttribute(EAPsimAttrType.EAPSIM_AT_EXPERIMENTAL);
	    
		this.eapSimAttrList = new ArrayList<EAPsimAttribute>();
		this.eapSimAttrList.add(null);
		this.eapSimAttrList.add(attrUNKNOWN);
		this.eapSimAttrList.add(attrRAND);
		this.eapSimAttrList.add(attrPadding);
		this.eapSimAttrList.add(attrNonceMT);
		this.eapSimAttrList.add(attrPerID);
		this.eapSimAttrList.add(attrMAC);
		this.eapSimAttrList.add(attrNotf);
		this.eapSimAttrList.add(attrAnyID);
		this.eapSimAttrList.add(attrID);
		this.eapSimAttrList.add(attrVerLst);
		this.eapSimAttrList.add(attrSelVer);
		this.eapSimAttrList.add(attrFullID);
		this.eapSimAttrList.add(attrCnt);
		this.eapSimAttrList.add(attrCntSmall);
		this.eapSimAttrList.add(attrNonceS);
		this.eapSimAttrList.add(attrClErrCode);
		this.eapSimAttrList.add(attrIV);
		this.eapSimAttrList.add(attrENC);
		this.eapSimAttrList.add(attrNxtPsdnm);
		this.eapSimAttrList.add(attrNxtID);
		this.eapSimAttrList.add(attrResInd);
		this.eapSimAttrList.add(attrEXP);
    }
	
	public void setTestAttrList(int eapSimInd, int eapSimAttrInd, EAPtypePldSim eapSim) {
		DebugInfo.printMessage(1,"setTestAttrList");
		if(eapSim == null) {
			DebugInfo.printMessage(1,"!!! ERROR: setTestAttrList: null pointer");
			return;
		}

		eapSim.clearAttributesList();
		
		switch(eapSimInd) {
		case 1: break;
		case 2: 
			eapSim.addAttribute(attrID);
			eapSim.addAttribute(attrNonceMT);
			break;
		case 3: 
			eapSim.addAttribute(attrID);
			eapSim.addAttribute(attrSelVer);
			break;
		case 4: 
			eapSim.addAttribute(attrNonceMT);
			eapSim.addAttribute(attrSelVer);
			break;
		case 5: 
			eapSim.addAttribute(attrID);
			eapSim.addAttribute(attrNonceMT);
			eapSim.addAttribute(attrSelVer);
			break;
		}
		
		eapSim.addAttribute(eapSimAttrList.get(eapSimAttrInd));
		
		DebugInfo.printMessage(1,"setTestAttrList return");
	}
	
	public int getSize() {
		if(eapSimAttrList != null)
			return eapSimAttrList.size();
		else
			return 0;
	}
}

