package ru.ispras.telecomtesk.eap;

import java.io.*;
import java.math.BigInteger;
import java.net.*;
import java.util.ArrayList;
import java.security.*;

import javax.crypto.interfaces.DHPublicKey;
import ru.ispras.telecomtesk.eap.EAPoverRadius.*;


public class tryEAPinit
{
    public static void main(String args[]) throws Exception {
    	tryEAPinitRun test1 = new tryEAPinitRun();
    	test1.runtest();
    }
}

class tryEAPinitRun
{
	// messages received from NUT
	public static int destination_port = 1812; 

	public static byte[] sendData = new byte[65000];
	public static int sendDataLength = 0;
	public static byte[] receiveData = new byte[65000];
	public static int receiveDataLength = 0;

	public static InetAddress remoteIPAddress;
//	public static String remoteHost = "192.168.56.101";
	public static byte[] remoteHostRaw = {(byte)(10),(byte)(10),(byte)(12),(byte)(12)};
//	public static byte[] remoteHostRaw = {(byte)(192),(byte)(168),(byte)(56),(byte)(101)};
//	public static byte[] remoteHostRaw = {(byte)(127),(byte)(0),(byte)(0),(byte)(1)};

	EAPRadiusHeader	receivedRadPkt;
	EAPheader receivedEAPPkt;
	DatagramPacket receivePacket;
	
	
	public void runtest() throws Exception {
		DebugInfo.debugLevel = 2;
		DebugInfo.maxDumpSize = 1000;

		DatagramSocket clientSocket = new DatagramSocket();
		clientSocket.setSoTimeout(2000);
//		remoteIPAddress = InetAddress.getByName(remoteHost);
		remoteIPAddress = InetAddress.getByAddress(remoteHostRaw);
		DatagramPacket sendPacket;
		receivePacket = new DatagramPacket(receiveData, receiveData.length);

		String SharedSecretStr = "testing123";
//		String SharedSecretStr = "secret";
		byte[] SharedSecret = SharedSecretStr.getBytes();
		String UserPwdStr = "tester"; // "123456";
//		String UserPwdStr = "efqcltpm+>y]";
		byte[] UserPwd = UserPwdStr.getBytes();

// send first message		
		DebugInfo.printMessage(1,"try create pkt1  EAP_TYPE_IDENTITY");

		int radiusIdentifier = 90; // random value
		EAPRadiusHeader	radPkt = new EAPRadiusHeader(1,radiusIdentifier); // radius fields: 1-REQUEST, identifier
	
		// create radius userName-attribute
	    EAPRadiusAttributes radAttr_name = new EAPRadiusAttributes(1);
		String userName = "tester"; // "sqltest";
		radAttr_name.value = userName.getBytes();

		// create eap message EAP-IDENTITY and radius eap-attribute
		EAPpldResponse eapInitMsg = new EAPpldResponse(10, EAPtype.EAP_TYPE_IDENTITY);
	    eapInitMsg.EAPmethodPld = new EAPtypePldID(userName.getBytes());
	    eapInitMsg.toByteArray();
		EAPRadiusAttributes radAttr_eap = new EAPRadiusAttributes(79,eapInitMsg.net_msg);

		// create radius eapAuthentificator-attribute
	    EAPRadiusAttributes radAttr_auth = new EAPRadiusAttributes(80);
	    
		int res = 0;

		// combine radius message
        radPkt.addAttribute(radAttr_name);
        radPkt.addAttribute(radAttr_eap);
        radPkt.addAttribute(radAttr_auth);

		radAttr_auth.setMessageAuthenticator(radPkt,SharedSecret);
		radPkt.toByteArray();

		DebugInfo.printDump(1, radPkt.net_msg, "Radius packet");
		DebugInfo.printMessage(1,"Sending Radius packet");

		sendPacket = new DatagramPacket(radPkt.net_msg, radPkt.net_msg.length, remoteIPAddress, destination_port);
		clientSocket.send(sendPacket);

		DebugInfo.printMessage(1,"packet has been sended");


// parse received message		
		try {
			clientSocket.receive(receivePacket);
			DebugInfo.printMessage(1,"sendPacketsSequence: packet1 has been received, length: " + receivePacket.getLength());
			res = parseRecvMessage();
			if((res != 0)||(receivedRadPkt.code != 3)) {
				DebugInfo.printMessage(1,"!!! sendPacketsSequence: Received unexpected message");
			}
		} catch (IOException err) {
			DebugInfo.printMessage(1,"sendPacketsSequence: receive packet1: " + err);
		}
		
		
// send second message		
		DebugInfo.printMessage(1,"try create pkt2 EAP-MD5");

		radiusIdentifier++;
		radPkt = new EAPRadiusHeader(1,radiusIdentifier); 
		
		// create eap message EAP-MD5 and radius eap-attribute
        EAPpldBasic eapMsg2 = new EAPpldBasic();
        eapMsg2.code = EAPcode.EAP_CODE_RESPONSE;
        eapMsg2.identifier = receivedEAPPkt.identifier;
        eapMsg2.type = EAPtype.EAP_TYPE_MD5CHALLENGE;
        EAPtypePldMd5 eapPldMd5 = new EAPtypePldMd5();
        eapPldMd5.setMD5value(eapMsg2.identifier, ((EAPtypePldMd5)(((EAPpldBasic)receivedEAPPkt).EAPmethodPld)).value, UserPwd);
        
        eapMsg2.EAPmethodPld = eapPldMd5;
        eapMsg2.toByteArray();
		EAPRadiusAttributes radAttr_eap2 = new EAPRadiusAttributes(79,eapMsg2.net_msg);
        
		
		// add radius userName-attribute
        radPkt.addAttribute(radAttr_name);
        
		// add radius state-attribute from received radius packet
		EAPRadiusAttributes stateRadiusRecv2 = receivedRadPkt.getAttributeByType(24); 
		if((stateRadiusRecv2 != null) && (stateRadiusRecv2.value != null)) {
			DebugInfo.printMessage(1,"State Attribute found");
	        radPkt.addAttribute(stateRadiusRecv2);
		}
		
        radPkt.addAttribute(radAttr_auth);
        radPkt.addAttribute(radAttr_eap2);

		radAttr_auth.setMessageAuthenticator(radPkt,SharedSecret);
		radPkt.toByteArray();
		

		DebugInfo.printDump(1, radPkt.net_msg, "Radius packet");
		DebugInfo.printMessage(1,"Sending Radius packet");

		sendPacket = new DatagramPacket(radPkt.net_msg, radPkt.net_msg.length, remoteIPAddress, destination_port);
		clientSocket.send(sendPacket);

		DebugInfo.printMessage(1,"packet has been sended");


// parse received message		
		try {
			clientSocket.receive(receivePacket);
			DebugInfo.printMessage(1,"sendPacketsSequence: packet1 has been received, length: " + receivePacket.getLength());
			res = parseRecvMessage();
			if((res != 0)||(receivedRadPkt.code != 3)) {
				DebugInfo.printMessage(1,"!!! sendPacketsSequence: Received unexpected message");
			}
		} catch (IOException err) {
			DebugInfo.printMessage(1,"sendPacketsSequence: receive packet1: " + err);
		}

// close socket		
		clientSocket.close();
		DebugInfo.printMessage(1,"socket closed");
	}

	
	int parseRecvMessage() {
		int res1, res2 = 0;
		
		DebugInfo.printDump(2, receivePacket.getData(), 0, receivePacket.getLength() , "received packet");

		// parse received message
		receivedRadPkt = new EAPRadiusHeader();
		receivedRadPkt.net_msg = new byte[receivePacket.getLength()];
		System.arraycopy(receivePacket.getData(), 0, receivedRadPkt.net_msg, 0, receivePacket.getLength());
		res1 = receivedRadPkt.parseNetMessage();
		DebugInfo.printMessage(1,"packet was parsed, res: " + res1);
		receivedRadPkt.print();

		// parse EAP received message
		receivedEAPPkt = null;
		EAPRadiusAttributes eapRadAttr = receivedRadPkt.getAttributeByType(79);
		if((eapRadAttr != null) && (eapRadAttr.value != null)) {
			DebugInfo.printMessage(1,"EAP message found, try parse");
			if(eapRadAttr.value.length > 4)
				receivedEAPPkt = new EAPpldBasic();
			else
				receivedEAPPkt = new EAPheader();
			
			receivedEAPPkt.net_msg = new byte[eapRadAttr.value.length];
			System.arraycopy(eapRadAttr.value, 0, receivedEAPPkt.net_msg, 0, eapRadAttr.value.length);
			res2 = receivedEAPPkt.parseNetMessage(); 
			DebugInfo.printMessage(1,"EAP packet was parsed, res: " + res2);
			receivedEAPPkt.print("   ");
		}
		
		return (res1 + res2);
	}

}

