package ru.ispras.telecomtesk.tlsv12.HandshakeMessages;

import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.*;

public class CertificateRequest extends HandshakeMessage
{
	public byte[] certificate_types;
	public SignatureAndHashAlgorithm[] supported_signature_algorithms;
	public String[] certificate_authorities;
	
	public CertificateRequest(byte[] cert, SignatureAndHashAlgorithm[] sah, String[] cert_auth) {
		msg_type = 13;
		certificate_types = cert;
		supported_signature_algorithms = sah;
		certificate_authorities = cert_auth;
	}
	public CertificateRequest(ClientCertificateType[] cert, SignatureAndHashAlgorithm[] sah, String[] cert_auth) {
		msg_type = 13;
		certificate_types = new byte[cert.length];
		for(int i=0; i<cert.length; i++)
			certificate_types[i] = cert[i].getValue();
		supported_signature_algorithms = sah;
		certificate_authorities = cert_auth;
	}
	public short getAuthoritiesLength() {
		int ext_len = 0;
		for (int j = 0; j < certificate_authorities.length; j++) {
			// <field 'length' of 2 byte> + number of char(2 bytes each)
			ext_len += 2 + certificate_authorities[j].length();
		}
		return (short)ext_len;
	}
	public int getSize() {
		int size;
		size = 9 + certificate_types.length + 2*supported_signature_algorithms.length + getAuthoritiesLength();
		return size;
	}
	public String toString() {
                int i;
		String res = "HandshakeMessage: type CertificateRequest, length " + length;
		if(net_msg != null)
			res += ", net_msg: " + net_msg.length;
		res += "\n      certificate_types: ";


		for(i=0;i<certificate_types.length;i++) {
			switch(certificate_types[i]) {
				case 1: res += "rsa_sign;"; break;
				case 2: res += "dss_sign;"; break;
				case 3: res += "rsa_fixed_dh;"; break;
				case 4: res += "dss_fixed_dh;"; break;
				case 5: res += "rsa_ephemeral_dh_RESERVED;"; break;
				case 6: res += "dss_ephemeral_dh_RESERVED;"; break;
				case 20: res += "fortezza_dms_RESERVED;"; break;
				default: res += "unknown=" + certificate_types[i] + ";";
			}
		}


		res += "\n      signature_algorithms: ";

		for(i=0;i<supported_signature_algorithms.length;i++) {
			res += supported_signature_algorithms[i].toString() + ";";
		}


		return res;
	}
	
	/*
	 * (non-Javadoc)
	 * @see ru.ispras.telecomtesk.tls.HandshakeMessages.HandshakeMessage#toByteArray()
	 * 
	 * size of CertificateRequest in bytes:
	 * type + length = 4
	 * <field 'length' of 1 byte> + certificate_types = 1 + certificate_types.length
	 * <field 'length' of 2 byte> + supported_signature_algorithms = 2 + (2 * supported_signature_algorithms.length)
	 * <field 'length' of 2 byte> + <certificate_authorities> = 2 + <2 + string.length>
	 * 
	 * 4 + 1 + certificate_types.length + 2 + 2*supported_signature_algorithms.length + 2 + <2 + string.length>
	 */
	public int toByteArray() {
		int i = 0, j, res = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		// write msg_type, full length and version
		byte[] ba_len = hton(length);
		ba[0] = msg_type;
		System.arraycopy(ba_len, 1, ba, 1, 3);
		i=4;
		// write certificate_types
		ba[i++] = (byte) certificate_types.length;
		for (j = 0; j < certificate_types.length; j++)
			ba[i++] = certificate_types[j];
		// write supported_signature_algorithms
		j = 2 * supported_signature_algorithms.length;
		ba[i++] = (byte)(j >>> 8);
		ba[i++] = (byte)j;
		for (j = 0; j < supported_signature_algorithms.length; j++) {
			ba[i++] = (byte)supported_signature_algorithms[j].hash;
			ba[i++] = (byte)supported_signature_algorithms[j].signature;
		}
		// write certificate_authorities
		j = getAuthoritiesLength();
		ba[i++] = (byte)(j >>> 8);
		ba[i++] = (byte)j;
		byte[] ext;
		for (j = 0; j < certificate_authorities.length; j++) {
			ext = certificate_authorities[j].getBytes();
			ba[i++] = (byte)(ext.length >>> 8);
			ba[i++] = (byte)ext.length;
			System.arraycopy(ext, 0, ba, i, (short)(ext.length));
			i += (short)(ext.length);
		}
		
		net_msg = ba;
		return res;
	}
}
