package ru.ispras.telecomtesk.tlsv12.HandshakeMessages;

import java.security.*;
import java.util.ArrayList;

import javax.crypto.spec.SecretKeySpec;

import ru.ispras.telecomtesk.tlsv12.DebugInfo;
import ru.ispras.telecomtesk.tlsv12.RecordProtocol.DigitallySigned;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.HashAlgorithm;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.SignatureAlgorithm;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.SignatureAndHashAlgorithm;
import ru.ispras.telecomtesk.tlsv12.TLSState.TLSSecurityParameters;

//   HelloRequest message MUST NOT be included in the message hashes that are
//   used in the Finished messages and the certificate verify message. 

public class CertificateVerify extends HandshakeMessage
{
	public DigitallySigned ds;
	public boolean set_signature = false;
	public int error = 0;
	
	public CertificateVerify(SignatureAndHashAlgorithm sah) {
		msg_type = 15; 
		ds = new DigitallySigned(sah); 
	}
	public CertificateVerify(DigitallySigned ds) {
		msg_type = 15; 
		this.ds = ds;
		set_signature = true;
	}
	
	public void calcSignature(PrivateKey private_key, HandshakeMessage[] msglist) throws Exception {
		Signature sig;
		String hash_name, sig_name, alg_name;
		
		if((private_key != null) && (msglist != null)) {
			sig_name = SignatureAlgorithm.valueOf(ds.alg.signature).toString();
			hash_name = HashAlgorithm.valueOf(ds.alg.hash).toString();
			alg_name = hash_name + "with" + sig_name;
			
			sig = Signature.getInstance(alg_name,"BC");
			sig.initSign(private_key);

			for(int i=0;i<msglist.length;i++) {
				if(msglist[i] != null) {
					//   HelloRequest message MUST NOT be included in the message hashes that are
					//   used in the Finished messages and the certificate verify message. 
					if(msglist[i] instanceof HelloRequest) continue;
					if(msglist[i] instanceof HandshakeUndefinedType) continue;  // TEST message type, not include in Hash calculating
					if(msglist[i].net_msg != null) {
						DebugInfo.printDump(5,msglist[i].net_msg,"net_msg");
						sig.update(msglist[i].net_msg);
					}
				}
			}
			this.ds.setSignature(sig.sign());
			set_signature = true;
		}
		else
			error = -1;
	}

	public void calcSignature(PrivateKey private_key, ArrayList msglist) throws Exception {
		Signature sig;
		String hash_name, sig_name, alg_name;
		Object curmsg;
		
		if((private_key != null) && (msglist != null) && (!msglist.isEmpty())) {
			sig_name = SignatureAlgorithm.valueOf(ds.alg.signature).toString();
			hash_name = HashAlgorithm.valueOf(ds.alg.hash).toString();
			alg_name = hash_name + "with" + sig_name;
			
			sig = Signature.getInstance(alg_name,"BC");
			sig.initSign(private_key);

			for(int i=0;i<msglist.size();i++) {
				curmsg = msglist.get(i);
				if(curmsg instanceof HandshakeMessage) {
					//   HelloRequest message MUST NOT be included in the message hashes that are
					//   used in the Finished messages and the certificate verify message. 
					if(curmsg instanceof HelloRequest) continue;
					if(curmsg instanceof HandshakeUndefinedType) continue;  // TEST message type, not include in Hash calculating
					if(((HandshakeMessage)curmsg).net_msg != null) {
						DebugInfo.printDump(5,((HandshakeMessage)curmsg).net_msg,"net_msg");
						sig.update(((HandshakeMessage)curmsg).net_msg);
					}
				}
			}
			this.ds.setSignature(sig.sign());
			set_signature = true;
		}
		else
			error = -1;
	}

	public void calcSignature(PrivateKey private_key, ArrayList msglist, int offset, int len) throws Exception {
		Signature sig;
		String hash_name, sig_name, alg_name;
		Object curmsg;
		
		if((private_key != null) && (msglist != null) && (!msglist.isEmpty())) {
			if((len == 0) || (offset > msglist.size() - 1))
				return;
			sig_name = SignatureAlgorithm.valueOf(ds.alg.signature).toString();
			hash_name = HashAlgorithm.valueOf(ds.alg.hash).toString();
			alg_name = hash_name + "with" + sig_name;
			
			sig = Signature.getInstance(alg_name,"BC");
			sig.initSign(private_key);

			for(int i=offset;i<msglist.size() && len>0;i++,len--) {
				curmsg = msglist.get(i);
				if(curmsg instanceof HandshakeMessage) {
					//   HelloRequest message MUST NOT be included in the message hashes that are
					//   used in the Finished messages and the certificate verify message. 
					if(curmsg instanceof HelloRequest) continue;
					if(curmsg instanceof HandshakeUndefinedType) continue;  // TEST message type, not include in Hash calculating
					if(((HandshakeMessage)curmsg).net_msg != null) {
						DebugInfo.printDump(5,((HandshakeMessage)curmsg).net_msg,"net_msg");
						sig.update(((HandshakeMessage)curmsg).net_msg);
					}
				}
			}
			this.ds.setSignature(sig.sign());
			set_signature = true;
		}
		else
			error = -1;
	}

	public boolean verifySignature(PublicKey public_key, HandshakeMessage[] msglist, byte[] signat) throws Exception {
		Signature sig;
		String hash_name, sig_name, alg_name;
		PublicKey pubkey;
		boolean res = false;
		
		if((public_key != null) && (msglist != null) && (signat != null)) {
			sig_name = SignatureAlgorithm.valueOf(ds.alg.signature).toString();
			hash_name = HashAlgorithm.valueOf(ds.alg.hash).toString();
			alg_name = hash_name + "with" + sig_name;
			
			sig = Signature.getInstance(alg_name,"BC");
			sig.initVerify(public_key);

			for(int i=0;i<msglist.length;i++) {
				if(msglist[i] != null) {
					//   HelloRequest message MUST NOT be included in the message hashes that are
					//   used in the Finished messages and the certificate verify message. 
					if(msglist[i] instanceof HelloRequest) continue;
					if(msglist[i] instanceof HandshakeUndefinedType) continue;  // TEST message type, not include in Hash calculating
					if(msglist[i].net_msg != null) {
						DebugInfo.printDump(5,msglist[i].net_msg,"net_msg");
						sig.update(msglist[i].net_msg);
					}
				}
			}
			res = sig.verify(signat);
		}
		
		return res;
	}

	public boolean verifySignature(PublicKey public_key, ArrayList msglist, byte[] signat) throws Exception {
		Signature sig;
		String hash_name, sig_name, alg_name;
		PublicKey pubkey;
		Object curmsg;
		boolean res = false;
		
		if((public_key != null) && (msglist != null) && (!msglist.isEmpty()) && (signat != null)) {
			sig_name = SignatureAlgorithm.valueOf(ds.alg.signature).toString();
			hash_name = HashAlgorithm.valueOf(ds.alg.hash).toString();
			alg_name = hash_name + "with" + sig_name;
			
			sig = Signature.getInstance(alg_name,"BC");
			sig.initVerify(public_key);

			for(int i=0;i<msglist.size();i++) {
				curmsg = msglist.get(i);
				if(curmsg instanceof HandshakeMessage) {
					//   HelloRequest message MUST NOT be included in the message hashes that are
					//   used in the Finished messages and the certificate verify message. 
					if(curmsg instanceof HelloRequest) continue;
					if(curmsg instanceof HandshakeUndefinedType) continue;  // TEST message type, not include in Hash calculating
					if(((HandshakeMessage)curmsg).net_msg != null) {
						DebugInfo.printDump(5,((HandshakeMessage)curmsg).net_msg,"net_msg");
						sig.update(((HandshakeMessage)curmsg).net_msg);
					}
				}
			}
			res = sig.verify(signat);
		}
		
		return res;
	}

	public boolean verifySignature(PublicKey public_key, ArrayList msglist, int offset, int len, byte[] signat) throws Exception {
		Signature sig;
		String hash_name, sig_name, alg_name;
		PublicKey pubkey;
		Object curmsg;
		boolean res = false;
		
		if((public_key != null) && (msglist != null) && (!msglist.isEmpty()) && (signat != null)) {
			if((len == 0) || (offset > msglist.size() - 1))
				return false;
			sig_name = SignatureAlgorithm.valueOf(ds.alg.signature).toString();
			hash_name = HashAlgorithm.valueOf(ds.alg.hash).toString();
			alg_name = hash_name + "with" + sig_name;
			
			sig = Signature.getInstance(alg_name,"BC");
			sig.initVerify(public_key);

			for(int i=offset;i<msglist.size() && len>0;i++,len--) {
				curmsg = msglist.get(i);
				if(curmsg instanceof HandshakeMessage) {
					//   HelloRequest message MUST NOT be included in the message hashes that are
					//   used in the Finished messages and the certificate verify message. 
					if(curmsg instanceof HelloRequest) continue;
					if(curmsg instanceof HandshakeUndefinedType) continue;  // TEST message type, not include in Hash calculating
					if(((HandshakeMessage)curmsg).net_msg != null) {
						DebugInfo.printDump(5,((HandshakeMessage)curmsg).net_msg,"net_msg");
						sig.update(((HandshakeMessage)curmsg).net_msg);
					}
				}
			}
			res = sig.verify(signat);
		}
		
		return res;
	}

	public boolean verifySignature(PublicKey public_key, ArrayList msglist, int offset, int len) throws Exception {
		Signature sig;
		String hash_name, sig_name, alg_name;
		PublicKey pubkey;
		Object curmsg;
		boolean res = false;
		
		if((public_key != null) && (msglist != null) && (!msglist.isEmpty()) && (ds.signature != null)) {
			if((len == 0) || (offset > msglist.size() - 1))
				return false;
			sig_name = SignatureAlgorithm.valueOf(ds.alg.signature).toString();
			hash_name = HashAlgorithm.valueOf(ds.alg.hash).toString();
			alg_name = hash_name + "with" + sig_name;
			
			sig = Signature.getInstance(alg_name,"BC");
			sig.initVerify(public_key);

			for(int i=offset;i<msglist.size() && len>0;i++,len--) {
				curmsg = msglist.get(i);
				if(curmsg instanceof HandshakeMessage) {
					//   HelloRequest message MUST NOT be included in the message hashes that are
					//   used in the Finished messages and the certificate verify message. 
					if(curmsg instanceof HelloRequest) continue;
					if(curmsg instanceof HandshakeUndefinedType) continue;  // TEST message type, not include in Hash calculating
					if(((HandshakeMessage)curmsg).net_msg != null) {
						DebugInfo.printDump(5,((HandshakeMessage)curmsg).net_msg,"net_msg");
						sig.update(((HandshakeMessage)curmsg).net_msg);
					}
				}
			}
			res = sig.verify(ds.signature);
		}
		
		return res;
	}

	public int getSize() {
		if((set_signature == false) || (ds.signature == null) || (error != 0))
				return 0;
		else
			return (8 + (short)ds.signature.length);
	}


	public String toString() {
                int i;
		String res = "HandshakeMessage: type CertificateVerify, length " + length;
		if(net_msg != null)
			res += ", net_msg: " + net_msg.length;
		res += "\n      SignatureAndHashAlgorithm: ";

		if(ds != null) {
			res += ds.alg.toString();
			res += "\n      signature length: ";
			if(ds.signature != null) 
				res += ds.signature.length;
		}

		return res;
	}


	public int toByteArray() {
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		int res = 0;

		if(ba.length != 0) {
			int i = 0, j;
			// write msg_type, full length and version
			byte[] ba_len = hton(length);
			ba[0] = msg_type;
			System.arraycopy(ba_len, 1, ba, 1, 3);
			i=4;
			ba[i++] = ds.alg.hash;
			ba[i++] = ds.alg.signature;
			ba[i++] = (byte)(ds.signature.length >>> 8);
			ba[i++] = (byte)ds.signature.length;
			System.arraycopy(ds.signature, 0, ba, i, (short)ds.signature.length);
		}

		net_msg = ba;
		return res;
	}
}
