package ru.ispras.telecomtesk.tlsv12.HandshakeMessages;

import java.util.Random;
import ru.ispras.telecomtesk.tlsv12.DebugInfo;
import ru.ispras.telecomtesk.tlsv12.TLSState.TLSSecurityParameters;
import ru.ispras.telecomtesk.tlsv12.TLSState.TLSSecurityParameters;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.TLSCipherSuite;
import java.security.*;
import java.util.ArrayList;

public class Finished extends HandshakeMessage
{
	String[] finished_label = {"client finished","server finished"}; 
	public byte[] verify_data = null;
	public boolean isServer = false;
	private int label;

	public Finished(boolean isServer) {
		msg_type = 20; 
		this.isServer = isServer;
		label = (isServer == false ? 0 : 1);
	}
	
	public Finished(boolean isServer, byte[] verify_data) {
		msg_type = 20; 
		this.isServer = isServer;
		label = (isServer == false ? 0 : 1);
		this.verify_data = verify_data;
	}

	public int getVerifyDataLength(TLSCipherSuite cipherSuite) {
		int res = 0;
		switch (cipherSuite) {
			default:
				res = 12; // in bytes
		}
		return res;
	}

	public int calcVerifyData(TLSSecurityParameters param, byte[] msg) throws Exception {
		byte[] msghash;
		MessageDigest md;
		String alg;
		int res = 0;
		
		DebugInfo.printMessage(5,"Finished::calcVerifyData start");
		
		if((param != null) && (msg != null)) {
			// calculate Hash(handshake_messages)
			switch (param.prf) {
				case tls_prf_sha256:
					alg = "SHA-256";
					break;
				default:
					DebugInfo.printMessage(3,"===Error! Finished::calcVerifyData: Algorithm not supported");
					return -2;
			}

			md = MessageDigest.getInstance(alg);
			DebugInfo.printMessage(5,"Finished::calcVerifyData: getInstance ok");
			msghash = md.digest(msg);
			DebugInfo.printMessage(5,"Finished::calcVerifyData: digest ok");

			verify_data = new byte[getVerifyDataLength(param.cs)];
			res = param.prf(param.master_secret, finished_label[label], msghash, verify_data);
			DebugInfo.printMessage(5,"Finished::calcVerifyData: prf ok");
		}
		else
			res = -1;
		
		return res;
	}

	public int calcVerifyData(TLSSecurityParameters param, byte[][] msg) throws Exception {
		byte[] msghash;
		MessageDigest md;
		String alg;
		int res = 0;
		
		DebugInfo.printMessage(5,"Finished::calcVerifyData start");
		
		if((param != null) && (msg != null)) {
			// calculate Hash(handshake_messages)
			switch (param.prf) {
				case tls_prf_sha256:
					alg = "SHA-256";
					break;
				default:
					DebugInfo.printMessage(3,"===Error! Finished::calcVerifyData: Algorithm not supported");
					return -2;
			}

			md = MessageDigest.getInstance(alg);
			DebugInfo.printMessage(5,"Finished::calcVerifyData: getInstance ok");

			for(int i=0;i<msg.length;i++) {
				if(msg[i] != null)
					md.update(msg[i]);
			}
			DebugInfo.printMessage(5,"Finished::calcVerifyData: update ok");

			msghash = md.digest();
			DebugInfo.printMessage(5,"Finished::calcVerifyData: digest ok");

			verify_data = new byte[getVerifyDataLength(param.cs)];
			res = param.prf(param.master_secret, finished_label[label], msghash, verify_data);
			DebugInfo.printMessage(5,"Finished::calcVerifyData: prf ok");
		}
		else
			res = -1;
		
		return res;
	}

	public int calcVerifyData(TLSSecurityParameters param, HandshakeMessage[] msglist) throws Exception {
		byte[] msghash;
		MessageDigest md;
		String alg;
		int res = 0;
		
		DebugInfo.printMessage(5,"Finished::calcVerifyData start");
		
		if((param != null) && (msglist != null)) {
			// calculate Hash(handshake_messages)
			switch (param.prf) {
				case tls_prf_sha256:
					alg = "SHA-256";
					break;
				default:
					DebugInfo.printMessage(3,"===Error! Finished::calcVerifyData: Algorithm not supported");
					return -2;
			}

			md = MessageDigest.getInstance(alg);
			DebugInfo.printMessage(5,"Finished::calcVerifyData: getInstance ok");

			for(int i=0;i<msglist.length;i++) {
				if(msglist[i] != null) {
					//   HelloRequest message MUST NOT be included in the message hashes that are
					//   used in the Finished messages and the certificate verify message. 
					if(msglist[i] instanceof HelloRequest) continue;
					if(msglist[i] instanceof HandshakeUndefinedType) continue;  // TEST message type, not include in Hash calculating
					if(msglist[i].net_msg != null) {
						DebugInfo.printDump(5,msglist[i].net_msg,"net_msg");
						md.update(msglist[i].net_msg);
					}
				}
			}
			DebugInfo.printMessage(5,"Finished::calcVerifyData: update ok");

			msghash = md.digest();
			DebugInfo.printMessage(5,"Finished::calcVerifyData: digest ok");

			verify_data = new byte[getVerifyDataLength(param.cs)];
			res = param.prf(param.master_secret, finished_label[label], msghash, verify_data);
			DebugInfo.printMessage(5,"Finished::calcVerifyData: prf ok");
		}
		else
			res = -1;
		
		return res;
	}

	public int calcVerifyData(TLSSecurityParameters param, ArrayList msglist) throws Exception {
		byte[] msghash;
		MessageDigest md;
		String alg;
		Object curmsg;
		int res = 0;
		
		DebugInfo.printMessage(5,"Finished::calcVerifyData start");
		
		if((param != null) && (msglist != null)&&(!msglist.isEmpty())) {
			// calculate Hash(handshake_messages)
			switch (param.prf) {
				case tls_prf_sha256:
					alg = "SHA-256";
					break;
				default:
					DebugInfo.printMessage(3,"===Error! Finished::calcVerifyData: Algorithm not supported");
					return -2;
			}

			md = MessageDigest.getInstance(alg);
			DebugInfo.printMessage(5,"Finished::calcVerifyData: getInstance ok");

			for(int i=0;i<msglist.size();i++) {
				curmsg = msglist.get(i);
				if(curmsg instanceof HandshakeMessage) {
					//   HelloRequest message MUST NOT be included in the message hashes that are
					//   used in the Finished messages and the certificate verify message. 
					if(curmsg instanceof HelloRequest) continue;
					if(curmsg instanceof HandshakeUndefinedType) continue;  // TEST message type, not include in Hash calculating
					if(((HandshakeMessage)curmsg).net_msg != null) {
						DebugInfo.printDump(5,((HandshakeMessage)curmsg).net_msg,"net_msg");
						md.update(((HandshakeMessage)curmsg).net_msg);
					}
				}
			}
			DebugInfo.printMessage(5,"Finished::calcVerifyData: update ok");

			msghash = md.digest();
			DebugInfo.printMessage(5,"Finished::calcVerifyData: digest ok");

			verify_data = new byte[getVerifyDataLength(param.cs)];
			res = param.prf(param.master_secret, finished_label[label], msghash, verify_data);
			DebugInfo.printMessage(5,"Finished::calcVerifyData: prf ok");
		}
		else
			res = -1;
		
		return res;
	}

	public int calcVerifyData(TLSSecurityParameters param, ArrayList msglist, int offset, int len) throws Exception {
		byte[] msghash;
		MessageDigest md;
		String alg;
		Object curmsg;
		int res = 0;

		DebugInfo.printMessage(5,"Finished::calcVerifyData start");
		
		if((param != null) && (msglist != null)&&(!msglist.isEmpty())) {
			if((len == 0) || (offset > msglist.size() - 1)) {
				DebugInfo.printMessage(5,"===Error! Finished::calcVerifyData bad message list length");
				return -3;
			}
			// calculate Hash(handshake_messages)
			switch (param.prf) {
				case tls_prf_sha256:
					alg = "SHA-256";
					break;
				default:
					DebugInfo.printMessage(3,"===Error! Finished::calcVerifyData: Algorithm not supported");
					return -2;
			}

			md = MessageDigest.getInstance(alg);
			DebugInfo.printMessage(5,"Finished::calcVerifyData: getInstance ok");
			
			for(int i=offset;i<msglist.size() && len>0;i++,len--) {
				curmsg = msglist.get(i);
				if(curmsg instanceof HandshakeMessage) {
					//   HelloRequest message MUST NOT be included in the message hashes that are
					//   used in the Finished messages and the certificate verify message. 
					if(curmsg instanceof HelloRequest) continue;
					if(curmsg instanceof HandshakeUndefinedType) continue;  // TEST message type, not include in Hash calculating
					if(((HandshakeMessage)curmsg).net_msg != null) {
						DebugInfo.printDump(5,((HandshakeMessage)curmsg).net_msg,"net_msg");
						md.update(((HandshakeMessage)curmsg).net_msg);
					}
				}
			}
			DebugInfo.printMessage(5,"Finished::calcVerifyData: update ok");

			msghash = md.digest();
			DebugInfo.printMessage(5,"Finished::calcVerifyData: digest ok");

			verify_data = new byte[getVerifyDataLength(param.cs)];
			res = param.prf(param.master_secret, finished_label[label], msghash, verify_data);
			DebugInfo.printMessage(5,"Finished::calcVerifyData: prf ok");
		}
		else
			res = -1;
		
		return res;
	}

	public void setVerifyData(byte[] vdata) {
		verify_data = vdata;
	}
	
	public void setRandomVerifyData(TLSSecurityParameters param) {
		if(param != null) {
			verify_data = new byte[getVerifyDataLength(param.cs)];
			Random rndObj = new Random(System.currentTimeMillis());
			rndObj.nextBytes(verify_data);
		}
		else
			DebugInfo.printMessage(5,"===Warning! Finished::setRandomVerifyData: Null pointer");
	}
	
	public int getSize() {
		return 4 + verify_data.length;
	}

	public int toByteArray() {
		int res = 0;
		if(isLengthSet == false) setLength();
		byte[] ba = new byte[getSize()];
		byte[] ba_len = hton(length);
		ba[0] = msg_type;
		System.arraycopy(ba_len, 1, ba, 1, 3);
		System.arraycopy(verify_data, 0, ba, 4, verify_data.length);
		net_msg = ba;
		return res;
	}
}