package ru.ispras.telecomtesk.tlsv12.HandshakeMessages;

import java.io.ByteArrayInputStream;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

public class HMCertificate extends HandshakeMessage {    // HandshakeMessages Certificate
	public byte[][] certificate_list;

	public HMCertificate(byte[][] cert_list) {
		msg_type = 11;
		if(cert_list != null)
			certificate_list = cert_list;
		else
			certificate_list = new byte[0][];
	}
	public byte[] getFirstCertificate() {
		if(certificate_list != null)
			return certificate_list[0];
		else
			return null;
	}
	public int getSize() {
		int size = 7; // 4 bytes HandshakeMessage header  + 3 bytes length of certificate_list
		if(certificate_list != null)
			for (int j = 0; j < certificate_list.length; j++) {
				if(certificate_list[j] != null)
					size += 3 + int24bits(certificate_list[j].length);
			}
		return size;
	}
	public boolean isCertificateListEmpty() {
		if((certificate_list != null) && (certificate_list.length > 0) && (certificate_list[0] != null))
			return false;
		else
			return true;
	}
	public int toByteArray() {
		int i = 0, j, len, res = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		len = getSize();
		byte[] ba = new byte[len];
		// write msg_type, full length and version
		byte[] ba_len = hton(length);
		ba[0] = msg_type;
		System.arraycopy(ba_len, 1, ba, 1, 3);
		i=4;
		// write certificate_list
		ba_len = hton(len - 7); // full length minus 4 bytes of HandshakeMessage header and 3 bytes of certificate_list.length
		System.arraycopy(ba_len, 1, ba, i, 3);
		i+=3;
		for (j = 0; j < certificate_list.length; j++) {
			if(certificate_list[j] != null) {
				ba_len = hton(certificate_list[j].length);
				System.arraycopy(ba_len, 1, ba, i, 3);
				i+=3;
				len = int24bits(certificate_list[j].length);
				System.arraycopy(certificate_list[j], 0, ba, i, len);
				i += len;
			}
		}
		
		net_msg = ba;
		return res;
	}

	public String toString() {
		String res = "HandshakeMessage.Certificate: ";
		if(certificate_list != null) {
			X509Certificate curcert;
			for(int i=0; i < certificate_list.length; i++) {
				if(certificate_list[i] != null) {
					res += "\n   Certificate " + i + ": length " + certificate_list[i].length;
					try {
						CertificateFactory factory = CertificateFactory.getInstance("X509");
						curcert = (X509Certificate)factory.generateCertificate( new ByteArrayInputStream(certificate_list[i]) );
						res += ", X509, Key type: " + curcert.getPublicKey().getAlgorithm();
						res += ", SigAlgName: " + curcert.getSigAlgName();
					}
					catch (Exception err) {
						res += ", not X509 type";
					}
				}
			}
		}
		else
			res += "no certificate";

		return res;
	}	
}
