package ru.ispras.telecomtesk.tlsv12.HandshakeMessages;

import java.util.ArrayList;

import ru.ispras.telecomtesk.tlsv12.DebugInfo;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.TLSExtensions.HelloExtension;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.TLSExtensions.RenegotiationExtension;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.TLSExtensions.SignatureAndHashAlgorithmExt;
import ru.ispras.telecomtesk.tlsv12.Mutation.MutateField;
import ru.ispras.telecomtesk.tlsv12.RecordProtocol.DigitallySigned;
import ru.ispras.telecomtesk.tlsv12.RecordProtocol.ProtocolVersion;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.*;
import ru.ispras.telecomtesk.tlsv12.TLSState.TLSSecurityParameters;

public class HandshakeMessage
{
	@MutateField
	public byte msg_type;
	@MutateField
	public int length;
	protected boolean isLengthSet = false;
	public byte[] net_msg = null; // byte array representation of message
	
	public void setMessageType(byte type) {
		msg_type = type;
	}
	public int getSize() {
		return 4;
	}
	public void setLength() {
		length = getSize() > 4 ? getSize() - 4 : 0;
		isLengthSet = true;
	}
	public void setLength(int len) {
		length = len;
		isLengthSet = true;
	}
	public int getLength() {
		return length;
	}
	public String toString() {
		String res = "HandshakeMessage: type ";
		
		switch(msg_type) {
		case 0: res += "HelloRequest"; break;
		case 1: res += "ClientHello"; break;
		case 2: res += "ServerHello"; break;
		case 11: res += "Certificate"; break;
		case 12: res += "ServerKeyExchange"; break;
		case 13: res += "CertificateRequest"; break;
		case 14: res += "ServerHelloDone"; break;
		case 15: res += "CertificateVerify"; break;
		case 16: res += "ClientKeyExchange"; break;
		case 20: res += "Finished"; break;
		default: res += "unknown";
		}

		res += ", length " + length;
		if(net_msg != null)
			res += ", net_msg: " + net_msg.length;
		return res;
	}
	public int toByteArray() {
		if(isLengthSet == false) setLength();
		net_msg = hton(length);
		net_msg[0] = msg_type;
		return 0;
	}
	static public byte[] hton(long x) {
		byte[] res = new byte[8];
		for (int i = 0; i < 8; i++)
		{
			res[i] = (new Long(x >>> 56)).byteValue();
			x <<= 8;
		}
		return res;
	}
	static public byte[] hton(int x) {
		byte[] res = new byte[4];
		for (int i = 0; i < 4; i++)
		{
			res[i] = (new Integer(x >>> 24)).byteValue();
			x <<= 8;
		}
		return res;
	}
	static public byte[] hton(short x) {
		byte[] res = new byte[2];
		for (int i = 0; i < 2; i++)
		{
			res[i] = (new Integer(x >>> 8)).byteValue();
			x <<= 8;
		}
		return res;
	}
	// byte array up to 4 bytes long will be converted
	static public int ntoh(byte[] x)
	{
		int res = 0, i = 0;
		for (; i < 4 && i < x.length; i++)
		{
			res <<= 8;
			res |= (x[i] & 0xFF);
		}
		return res;
	}
	// byte array up to len bytes long will be converted beginning from position pos
	static public int ntoh(byte[] x, int pos, int len) {
    	int res = 0;
    	while(len-- > 0) {
    		res <<= 8;
    		res |= (x[pos++] & 0xFF);
    	}
    	return res;
    }
	// byte array up to 2 bytes long will be converted
	static public short ntohs(byte[] x)
	{
		short res = 0;
		if(x.length >= 2) {
				res = (short)(x[0] & 0xFF);
				res <<= 8;
				res |= (x[1] & 0xFF);
		}
		return res;
	}
	static public short ntohs(byte[] x, int pos)
	{
		short res = 0;
		if(x.length >= 2) {
				res = (short)(x[pos++] & 0xFF);
				res <<= 8;
				res |= (x[pos] & 0xFF);
		}
		return res;
	}

	static public int int24bits(int a)
	{
		a &= 0x00ffffff;
		return a;
	}

    
    public static int parseHandshakeMessagesList(byte[] msgs, ArrayList msglst, TLSSecurityParameters params) {
    	int i,j,offset,hdrlen;
    	byte[] msgByteArray;
    	HandshakeMessage curHM;
    	int err = 0;
    	
    	if((msgs != null) && (msglst != null)) {
    		for(i=0,offset = msgs.length; i < msgs.length; ) {
    			if(offset < 4) {
    				err = -1;
					DebugInfo.printMessage(3,"===Error! HandshakeMessage::parseHandshakeMessagesList: Handshake Message header length is wrong");
    				break;
    			}
    			// get length of current Handshake Message
    			hdrlen = 4 + HandshakeMessage.ntoh(msgs, i+1, 3);
    			if(offset < hdrlen) {
    				err = -2;
    				DebugInfo.printMessage(3,"===Error! HandshakeMessage::parseHandshakeMessagesList: Handshake Message length is wrong");
    				break;
    			}
    			msgByteArray = new byte[hdrlen];
				System.arraycopy(msgs, i, msgByteArray, 0, hdrlen);
				curHM = parseHandshakeMessage(msgByteArray, params);
				if(err < 0)
					break;
				else
					msglst.add(curHM);
				
				i += hdrlen;
				offset -= hdrlen;
    		}
    	}
    	return err;
    }

    
    public static HandshakeMessage parseHandshakeMessage(byte[] msg, TLSSecurityParameters params) {
    	HandshakeMessage hm = null;
    	int len = 0;

		if(msg.length < 4) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHandshakeMessage: header length failed");
    		return hm;
		}
		// get length of current Handshake Message
		len = HandshakeMessage.ntoh(msg, 1, 3);
		if(len != msg.length - 4) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHandshakeMessage: message length failed");
    		return hm;
		}
		
    	switch(msg[0]) {
    	case 0: hm = parseHelloRequest(msg); break;
    	case 1: hm = parseHello(msg); break;
    	case 2: hm = parseHello(msg); 
    			params.setAlgorithmParams(TLSCipherSuite.valueOf(((Hello)hm).cipher_suites[0]));
    			break;
    	case 11: hm = parseCertificate(msg); break;
    	case 12: hm = parseServerKeyExchange(msg, params.ke_alg); break;
    	case 13: hm = parseCertificateRequest(msg); break;
    	case 14: hm = parseServerHelloDone(msg); break;
    	case 15: hm = parseCertificateVerify(msg); break;
    	case 16: hm = parseClientKeyExchange(msg, params.ke_alg); break;
    	case 20: hm = parseFinished(msg, !params.isServer); break;
    	default:
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHandshakeMessage: unknown message type " + msg[0]);
    	}
    	
    	hm.length = len;
    	return hm;
    }

    public static HandshakeMessage parseHelloRequest(byte[] msg) {
    	HandshakeMessage hm = null;

    	if(msg.length == 4) {
			hm = new HelloRequest();
			hm.net_msg = msg;
		}
		else {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHelloRequest: message length failed: " + msg.length);
		}

    	return hm;
    }
    	
    public static HandshakeMessage parseHello(byte[] msg) {
    	HandshakeMessage hm = null;
    	int pos = 0, len, i;

    	// 39 bytes: 4 (header) + 2 (ProtocolVersion) + 32 (Random) + 1 (SessionID length)
    	if(msg.length < 39) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHello: message header length failed");
    		return null;
		}
    	ProtocolVersion pv = new ProtocolVersion(msg[4],msg[5]);
    	byte[] rnd = new byte[32];
    	System.arraycopy(msg, 6, rnd, 0, 32);

    	// parse SessionID
    	len = msg[38];
    	if(msg.length < 39 + len) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHello: SessionID failed");
    		return null;
		}
    	byte[] sid = new byte[len];
    	if(len > 0)
    		System.arraycopy(msg, 39, sid, 0, len);
    	pos = 39 + len;
    	
		short[] cipher_suites;
    	byte[] compression_methods;

    	// Client Hello
    	if(msg[0] != 2) { 
        	// parse CipherSuite
    		if(msg.length < pos + 2) {
    			DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseClientHello: CipherSuite length failed");
    			return null;
    		}
    		len = HandshakeMessage.ntohs(msg,pos);
    		pos += 2;
    		if(msg.length < pos + len) {
    			DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseClientHello: CipherSuite failed");
    			return null;
    		}
    		if(len % 2 != 0) {
    			DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseClientHello: CipherSuite length must be even");
    			return null;
    		}
    		cipher_suites = new short[len/2];
    		for(i=0;i<cipher_suites.length;i++) {
    			cipher_suites[i] = HandshakeMessage.ntohs(msg,pos);
    			pos += 2;
    		}
        	// parse CompressionMethod
        	if(msg.length < pos + 1) {
        		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseClientHello: CompressionMethod length failed");
        		return null;
    		}
        	len = msg[pos++];
        	if(msg.length < pos + len) {
        		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseClientHello: CompressionMethod failed");
        		return null;
    		}
        	compression_methods = new byte[len];
        	if(len > 0) {
        		System.arraycopy(msg, pos, compression_methods, 0, len);
        		pos += len;
        	}
    	}
    	// Server Hello
    	else {
        	// parse CipherSuite (2 bytes) and CompressionMethod (1 byte)
    		if(msg.length < pos + 3) {
    			DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseServerHello: CipherSuite length failed");
    			return null;
    		}
    		cipher_suites = new short[1];
   			cipher_suites[0] = HandshakeMessage.ntohs(msg,pos);
   			pos += 2;
        	// parse CompressionMethod
        	compression_methods = new byte[1];
        	compression_methods[0] = msg[pos++];
    	}
    	
    	// parse Extensions
    	ArrayList<HelloExtension> extensions = new ArrayList<HelloExtension>();
    	if(msg.length > pos) {
    		int extlen = msg.length - pos;
    		if(extlen < 2) {
        		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHello: Extensions length failed");
        		return null;
    		}
    		extlen = HandshakeMessage.ntohs(msg,pos);
    		pos += 2;
    		if(extlen != msg.length - pos) {
        		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHello: Extensions length failed");
        		return null;
    		}
    		
        	HelloExtension curext;
        	short exttype;
        	
    		while(extlen > 0) {
        		if(extlen < 4) {
            		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHello: Extension header failed");
            		return null;
        		}
        		exttype = HandshakeMessage.ntohs(msg,pos);
        		pos+=2;
        		len = HandshakeMessage.ntohs(msg,pos);
        		pos+=2;
        		
        		if(extlen < 4 + len) {
            		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHello: Extension failed");
            		return null;
        		}
        		
        		switch (exttype) {
        		case 13:
        			int len2;
            		len2 = HandshakeMessage.ntohs(msg,pos);
            		pos+=2;

            		if(len != 2 + len2) {
                		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHello: Extension length failed");
                		return null;
            		}

            		if(len2 % 2 != 0) {
        	    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHello: SignatureAndHashAlgorithm Extension length must be even");
        	    		return null;
        			}
        			SignatureAndHashAlgorithm[] sah = new SignatureAndHashAlgorithm[len2/2];
        	    	for(i=0;i<sah.length;i++)
        	    		sah[i] = new SignatureAndHashAlgorithm(msg[pos++],msg[pos++]);
        			curext = new SignatureAndHashAlgorithmExt(sah);
        			break;
        		case (short)0xff01:
            		len2 = msg[pos++];

            		if(len != 1 + len2) {
                		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseHello: Extension length failed");
                		return null;
            		}

            		if(len2 > 0) {
            			byte[] vf_data = new byte[len2];
            			System.arraycopy(msg, pos, vf_data, 0, len2);
            			pos += len2;
            			curext = new RenegotiationExtension(vf_data);
            		}
            		else
            			curext = new RenegotiationExtension();
            			
        			break;
        		default:
                	byte[] extdata = new byte[len];
        			System.arraycopy(msg, pos, extdata, 0, len);
        			curext = new HelloExtension(exttype, extdata);
        			pos += len;
        		}
            	extensions.add(curext);
        		extlen -= 4 + len;
    		}
		}
		
    	hm = new Hello(pv,rnd,sid,cipher_suites,compression_methods,extensions);
		hm.net_msg = msg;
		if(msg[0] == 2)
			((Hello)hm).setTypeServerHello();

    	return hm;
    }
    	
    public static HandshakeMessage parseCertificate(byte[] msg) {
    	HandshakeMessage hm = null;
    	int pos = 0, crlen, certlistlen;

    	if(msg.length < 7) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificate: message header failed");
    		return null;
		}
    	certlistlen = HandshakeMessage.ntoh(msg,4,3);
    	if(msg.length != 7 + certlistlen) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificate: message length failed");
    		return null;
		}

    	pos = 7;
    	ArrayList cert_list = new ArrayList();
		byte[] cur_cert;
		
    	while(certlistlen > 0) {
    		if(certlistlen < 3) {
        		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificate: Certificate header failed");
        		return null;
    		}
    		crlen = HandshakeMessage.ntoh(msg,pos,3);
    		pos+=3;
    		if(certlistlen < 3 + crlen) {
        		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificate: Certificate failed");
        		return null;
    		}
    		cur_cert = new byte[crlen];
        	System.arraycopy(msg, pos, cur_cert, 0, crlen);
    		cert_list.add(cur_cert);
    		pos += crlen;
    		certlistlen -= 3 + crlen;
    	}
    	
    	byte [][] cert_list_array = new byte [cert_list.size()][];
    	for(int i=0; i<cert_list.size(); i++)
    		cert_list_array[i] = (byte [])cert_list.get(i);
    	hm = new HMCertificate(cert_list_array);
		hm.net_msg = msg;

    	return hm;
    }
    	
    public static HandshakeMessage parseServerKeyExchange(byte[] msg, KeyExchangeAlgorithm alg) {
    	HandshakeMessage hm = null;
    	int pos = 0, len;

    	// parse dh_p
    	if(msg.length < 6) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseServerKeyExchange: message header failed, " + alg.name());
    		return null;
		}
    	len = HandshakeMessage.ntohs(msg,4);
    	if(msg.length < 6 + len) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseServerKeyExchange: dh_p failed, " + alg.name());
    		return null;
		}
    	byte[] dh_p = new byte[len];
    	System.arraycopy(msg, 6, dh_p, 0, len);
    	pos = 6 + len;

    	// parse dh_g
    	if(msg.length < pos + 2) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseServerKeyExchange: dh_g length failed, " + alg.name());
    		return null;
		}
    	len = HandshakeMessage.ntohs(msg,pos);
    	pos += 2;
    	if(msg.length < pos + len) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseServerKeyExchange: dh_g failed, " + alg.name());
    		return null;
		}
    	byte[] dh_g = new byte[len];
    	System.arraycopy(msg, pos, dh_g, 0, len);
    	pos += len;
    	
    	// parse dh_Ys
    	if(msg.length < pos + 2) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseServerKeyExchange: dh_Ys length failed, " + alg.name());
    		return null;
		}
    	len = HandshakeMessage.ntohs(msg,pos);
    	pos += 2;
    	if(msg.length < pos + len) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseServerKeyExchange: dh_Ys failed, " + alg.name());
    		return null;
		}
    	byte[] dh_Ys = new byte[len];
    	System.arraycopy(msg, pos, dh_Ys, 0, len);
    	pos += len;

    	DigitallySigned ds = null;
    	// parse DigitallySigned
    	if((alg == KeyExchangeAlgorithm.DHE_DSS) || (alg == KeyExchangeAlgorithm.DHE_RSA)) {
    		if(msg.length < pos + 4) {
    			DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseServerKeyExchange: DigitallySigned length failed, " + alg.name());
    			return null;
    		}
    		len = HandshakeMessage.ntohs(msg,pos+2);
    		if(msg.length < pos + 4 + len) {
    			DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseServerKeyExchange: DigitallySigned failed, " + alg.name());
    			return null;
			}
    	
    		byte[] signature = new byte[len];
    		System.arraycopy(msg, pos + 4, signature, 0, len);
    		ds = new DigitallySigned(msg[pos], msg[pos+1], signature);
    		pos += 4 + len;
    	}
    	
		if(msg.length != pos) {
			DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseServerKeyExchange: Full length failed");
			return null;
		}

		hm = new ServerKeyExchange(alg,dh_p,dh_g,dh_Ys,ds);
		hm.net_msg = msg;

    	return hm;
    }
    	
    public static HandshakeMessage parseCertificateRequest(byte[] msg) {
    	HandshakeMessage hm = null;
    	int pos = 0, i;
    	int alglen, dnlen;

		// parse certificate_types
    	if(msg.length < 5) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificateRequest: certificate_types length failed");
    		return null;
		}
    	if(msg[4] > msg.length - 5) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificateRequest: certificate_types failed");
    		return null;
		}
    	byte[] cer_types = new byte[msg[4]];
    	System.arraycopy(msg, 5, cer_types, 0, msg[4]);
    	pos = 5 + msg[4];
    	
		// parse supported_signature_algorithms
    	if(msg.length < pos + 2) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificateRequest: algorithms length failed");
    		return null;
		}

    	alglen = HandshakeMessage.ntohs(msg,pos);
		pos += 2;
    	if(msg.length < pos + alglen) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificateRequest: algorithms failed");
    		return null;
		}
    	if(alglen % 2 != 0) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificateRequest: algorithms length must be even");
    		return null;
		}
    	
    	SignatureAndHashAlgorithm[] algs = new SignatureAndHashAlgorithm[alglen/2];
    	for(i=0;i<algs.length;i++)
    		algs[i] = new SignatureAndHashAlgorithm(msg[pos++],msg[pos++]);
    	
		// parse certificate_authorities
    	if(msg.length < pos + 2) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificateRequest: authorities length failed");
    		return null;
		}
    	alglen = HandshakeMessage.ntohs(msg,pos);
		pos += 2;
    	if(msg.length != pos + alglen) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificateRequest: authorities failed");
    		return null;
		}

		ArrayList auth_list = new ArrayList();
		String cur_auth;
		
    	while(alglen > 0) {
    		if(alglen < 2) {
        		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificateRequest: DistinguishedName length failed");
        		return null;
    		}
    		dnlen = HandshakeMessage.ntohs(msg,pos);
    		pos += 2;
    		if(alglen < 2 + dnlen) {
        		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificateRequest: DistinguishedName failed");
        		return null;
    		}
    		cur_auth = new String(msg,pos,dnlen);
    		auth_list.add(cur_auth);
    		pos += dnlen;
    		alglen -= 2 + dnlen;
    	}
    	
    	String [] auth_list_array = new String [auth_list.size()];
    	for(i=0; i<auth_list.size(); i++)
    		auth_list_array[i] = (String)auth_list.get(i);
    	hm = new CertificateRequest(cer_types, algs, auth_list_array);
		hm.net_msg = msg;

		return hm;
    }
    	
    public static HandshakeMessage parseServerHelloDone(byte[] msg) {
    	HandshakeMessage hm = null;

    	if(msg.length == 4) {
    		hm = new ServerHelloDone();
			hm.net_msg = msg;
		}
    	else {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseServerHelloDone: message length failed: " + msg.length);
    	}

    	return hm;
    }
    	
    public static HandshakeMessage parseCertificateVerify(byte[] msg) {
    	HandshakeMessage hm = null;
    	int siglen = 0;

    	// check header length: 4 bytes + 2 byte (HashAlgorithm and SignatureAlgorithm) + 2 byte (signature length)
    	if(msg.length < 8) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificateVerify: message length failed");
    		return null;
		}

    	// check signature length
    	siglen = HandshakeMessage.ntohs(msg,6);
    	if(msg.length != siglen + 8) {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseCertificateVerify: signature length failed");
    		return null;
		}

    	byte[] signature = new byte[siglen];
    	System.arraycopy(msg, 8, signature, 0, siglen);
    	
    	hm = new CertificateVerify(new DigitallySigned(msg[4], msg[5], signature));
		hm.net_msg = msg;

		return hm;
    }
    	
    public static HandshakeMessage parseClientKeyExchange(byte[] msg, KeyExchangeAlgorithm alg) {
    	HandshakeMessage hm = null;
    	byte[] encsec = null;
    	int pos = 0, len;
 
		switch (alg) {
		case RSA:
	    	if(msg.length < 6) {
	    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseClientKeyExchange: message header failed, " + alg.name());
	    		return null;
			}
	    	len = HandshakeMessage.ntohs(msg,4);
	    	if(msg.length != 6 + len) {
	    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseClientKeyExchange: EncryptedPreMasterSecret failed, " + alg.name());
	    		return null;
			}
	    	encsec = new byte[len];
	    	System.arraycopy(msg, 6, encsec, 0, len);
	    	break;
		case DH_DSS:
		case DH_RSA:
		case DH_ANON:
		case DHE_DSS:
		case DHE_RSA:
			if(msg.length != 4) // i.e. implicitPublicValueEncoding == false
			{
		    	if(msg.length < 6) {
		    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseClientKeyExchange: message header failed, " + alg.name());
		    		return null;
				}
		    	len = HandshakeMessage.ntohs(msg,4);
		    	if(msg.length != 6 + len) {
		    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseClientKeyExchange: ClientDiffieHellmanPublic failed, " + alg.name());
		    		return null;
				}
		    	encsec = new byte[len];
		    	System.arraycopy(msg, 6, encsec, 0, len);
			}
	    	break;
		default:
	    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseClientKeyExchange: unknown algorithm, " + alg.name());
	    		return null;
		}
		
    	hm = new ClientKeyExchange(alg,encsec,true);
		hm.net_msg = msg;

    	return hm;
    }
    	
    public static HandshakeMessage parseFinished(byte[] msg, boolean IsResponseFromServer) {
    	HandshakeMessage hm = null;

    	if(msg.length >= 4) {
        	byte[] vr = new byte[msg.length - 4];
        	System.arraycopy(msg, 4, vr, 0, msg.length - 4);
    		hm = new Finished(IsResponseFromServer,vr);
			hm.net_msg = msg;
		}
    	else {
    		DebugInfo.printMessage(4,"===Error! HandshakeMessage::parseFinished: message length failed: " + msg.length);
    	}

    	return hm;
    }

}

