package ru.ispras.telecomtesk.tlsv12.HandshakeMessages;

import java.lang.System;
import java.util.ArrayList;
import java.util.Random;

import ru.ispras.telecomtesk.tlsv12.DebugInfo;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.TLSExtensions.*;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.SignatureAndHashAlgorithm;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.HashAlgorithm;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.SignatureAlgorithm;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.TLSCipherSuite;
import ru.ispras.telecomtesk.tlsv12.RecordProtocol.ProtocolVersion;
import ru.ispras.telecomtesk.tlsv12.Mutation.MutateField;

public class Hello extends HandshakeMessage
{
	@MutateField
	public ProtocolVersion version;
	@MutateField
	public byte[] rnd = new byte[32]; // Random Hello 32 bytes
	@MutateField
	public byte[] sessionID; // 0..32 bytes
	@MutateField
	public short[] cipher_suites; // 2..2^16-2 bytes
	@MutateField
	public byte[] compression_methods; // 1..2^8-1  enum { null(0), (255) }
	@MutateField
	public ArrayList<HelloExtension> extensions; // 0..2^16-1 

	public Hello() {
		msg_type = 1;
		version = new ProtocolVersion();
		sessionID = new byte[0];
		cipher_suites = new short[1];
		compression_methods = new byte[1];
		compression_methods[0] = 0; // null method
		// set random field
		Random rndObj = new Random(System.currentTimeMillis());
		rndObj.nextBytes(rnd);
		byte[] gmt_unix_time = hton((int)(System.currentTimeMillis() / 1000));
		System.arraycopy(gmt_unix_time, 0, rnd, 0, 4);
		extensions = new ArrayList<HelloExtension>(); 
	}

	public Hello(short[] cs) {
		msg_type = 1;
		version = new ProtocolVersion();
		sessionID = new byte[0];
		cipher_suites = cs;
		compression_methods = new byte[1];
		compression_methods[0] = 0; // null method
		// set random field
		Random rndObj = new Random(System.currentTimeMillis());
		rndObj.nextBytes(rnd);
		byte[] gmt_unix_time = hton((int)(System.currentTimeMillis() / 1000));
		System.arraycopy(gmt_unix_time, 0, rnd, 0, 4);
		extensions = new ArrayList<HelloExtension>(); 
	}

	public Hello(byte[] id, short[] cs) {
		msg_type = 1;
		version = new ProtocolVersion();
		if(id != null)
			sessionID = id;
		else
			sessionID = new byte[0];
		cipher_suites = cs;
		compression_methods = new byte[1];
		compression_methods[0] = 0; // null method
		// set random field
		Random rndObj = new Random(System.currentTimeMillis());
		rndObj.nextBytes(rnd);
		byte[] gmt_unix_time = hton((int)(System.currentTimeMillis() / 1000));
		System.arraycopy(gmt_unix_time, 0, rnd, 0, 4);
		extensions = new ArrayList<HelloExtension>(); 
	}

	public Hello(byte[] r, byte[] id, short[] cs) {
		msg_type = 1;
		version = new ProtocolVersion();
		if(r != null) {
			int len = r.length >= 32 ? 32 : r.length;
			System.arraycopy(r, 0, rnd, 0, len);
		}
		if(id != null)
			sessionID = id;
		else
			sessionID = new byte[0];
		cipher_suites = cs;
		compression_methods = new byte[1];
		compression_methods[0] = 0; // null method
		extensions = new ArrayList<HelloExtension>(); 
	}

	public Hello(ProtocolVersion ver, byte[] r, byte[] id, short[] cs) {
		msg_type = 1;
		version = ver;
		if(r != null) {
			int len = r.length >= 32 ? 32 : r.length;
			System.arraycopy(r, 0, rnd, 0, len);
		}
		if(id != null)
			sessionID = id;
		else
			sessionID = new byte[0];
		cipher_suites = cs;
		compression_methods = new byte[1];
		compression_methods[0] = 0; // null method
		extensions = new ArrayList<HelloExtension>(); 
	}

	public Hello(ProtocolVersion ver, byte[] r, byte[] id, short[] cs, byte[] cm, ArrayList<HelloExtension> ext) {
		msg_type = 1;
		version = ver;
		if(r != null) {
			int len = r.length >= 32 ? 32 : r.length;
			System.arraycopy(r, 0, rnd, 0, len);
		}
		if(id != null)
			sessionID = id;
		else
			sessionID = new byte[0];
		cipher_suites = cs;
		compression_methods = cm;
		if(ext != null)
			extensions = ext;
		else
			extensions = new ArrayList<HelloExtension>(); 
	}

	public int getGMTunixTimeFromRandomValue() {
		return ntoh(rnd);
	}
	public boolean isSessionIDEmpty() {
		return (sessionID == null) || (sessionID.length == 0);
	}
	public boolean compareSessionID(byte[] id) {
		boolean res = false;
		int i;
		DebugInfo.printDump(4,sessionID,"Hello::compareSessionID, Hello id:");
		DebugInfo.printDump(4,id,"compared id:");
		if((id != null) && (sessionID != null) && (sessionID.length > 0) && (sessionID.length == id.length)){
			for(i=0,res = true; i < sessionID.length; i++)
				if(sessionID[i] != id[i]) {
					res = false;
					break;
				}
		}
		DebugInfo.printMessage(4,"compareSessionID return " + res);
		return res;
	}
	public void addExtension(HelloExtension ext) {
		extensions.add(ext);
	}
	public HelloExtension getExtension(int index) {
		HelloExtension ext = null;
		try {
			ext = extensions.get(index);
		}
		catch (Exception ignored) {}
		
		return ext;
	}
	public HelloExtension getExtension(short type) {
		HelloExtension ext = null;
		try {
			if(extensions != null)
				for(int i = 0; i<extensions.size(); i++) {
					if(extensions.get(i).type == type) {
						ext = extensions.get(i);
						break;
					}
				}
		}
		catch (Exception ignored) {}
		
		return ext;
	}
	public void removeExtension(HelloExtension ext) {
		extensions.remove(ext);
	}
	public void clearExtensionList() {
		extensions.clear();
	}
	public int getExtensionLength() {
		int ext_len = 0;
		for (int j = 0; j < extensions.size(); j++) {
			ext_len += ((HelloExtension)extensions.get(j)).getSize();
		}
		return ext_len;
	}
	public int getSize() {
		int size;
		size = 42 + sessionID.length;
		if(msg_type == 1) 
			size += 2*cipher_suites.length + compression_methods.length;
		if(getExtensionLength() > 0)
			size += 2 + getExtensionLength();
		return size;
	}
	public void setTypeClientHello() {
		msg_type = 1;
	}
	public void setTypeServerHello() {
		msg_type = 2;
	}
	public int indexOfExtension(short type) {
		int res = -1;
		
		if(extensions != null) {
			for(int i=0; i<extensions.size(); i++) {
				if(type == ((HelloExtension)extensions.get(i)).type) {
					res = i;
					break;
				}
			}
		}
		return res;
	}
	public int lastIndexOfExtension(short type) {
		int res = -1;
		
		if(extensions != null) {
			for(int i=extensions.size()-1; i>=0; i--) {
				if(type == ((HelloExtension)extensions.get(i)).type) {
					res = i;
					break;
				}
			}
		}
		return res;
	}
	public boolean checkDuplicateExtensions() {
		HelloExtension curext;
		boolean res = false;
		
		if(extensions != null) {
			for(int i=0; i<extensions.size(); i++) {
				curext = (HelloExtension)extensions.get(i);
				if(i != lastIndexOfExtension(curext.type)) {
					res = true;
					break;
				}
			}
		}
		return res;
	}
	
	public boolean checkSCSV() {
		boolean res = false;
		for(int i=0; i<cipher_suites.length; i++)
			if(TLSCipherSuite.valueOf(cipher_suites[i]) == TLSCipherSuite.TLS_EMPTY_RENEGOTIATION_INFO_SCSV) {
				res = true;
				break;
			}
		return res;
	}

	public RenegotiationExtension getRenegotiationExtension() {
		HelloExtension curext;
		RenegotiationExtension rnext = null;
		
		DebugInfo.printMessage(5,"Hello::getRenegotiationExtension start");
		if(extensions != null) {
			for(int i=0; i<extensions.size(); i++) {
				curext = (HelloExtension)extensions.get(i);
				if(curext.type == (short)0xFF01) {
					rnext = (RenegotiationExtension)curext;
					break;
				}
			}
		}
		if(rnext != null)
			DebugInfo.printMessage(4,"Hello::getRenegotiationExtension return extension");
		else
			DebugInfo.printMessage(4,"Hello::getRenegotiationExtension return null");
			
		return rnext;
	}
/* 
 * (non-Javadoc)
 * @see ru.ispras.telecomtesk.tls.HandshakeMessages.HandshakeMessage#toByteArray()
 *
 * size of Hello in bytes:
 * type + length = 4
 * version = 2
 * random = 32
 * <field 'length' of 1 byte> + sessionID = 1 + sessionID.length
 * <field 'length' of 2 byte> + cipher_suites = 2 + (2 * cipher_suites.length) // = 2 for ServerHello
 * <field 'length' of 1 byte> + compression_methods = 1 + compression_methods.length // = 1 for ServerHello
 * <??? field 'length' of 2 byte> + extensions = 2 + ...
 * 
 * ClientHello:
 * 4 + 2 + 32 + 1 + sessionID.length + 2 + 2*cipher_suites.length + 1 + compression_methods.length + 2 + <extensions>
 * 
 * ServerHello:
 * 4 + 2 + 32 + 1 + sessionID.length + 2 + 1 + 2 + <extensions>
 */
	public int toByteArray() {
		int i = 0, j, res = 0;
		if(isLengthSet == false) setLength();
		// create byte array
		byte[] ba = new byte[getSize()];
		// write msg_type, full length and version
		byte[] ba_len = hton(length);
		ba[0] = msg_type;
		System.arraycopy(ba_len, 1, ba, 1, 3);
		i=4;
		ba[i++] = version.major;
		ba[i++] = version.minor;
		// write RandomHello rnd
		System.arraycopy(rnd, 0, ba, i, rnd.length);
		i+=rnd.length;
		// write sessionID
		ba[i++] = (byte) sessionID.length;
		System.arraycopy(sessionID, 0, ba, i, (byte)sessionID.length);
		i+= (byte)sessionID.length;
		if(msg_type != 2) {
			// write cipher_suites
			j = 2 * cipher_suites.length;
			ba[i++] = (byte)(j >>> 8);
			ba[i++] = (byte)j;
			for (j = 0; j < cipher_suites.length; j++) {
				ba[i++] = (byte)(cipher_suites[j] >>> 8);
				ba[i++] = (byte)cipher_suites[j];
			}
			// write compression_methods
			ba[i++] = (byte) compression_methods.length;
			System.arraycopy(compression_methods, 0, ba, i, (byte)compression_methods.length);
			i+= (byte)compression_methods.length;
		}
		else {
			// write cipher_suites
			ba[i++] = (byte)(cipher_suites[0] >>> 8);
			ba[i++] = (byte)cipher_suites[0];
			// write compression_methods
			ba[i++] = (byte) compression_methods[0];
		}
		//write extensions
		j = getExtensionLength();
		if(j > 0) {
			ba[i++] = (byte)(j >>> 8);
			ba[i++] = (byte)j;
			byte[] ext;
			for (j = 0; j < extensions.size(); j++) {
				ext = ((HelloExtension)extensions.get(j)).toByteArray();
				System.arraycopy(ext, 0, ba, i, ext.length);
				i += ext.length;
			}
		}
		
		net_msg = ba;
		return res;
	}

	public String toString() {
		String res;

		if(msg_type == 1) {
			res = "HandshakeMessage: type ClientHello, length " + length;
			if(net_msg != null)
				res += ", net_msg: " + net_msg.length;
		}
		else if(msg_type == 2) {		
                	int i;
			res = "HandshakeMessage: type ServerHello, length " + length;
			if(net_msg != null)
				res += ", net_msg: " + net_msg.length;

			res += "\n      ProtocolVersion: " + version.major + "." + version.minor;
			res += "\n      cipher_suites: ";
			for(i=0;i<cipher_suites.length;i++) {
				res += TLSCipherSuite.valueOf(cipher_suites[i]) + ";";
			}

			res += "\n      compression_methods: ";

			for(i=0;i<compression_methods.length;i++) {
				res += compression_methods[i] + ";";
			}

		}
		else {		
			res = "HandshakeMessage: Hello with type " + msg_type + ", length " + length;
			if(net_msg != null)
				res += ", net_msg: " + net_msg.length;
		}

		return res;
	}

	public void print() {
		int i,j;
		DebugInfo.printMessage(1,"\nHandshake Hello Message:");
		DebugInfo.printMessage(1,"ProtocolVersion: " + version.major + " " + version.minor);
		DebugInfo.printDump(1,rnd,"Random:");
		DebugInfo.printDump(1,sessionID,"session ID:");
		DebugInfo.printMessage(1,"cipher_suites, " + cipher_suites.length + ":");
		for(i=0;i<cipher_suites.length;i++) {
			DebugInfo.printMessage(1,"   " + TLSCipherSuite.toString(cipher_suites[i]));
		}
		DebugInfo.printMessage(1,"compression_methods, " + compression_methods.length + ":");
		System.out.print("   ");
		for(i=0;i<compression_methods.length;i++) {
			System.out.printf("%d ",compression_methods[i]);
		}
		DebugInfo.printMessage(1,"\nextensions, " + this.extensions.size() + ":");
		HelloExtension ext;
		for(i=0;i<this.extensions.size();i++) {
			ext = this.extensions.get(i);
			if(ext.type == 13) {
				SignatureAndHashAlgorithmExt sahext = (SignatureAndHashAlgorithmExt)ext;
				DebugInfo.printMessage(1,"type: Signature Algorithms");
                for(j=0;j<sahext.data.length;j++) {
					DebugInfo.printMessage(1,"   " + sahext.data[j].toString());
				}
			}
			else if(ext.type == (short)(0xFF01)) {
				DebugInfo.printDump(1,ext.data,"type: Renegotiation, data:");
			}
			else {
				DebugInfo.printDump(1,ext.data,"type: " + ext.type + ", data:");
			}
		}
	}
}

