package ru.ispras.telecomtesk.tlsv12.HandshakeMessages.TLSExtensions;

import ru.ispras.telecomtesk.tlsv12.DebugInfo;

public class RenegotiationExtension extends HelloExtension
{
	public RenegotiationExtension() {
		type = (short)(0xFF01); length = 1; data = new byte[0];
	}
	public RenegotiationExtension(byte[] data) {
		this.type = (short)(0xFF01); 
		if(data != null  &&  data.length > 0) {
			this.data = new byte[(byte)data.length];
			System.arraycopy(data, 0, this.data, 0, (byte)data.length);
		}
		else 
			data = new byte[0];

		this.length = (short)(1 + data.length); 
	}
	public void setLength(short len) {
		length = len;
	}
	public short getSize() {
		return (short)(5 + data.length);
	}
	public boolean isEmpty() {
		return data.length == 0;
	}
	public boolean checkData(byte[] clientVerifyData) {
		boolean res = false;
		int i;

		DebugInfo.printMessage(5, "RenegotiationExtension::checkData");
		if(data != null  &&  clientVerifyData != null
		  && data.length == clientVerifyData.length) {
			for(res=true, i=0; i<data.length; i++)
				if(data[i] != clientVerifyData[i])  {
					res = false;
					break;
				}
		}
		DebugInfo.printMessage(4, "RenegotiationExtension::checkData return " + res);
		return res;
	}
	public boolean checkData(byte[] clientVerifyData, byte[] serverVerifyData) {
		boolean res = false;
		int i,j;

		DebugInfo.printMessage(5, "RenegotiationExtension::checkData");
		if(data != null  &&  clientVerifyData != null
		  && data.length == clientVerifyData.length + serverVerifyData.length) {
			res=true;
			for(i=0; i<clientVerifyData.length; i++)
				if(clientVerifyData[i] != data[i])  {
					DebugInfo.printMessage(4, "RenegotiationExtension::checkData clientVerifyData failed");
					res = false;
					break;
				}
			for(j=0; j<serverVerifyData.length; j++,i++)
				if(serverVerifyData[j] != data[i])  {
					DebugInfo.printMessage(4, "RenegotiationExtension::checkData serverVerifyData failed");
 					res = false;
					break;
				}
		}
		return res;
	}

	public byte[] toByteArray() {
		int i = 0;
		short len = length;
		byte[] res = new byte[getSize()];
		res[i++] = (byte)(type >>> 8);
		res[i++] = (byte)type;
		res[i++] = (byte)(len >>> 8);
		res[i++] = (byte)len;
		len --;
		res[i++] = (byte)len;
		if(data.length > 0)
			System.arraycopy(data, 0, res, 5, (byte)data.length);
		return res;
	}
}
