package ru.ispras.telecomtesk.tlsv12.RecordProtocol;

import java.util.ArrayList;

import ru.ispras.telecomtesk.tlsv12.DebugInfo;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.HandshakeMessage;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.*;
import ru.ispras.telecomtesk.tlsv12.TLSState.TLSConnection;
import ru.ispras.telecomtesk.tlsv12.TLSState.TLSState;

public class RecordLayer 
{
	public TLSContentType tls_type;
	public ProtocolVersion version;
	protected short length;
	public byte[][] content;
	protected boolean isLengthSet = false;
	public boolean incompleteArrayMessage = false;
	public int error = 0;
	public ArrayList msglist;
	
	public RecordLayer() {
		tls_type = TLSContentType.HANDSHAKE; 
		version = new ProtocolVersion();
	}
	public RecordLayer(TLSContentType type) {
		tls_type = type; 
		version = new ProtocolVersion();
	}
	public RecordLayer(TLSContentType type, ProtocolVersion ver) {
		tls_type = type; 
		version = new ProtocolVersion(ver.major, ver.minor);
	}

	public RecordLayer(TLSContentType type, int ver_major, int ver_minor) {
		tls_type = type; 
		version = new ProtocolVersion(ver_major, ver_minor);
	}

	public String toString() {
		return "RecordLayer: tls_type " + tls_type + ", ProtocolVersion " + version.major + " " + version.minor + ", length " + length;
	}

	public void print() {
		DebugInfo.printMessage(1,"RecordLayer::print");
		DebugInfo.printMessage(1,"   tls_type " + this.tls_type);
		DebugInfo.printMessage(1,"   length " + this.length);
		if(this.content != null) {
			DebugInfo.printMessage(1,"   content size " + this.content.length);
			for(int i=0; i<this.content.length; i++) {
				if(this.content[i] != null)
					DebugInfo.printMessage(1,"      content " + i + " size "+ this.content[i].length);
			}
		} else
			DebugInfo.printMessage(1,"   content null");
			
		DebugInfo.printMessage(1,"   isLengthSet " + this.isLengthSet);
		DebugInfo.printMessage(1,"   incompleteArrayMessage " + this.incompleteArrayMessage);
		DebugInfo.printMessage(1,"   error " + this.error);
		if(msglist != null)
			DebugInfo.printMessage(1,"   msglist size " + this.msglist.size());
		else
			DebugInfo.printMessage(1,"   msglist null" );
	}

	public int getSize() {
		if(content != null) {
			int contlen = 0;
			for(int i=0; i<content.length; i++){
				if(content[i] != null) 
					contlen += content[i].length;
			}
			return (5 + (short)contlen);
		}
		else
			return 5;
	}
	public void setLength() {
		length = (short)(getSize() - 5);
	}
	public void setLength(int len) {
		length = (short)len;
		isLengthSet = true;
	}
	public short getLength() {
		return length;
	}

	public byte[] toByteArrayOnlyHeader() {
		byte[] ba = new byte[5];
		ba[0] = tls_type.type;
		ba[1] = version.major;
		ba[2] = version.minor;
		ba[3] = (byte)(length >>> 8);
		ba[4] = (byte)length;
		return ba;
	}

	public byte[] toByteArray() {
		if(isLengthSet == false) setLength();
		byte[] ba = new byte[getSize()];
		ba[0] = tls_type.type;
		ba[1] = version.major;
		ba[2] = version.minor;
		ba[3] = (byte)(length >>> 8);
		ba[4] = (byte)length;
		for(int i=0,curpos=5; i<content.length; i++){
			if(content[i] != null) {
				System.arraycopy(content[i], 0, ba, curpos, (short)content[i].length);
				curpos += (short)content[i].length;
			}
		}
		return ba;
	}

	public void setNewContent(byte[][] data) {
		content = data;
	}

	public void setNewContent(byte[] data) {
		content = new byte[1][];
		content[0] = data;
	}

	public void initContent(int len) {
		content = new byte[len][];
		for(int i=0; i<content.length; i++) 
			content[i] = null;
	}

	public void setContent(byte[] text, byte[] padding, TLSConnection con) throws Exception {
		byte[] text_stream, enc_stream;
		int len, content_ind = 0, curlen, offset;
		boolean isBigDataLength = false;

		DebugInfo.printMessage(4,"RecordLayer::setContent");
		
		if((text == null) || (con == null)) return;
		
		DebugInfo.printDump(4,text,"RecordLayer::setContent data");

		CipherType ct = CipherType.unknown;
		MACAlgorithm ma = MACAlgorithm.mac_null;
		TLSState st = con.getWriteCurrentState();
		
		/*
		 *  create buffer for Content
		 *  
		 *  the length (in bytes) of the TLSPlaintext.fragment MUST NOT exceed 2^14 (=16384)
		 *  the length of the TLSCiphertext.fragment MUST NOT exceed 2^14 + 2^11(=2048)
		 *  
		 *  Number of fragment blocks equals len = (text.length / 16384 + 1)
		 *  TLSCiphertext may add len*2048 bytes, i.e. in blocks (len*2048)/16384 + 1 = len/8 + 1
		 *  All blocks: len + len/8 + 1, where len = (text.length / 16384 + 1)
		 */
		len = (text.length / 16384 + 1);
		len += len/8 + 1;
		initContent(len);
		
		if((st != null)&&(st.params != null)) {
			ct = st.params.cipher_type;
			ma = st.params.mac_alg;
		}
		
		if(text.length > 16384) {
			text_stream = new byte[16384];
			isBigDataLength = true;
		}
		else 
			text_stream = text;

		
		offset=0; len=0;

		do {
			// fill buffer text_stream from text
			if(isBigDataLength) {
				curlen = text.length - offset;
				if(curlen == 0) break;
				
				if(curlen <= 16384) {
					text_stream = new byte[curlen];
					System.arraycopy(text, offset, text_stream, 0, curlen);
					offset = text.length;
				}
				else {
					System.arraycopy(text, offset, text_stream, 0, 16384);
					offset += 16384;
				}
			}

			
		// compress cur_stream
			enc_stream = st.compress(text_stream);
		// encrypt compressed stream with MAC
			if((ct == CipherType.stream || ct == CipherType.block) && (ma != null)) {
				// generate MAC
				byte[] sequence_number = HandshakeMessage.hton(st.getSequenceNumber());
				byte[] tls_header = {tls_type.type, version.major, version.minor, (byte)(enc_stream.length >>> 8), (byte)enc_stream.length};
                byte[][] mac_buf = {sequence_number, tls_header, enc_stream};
                byte[] tls_mac = st.mac(mac_buf);
        		DebugInfo.printDump(5,tls_mac,"RecordLayer::setContent MAC");
                
                if(tls_mac != null) {
                	byte[][] tls_buf = {enc_stream, tls_mac};
                	enc_stream = st.encrypt(tls_buf, padding);
            		DebugInfo.printDump(5,enc_stream,"RecordLayer::setContent ENCRYPT");
                }
                else {
                	content = null;
                	System.out.println("RecordLayer::setContent: Calculate MAC failed");
                	System.out.println("RecordLayer::setContent: fragment index: " + content_ind);
                	return;
                }
			}
			else {
				enc_stream = st.encrypt(enc_stream, padding);
        		DebugInfo.printDump(5,enc_stream,"RecordLayer::setContent ENCRYPT");
			}
			
			if(enc_stream != null)
				content[content_ind++] = enc_stream;

		} while(isBigDataLength);
	}

	public void setContent(byte[][] text, byte[] padding, TLSConnection con) throws Exception {
		byte[] text_stream = new byte[16384];
		byte[]  enc_stream, tmp_stream;
		int len, content_ind = 0, curlen, full_len, offset, i;
		
		if((text == null) || (con == null)) return;
		
		CipherType ct = CipherType.unknown;
		MACAlgorithm ma = MACAlgorithm.mac_null;
		TLSState st = con.getWriteCurrentState();

		// Calculate full input data length
		for(i=0,full_len=0;i<text.length;i++) {
			if(text[i] != null) {
				DebugInfo.printMessage(6,"RecordLayer::setContent(byte[][]): input data");
				DebugInfo.printDump(6, text[i], null);
				full_len += text[i].length;
			}
		}

		/*
		 *  create buffer for Content
		 *  
		 *  the length (in bytes) of the TLSPlaintext.fragment MUST NOT exceed 2^14 (=16384)
		 *  the length of the TLSCiphertext.fragment MUST NOT exceed 2^14 + 2^11(=2048)
		 *  
		 *  Number of fragment blocks equals len = (text.length / 16384 + 1)
		 *  TLSCiphertext may add len*2048 bytes, i.e. in blocks (len*2048)/16384 + 1 = len/8 + 1
		 *  All blocks: len + len/8 + 1, where len = (text.length / 16384 + 1)
		 */
		len = (full_len / 16384 + 1);
		len += len/8 + 1;
		initContent(len);
		
		if((st != null)&&(st.params != null)) {
			ct = st.params.cipher_type;
			ma = st.params.mac_alg;
		}
		
		i=0; offset=0; len=0;
	
		while(true) {
			// fill buffer text_stream from text
			while(i < text.length && len < 16384) {
				if(text[i] == null) {i++; continue;}
				curlen = text[i].length - offset;
				if(len + curlen  <= 16384) {
					System.arraycopy(text[i], offset, text_stream, len, curlen);
					i++; 
					if(offset != 0) offset=0;
					len += curlen;
				}
				else {
					System.arraycopy(text[i], offset, text_stream, len, 16384 - len);
					offset += 16384 - len;
					len = 16384;
				}
			}
			if(len == 0) break;
			if(len < 16384) {
				tmp_stream = new byte[len];
				System.arraycopy(text_stream, 0, tmp_stream, 0, len);
				text_stream = tmp_stream;
			}
			
		// compress cur_stream
			enc_stream = st.compress(text_stream);
		// encrypt compressed stream with MAC
			if((ct == CipherType.stream || ct == CipherType.block) && (ma != null)) {
				// generate MAC
				byte[] sequence_number = HandshakeMessage.hton(st.getSequenceNumber());
				byte[] tls_header = {tls_type.type, version.major, version.minor, (byte)(enc_stream.length >>> 8), (byte)enc_stream.length};
                byte[][] mac_buf = {sequence_number, tls_header, enc_stream};
                byte[] tls_mac = st.mac(mac_buf);
                
                if(tls_mac != null) {
                	byte[][] tls_buf = {enc_stream, tls_mac};
                	enc_stream = st.encrypt(tls_buf, padding);
                }
                else {
                	content = null;
                	System.out.println("RecordLayer::setContent: Calculate MAC failed");
                	System.out.println("RecordLayer::setContent: fragment index: " + content_ind);
                	return;
                }
			}
			else
				enc_stream = st.encrypt(enc_stream, padding);
			
			if(enc_stream != null)
				content[content_ind++] = enc_stream;

			len = 0;
		}
	}

	// same as setContent, but length may exceed 2^14 (=16384) 
	public void setLargeContent(byte[] text, byte[] padding, TLSConnection con) throws Exception {
		byte[] enc_stream;

		DebugInfo.printMessage(4,"RecordLayer::setLargeContent");
		
		if((text == null) || (con == null)) return;
		
		DebugInfo.printDump(4,text,"RecordLayer::setLargeContent data");

		CipherType ct = CipherType.unknown;
		MACAlgorithm ma = MACAlgorithm.mac_null;
		TLSState st = con.getWriteCurrentState();
		
		/*
		 *  The only content block any length
		 */
		initContent(1);
		
		if((st != null)&&(st.params != null)) {
			ct = st.params.cipher_type;
			ma = st.params.mac_alg;
		}
		
		// compress cur_stream
		enc_stream = st.compress(text);
		// encrypt compressed stream with MAC
		if((ct == CipherType.stream || ct == CipherType.block) && (ma != null)) {
			// generate MAC
			byte[] sequence_number = HandshakeMessage.hton(st.getSequenceNumber());
			byte[] tls_header = {tls_type.type, version.major, version.minor, (byte)(enc_stream.length >>> 8), (byte)enc_stream.length};
            byte[][] mac_buf = {sequence_number, tls_header, enc_stream};
            byte[] tls_mac = st.mac(mac_buf);
        	DebugInfo.printDump(5,tls_mac,"RecordLayer::setLargeContent MAC");
                
            if(tls_mac != null) {
               	byte[][] tls_buf = {enc_stream, tls_mac};
               	enc_stream = st.encrypt(tls_buf, padding);
            	DebugInfo.printDump(5,enc_stream,"RecordLayer::setLargeContent ENCRYPT");
            }
            else {
               	content = null;
               	System.out.println("RecordLayer::setLargeContent: Calculate MAC failed");
               	return;
            }
		}
		else {
			enc_stream = st.encrypt(enc_stream, padding);
        	DebugInfo.printDump(5,enc_stream,"RecordLayer::setLargeContent ENCRYPT");
		}
			
		if(enc_stream != null)
			content[0] = enc_stream;
	}

	// same as setContent, but length may exceed 2^14 (=16384) 
	public void setLargeContent(byte[][] text, byte[] padding, TLSConnection con) throws Exception {
		byte[] text_stream;
		byte[]  enc_stream;
		int full_len, offset, i;
		
		if((text == null) || (con == null)) return;
		
		CipherType ct = CipherType.unknown;
		MACAlgorithm ma = MACAlgorithm.mac_null;
		TLSState st = con.getWriteCurrentState();

		// Calculate full input data length
		for(i=0,full_len=0;i<text.length;i++) {
			if(text[i] != null)
				full_len += text[i].length;
		}
		text_stream = new byte[full_len];
		
		// fill buffer text_stream from text
		for(i=0,offset=0;i<text.length;i++) {
			if(text[i] != null) {
				System.arraycopy(text[i], 0, text_stream, offset, text[i].length);
				offset += text[i].length;
			}
		}

		/*
		 *  The only content block any length
		 */
		initContent(1);
		
		if((st != null)&&(st.params != null)) {
			ct = st.params.cipher_type;
			ma = st.params.mac_alg;
		}
		
		// compress cur_stream
		enc_stream = st.compress(text_stream);
		// encrypt compressed stream with MAC
		if((ct == CipherType.stream || ct == CipherType.block) && (ma != null)) {
			// generate MAC
			byte[] sequence_number = HandshakeMessage.hton(st.getSequenceNumber());
			byte[] tls_header = {tls_type.type, version.major, version.minor, (byte)(enc_stream.length >>> 8), (byte)enc_stream.length};
            byte[][] mac_buf = {sequence_number, tls_header, enc_stream};
            byte[] tls_mac = st.mac(mac_buf);
                
            if(tls_mac != null) {
               	byte[][] tls_buf = {enc_stream, tls_mac};
               	enc_stream = st.encrypt(tls_buf, padding);
            }
            else {
               	content = null;
               	System.out.println("RecordLayer::setLargeContent: Calculate MAC failed");
               	return;
            }
		}
		else
			enc_stream = st.encrypt(enc_stream, padding);
			
		if(enc_stream != null)
			content[0] = enc_stream;
	}

	public void setContent(Object msg, byte[] padding, TLSConnection con) throws Exception {
		if(msg != null) {
			if(msg instanceof HandshakeMessage) {
				tls_type = TLSContentType.HANDSHAKE;
				if( ((HandshakeMessage)msg).net_msg == null)
					((HandshakeMessage)msg).toByteArray();
				setContent(((HandshakeMessage)msg).net_msg, padding, con);
			}
			else if(msg instanceof Alert) {
				tls_type = TLSContentType.ALERT;
				setContent(((Alert)msg).toByteArray(), padding, con);
			}
			else if(msg instanceof ChangeCipher) {
				tls_type = TLSContentType.CHANGE_CIPHER_SPEC;
				setContent(((ChangeCipher)msg).toByteArray(), padding, con);
			}
			else if(msg instanceof ApplicationData) {
				tls_type = TLSContentType.APPLICATION;
				setContent(((ApplicationData)msg).data, padding, con);
			}
			else if(msg instanceof byte[]) {
				tls_type = TLSContentType.APPLICATION;
				setContent((byte[])msg, padding, con);
			}
			else if(msg instanceof byte[][]) {
				tls_type = TLSContentType.APPLICATION;
				setContent((byte[][])msg, padding, con);
			}
		}
	}

	public void setLargeContent(Object msg, byte[] padding, TLSConnection con) throws Exception {
		if(msg != null) {
			if(msg instanceof HandshakeMessage) {
				tls_type = TLSContentType.HANDSHAKE;
				if( ((HandshakeMessage)msg).net_msg == null)
					((HandshakeMessage)msg).toByteArray();
				setLargeContent(((HandshakeMessage)msg).net_msg, padding, con);
			}
			else if(msg instanceof Alert) {
				tls_type = TLSContentType.ALERT;
				setLargeContent(((Alert)msg).toByteArray(), padding, con);
			}
			else if(msg instanceof ChangeCipher) {
				tls_type = TLSContentType.CHANGE_CIPHER_SPEC;
				setLargeContent(((ChangeCipher)msg).toByteArray(), padding, con);
			}
			else if(msg instanceof ApplicationData) {
				tls_type = TLSContentType.APPLICATION;
				setLargeContent(((ApplicationData)msg).data, padding, con);
			}
			else if(msg instanceof byte[]) {
				tls_type = TLSContentType.APPLICATION;
				setLargeContent((byte[])msg, padding, con);
			}
			else if(msg instanceof byte[][]) {
				tls_type = TLSContentType.APPLICATION;
				setLargeContent((byte[][])msg, padding, con);
			}
		}
	}

	public void setContent(Object msg, byte[] padding, TLSConnection con, TLSContentType type) throws Exception {
		if(msg != null) {
			tls_type = type;
			
			if(msg instanceof HandshakeMessage) {
				if( ((HandshakeMessage)msg).net_msg == null)
					((HandshakeMessage)msg).toByteArray();
				setContent(((HandshakeMessage)msg).net_msg, padding, con);
			}
			else if(msg instanceof Alert) {
				setContent(((Alert)msg).toByteArray(), padding, con);
			}
			else if(msg instanceof ChangeCipher) {
				setContent(((ChangeCipher)msg).toByteArray(), padding, con);
			}
			else if(msg instanceof ApplicationData) {
				setContent(((ApplicationData)msg).data, padding, con);
			}
			else if(msg instanceof byte[]) {
				setContent((byte[])msg, padding, con);
			}
			else if(msg instanceof byte[][]) {
				setContent((byte[][])msg, padding, con);
			}
		}
	}

	public void setContent(Class cl, ArrayList msglist, int offset, int len, byte[] padding, TLSConnection con) throws Exception {
		int data_ind, i, curlen;
		if((cl != null)&&(msglist != null)&&(offset < msglist.size())) {
			Object msg;
			for(data_ind=0,i=offset,curlen=len;i < msglist.size() && curlen>0; i++,curlen--) {
				msg = msglist.get(i);
				if(cl.isAssignableFrom(msg.getClass())) data_ind++;
			}
			if(data_ind == 0) return;
			
			DebugInfo.printMessage(3,"RecordLayer::setContent(Class): find " + data_ind + " messages");
			byte[][] data = new byte[data_ind][];
			i=offset; data_ind=0;
			
			if(HandshakeMessage.class.isAssignableFrom(cl)) {
				tls_type = TLSContentType.HANDSHAKE;
				for(;i < msglist.size() && len>0; i++,len--) {
					msg = msglist.get(i);
					if(msg instanceof HandshakeMessage) {
						if( ((HandshakeMessage)msg).net_msg == null)
							((HandshakeMessage)msg).toByteArray();
						data[data_ind++] = ((HandshakeMessage)msg).net_msg;
					}
				}
			}
			else if(Alert.class.isAssignableFrom(cl)) {
				tls_type = TLSContentType.ALERT;
				for(;i < msglist.size() && len>0; i++,len--) {
					msg = msglist.get(i);
					if(msg instanceof Alert) {
						data[data_ind++] = ((Alert)msg).toByteArray();
					}
				}
			}
			else if(ChangeCipher.class.isAssignableFrom(cl)) {
				tls_type = TLSContentType.CHANGE_CIPHER_SPEC;
				for(;i < msglist.size() && len>0; i++,len--) {
					msg = msglist.get(i);
					if(msg instanceof ChangeCipher) {
						data[data_ind++] = ((ChangeCipher)msg).toByteArray();
					}
				}
			}
			else if(ApplicationData.class.isAssignableFrom(cl)) {
				tls_type = TLSContentType.APPLICATION;
				for(;i < msglist.size() && len>0; i++,len--) {
					msg = msglist.get(i);
					if(msg instanceof ApplicationData) {
						data[data_ind++] = ((ApplicationData)msg).data;
					}
				}
			}
			else
				return;
			
			setContent(data, padding, con);
		}
	}

	public void printMessages(int debugLevel) {
		DebugInfo.printMessage(debugLevel, toString());
		Object msg;
		if(msglist != null  &&  !msglist.isEmpty()) {
			for(int i=0; i < msglist.size(); i++) {
				msg = msglist.get(i);
				if( (msg instanceof HandshakeMessage)
				 || (msg instanceof Alert)
				 || (msg instanceof ChangeCipher)
				 || (msg instanceof ApplicationData)) {
						DebugInfo.printMessage(debugLevel, "   " + msg.toString());
				}
				else
					DebugInfo.printMessage(debugLevel, "   message " + i + ": unknown type");
			}
		}
	}

    // decrypt one TLS message
    public int getTlsPlain(TLSState curstate) throws Exception {
    	error = 0;
    	if(curstate != null && content != null && content[0] != null) {
 			content[0] = curstate.decrypt(content[0]);
   			if(content[0] == null) {
   				error = -1;
   				DebugInfo.printMessage(2,"===Error! RecordLayer::getTlsPlain: Decrypt message failed");
   			}
    	}
    	else {
				error = -2;
   				DebugInfo.printMessage(3,"===Error! RecordLayer::getTlsPlain: Bad input data");
    	}
    	return error;
    }
    
    //  check and trim Padding and MAC
    public int trimPaddingAndMAC(TLSState curstate) throws Exception {
    	error = 0; // bad_record_mac alert
    	byte[] data;
    	int len; 
    	CipherType ct = curstate.params.cipher_type;
    	
    	if(curstate != null && content != null && content[0] != null) {
    		data = content[0];
    		DebugInfo.printDump(4,data,null);
   			if(ct == CipherType.stream || ct == CipherType.block) {
   				len = data.length;
   				// check Padding
   				if(curstate.params.cipher_type == CipherType.block) {
   					byte padlen = data[data.length - 1];
   					if(  padlen > data.length - 1
   					  || false == TLSState.checkPadding(data, data.length - padlen - 1)) {
   		   				DebugInfo.printMessage(2,"===Error! RecordLayer::trimPaddingAndMAC: bad padding: len " + padlen);
   		   				error = -1;
   		   				return error;
   					}
   					len -= padlen + 1; // length without padding
   				}
   				// check MAC
   				if(len < curstate.params.mac_length) {
	   				DebugInfo.printMessage(2,"===Error! RecordLayer::trimPaddingAndMAC: bad MAC length: " + len + " , need " + curstate.params.mac_length);
		   			error = -2;
					return error;
				}
   				len -= curstate.params.mac_length; // length of data without MAC
				byte[] seq_num = HandshakeMessage.hton(curstate.getSequenceNumber());
				byte[] tls_data = new byte[len];
				if(len > 0)
					System.arraycopy(data, 0, tls_data, 0, len);
				byte[] tls_header = {tls_type.type, version.major, version.minor, (byte)(len >>> 8), (byte)len};
                byte[][] mac_buf = {seq_num, tls_header, tls_data};
                byte[] tls_mac = curstate.mac(mac_buf);
                
                content[0] = tls_data;
                
                if(tls_mac != null && 
                   tls_mac.length == curstate.params.mac_length) {
                		for(int i=0; i<tls_mac.length; i++)
                			if(tls_mac[i] != data[len + i]) {
                				DebugInfo.printMessage(2, "RecordLayer::trimPaddingAndMAC: bad MAC");
                				DebugInfo.printDump(3, tls_mac, "Calculate MAC:");
                				DebugInfo.printDump(3, data, len, tls_mac.length, "\nneed MAC:");
           		   				error = -3;
                				return error;
                			}
                }
                else {
        			DebugInfo.printMessage(2,"===Error! RecordLayer::trimPaddingAndMAC: calculate MAC failed");
		   			error = -4;
					return error;
				}
   			}
    	}
    	else {
			error = -5;
			DebugInfo.printMessage(3,"===Error! RecordLayer::trimPaddingAndMAC: Bad input data");
    	}
    	return error;
    }
    
    // parse one TLS message
    public int parseTlsPlain(TLSState pending_state) {
    	error = 0;
    	int j;
   		msglist = new ArrayList();

   		if(content != null && content[0] != null) {
   			switch(tls_type) {
   				case CHANGE_CIPHER_SPEC:
   					for(j=0; j < content[0].length; j++)
   						msglist.add(new ChangeCipher());
   					break;
   				case ALERT:
   					int len = content[0].length;
   					if(len % 2 != 0) {
   						len--;
   						DebugInfo.printMessage(1,"===Error! RecordLayer::parseTlsPlain: ALERT incomplete data");
   						error = -1;
   					}
   					for(j=0; j < len; )
   						msglist.add(new Alert(content[0][j++],content[0][j++]));
   					break;
   				case HANDSHAKE:
   					error = HandshakeMessage.parseHandshakeMessagesList(content[0], msglist, pending_state.getParams());
   					break;
   				case APPLICATION:
					msglist.add(new ApplicationData(content[0]));
   					break;
   				default:	
					DebugInfo.printMessage(3,"===Error! RecordLayer::parseTlsPlain: unknown message type " + tls_type);
   					error = -1;
   			}
   		}
   		return error;
    }

	public int decryptAndParseTLS(TLSConnection con) {
		int res = -1;
		DebugInfo.printMessage(4,"RecordLayer::decryptAndParseTLS");

		if(con == null) {
			DebugInfo.printMessage(4,"connection not specified");
			return res;
		}
		
		if(version.major != 3) {
			DebugInfo.printMessage(1,"===Error! Bad major TLS version: " + version.major + " " + version.minor);
			DebugInfo.printDump(1,toByteArrayOnlyHeader(),"TLS header:");
			DebugInfo.printDump(1,content[0],"TLS data:");
    		res = 70;
   			return res;
		}

		if(version.minor != 3) {
			DebugInfo.printMessage(1,"===Warning! Bad minor TLS version: " + version.major + " " + version.minor);
			DebugInfo.printDump(1,toByteArrayOnlyHeader(),"TLS header:");
			DebugInfo.printDump(1,content[0],"TLS data:");
    		if((tls_type != TLSContentType.ALERT) && (tls_type != TLSContentType.HANDSHAKE)) {
        		res = 70;
    			return res;
    		}
		}

		// decrypt TLS record
		DebugInfo.printMessage(4,"try decrypt message");
		try {
			getTlsPlain(con.getReadCurrentState());
	    	if(error != 0) {
	    		res = 51;
	    		return res;
	    	}
		}
		catch (Exception err) {
			DebugInfo.printMessage(1,"===Error! RecordLayer::getTlsPlain: " + err);
    		res = 51;
    		return res;
		}

		// check and trim Padding and MAC
		DebugInfo.printMessage(3,"trim padding and MAC");
		try {
			trimPaddingAndMAC(con.getReadCurrentState());
	    	if(error != 0) {
	    		res = 20;
	    		return res;
	    	}
		}
		catch (Exception err) {
			DebugInfo.printMessage(1,"===Error! RecordLayer::trimPaddingAndMAC: " + err);
    		res = 20;
    		return res;
		}
			
    	// parse TLS record
		DebugInfo.printMessage(4,"parse message");
    	parseTlsPlain(con.getReadPendingState());
    	if(error != 0) {
    		res = 50;
    		return res;
    	}

    	printMessages(2);
    	return res;
	}


    public byte[] getApplicationData() throws Exception {
     	int j,len = 0,pos = 0;
     	byte[] data = null;
     	ApplicationData curAppDt;
   		
   		if(this.tls_type == TLSContentType.APPLICATION) {
   			for(j=0; j< this.msglist.size(); j++) {
   				curAppDt = (ApplicationData)(this.msglist.get(j));
   				if(curAppDt.data != null)
   					len += curAppDt.data.length;
   			}
   			
   			data = new byte[len];
   			
   			for(j=0; j< this.msglist.size(); j++) {
   				curAppDt = (ApplicationData)(this.msglist.get(j));
   				if(curAppDt.data != null) {
   	   				System.arraycopy(curAppDt.data, 0, data, pos, curAppDt.data.length);
   	   				pos += curAppDt.data.length;
   				}
   			}
   		}
   		
   		return data;
    }

    public static byte[] getApplicationData(ArrayList<RecordLayer> tlsResp) throws Exception {
     	int j,len = 0,pos = 0;
     	byte[] data = null, curDt;
     	ArrayList<byte[]> msglist = new ArrayList<byte[]>();
   		
   		if(tlsResp != null) {
   			for(j=0; j< tlsResp.size(); j++) {
   				msglist.add(tlsResp.get(j).getApplicationData());
   			}

   			for(j=0; j< msglist.size(); j++) {
   				curDt = msglist.get(j);
   				if(curDt != null)
   					len += curDt.length;
   			}
   			
   			data = new byte[len];
   			
   			for(j=0; j< msglist.size(); j++) {
   				curDt = msglist.get(j);
   				if(curDt != null) {
   	   				System.arraycopy(curDt, 0, data, pos, curDt.length);
   	   				pos += curDt.length;
   				}
   			}
   		}

   		return data;
    }
}
