package ru.ispras.telecomtesk.tlsv12.TLSAlgorithms;

public enum TLSCipherSuite {
	TLS_NULL_WITH_NULL_NULL               (0x0000),
	
	TLS_RSA_WITH_NULL_MD5                 (0x0001),
	TLS_RSA_WITH_NULL_SHA                 (0x0002),
	TLS_RSA_WITH_NULL_SHA256              (0x003B),
	TLS_RSA_WITH_RC4_128_MD5              (0x0004),
	TLS_RSA_WITH_RC4_128_SHA              (0x0005),
	TLS_RSA_WITH_3DES_EDE_CBC_SHA         (0x000A),
	TLS_RSA_WITH_AES_128_CBC_SHA          (0x002F),
	TLS_RSA_WITH_AES_256_CBC_SHA          (0x0035),
	TLS_RSA_WITH_AES_128_CBC_SHA256       (0x003C),
	TLS_RSA_WITH_AES_256_CBC_SHA256       (0x003D),
	
	TLS_DH_DSS_WITH_3DES_EDE_CBC_SHA      (0x000D),
	TLS_DH_RSA_WITH_3DES_EDE_CBC_SHA      (0x0010),
	TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA     (0x0013),
	TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA     (0x0016),
	TLS_DH_anon_WITH_RC4_128_MD5          (0x0018),
	TLS_DH_anon_WITH_3DES_EDE_CBC_SHA     (0x001B),
	TLS_DH_DSS_WITH_AES_128_CBC_SHA       (0x0030),
	TLS_DH_RSA_WITH_AES_128_CBC_SHA       (0x0031),
	TLS_DHE_DSS_WITH_AES_128_CBC_SHA      (0x0032),
	TLS_DHE_RSA_WITH_AES_128_CBC_SHA      (0x0033),
	TLS_DH_anon_WITH_AES_128_CBC_SHA      (0x0034),
	TLS_DH_DSS_WITH_AES_256_CBC_SHA       (0x0036),
	TLS_DH_RSA_WITH_AES_256_CBC_SHA       (0x0037),
	TLS_DHE_DSS_WITH_AES_256_CBC_SHA      (0x0038),
	TLS_DHE_RSA_WITH_AES_256_CBC_SHA      (0x0039),
	TLS_DH_anon_WITH_AES_256_CBC_SHA      (0x003A),
	TLS_DH_DSS_WITH_AES_128_CBC_SHA256    (0x003E),
	TLS_DH_RSA_WITH_AES_128_CBC_SHA256    (0x003F),
	TLS_DHE_DSS_WITH_AES_128_CBC_SHA256   (0x0040),
	TLS_DHE_RSA_WITH_AES_128_CBC_SHA256   (0x0067),
	TLS_DH_anon_WITH_AES_128_CBC_SHA256   (0x006C),
	TLS_DH_DSS_WITH_AES_256_CBC_SHA256    (0x0068),
	TLS_DH_RSA_WITH_AES_256_CBC_SHA256    (0x0069),
	TLS_DHE_DSS_WITH_AES_256_CBC_SHA256   (0x006A),
	TLS_DHE_RSA_WITH_AES_256_CBC_SHA256   (0x006B),
	TLS_DH_anon_WITH_AES_256_CBC_SHA256   (0x006D),
	TLS_EMPTY_RENEGOTIATION_INFO_SCSV     (0x00FF),

	UNKNOWN (0x7FFF);

	public short suite;
	
	public short getValue() {
		return suite;
	}
	private TLSCipherSuite(int suite) {
		this.suite = (short)suite;
	}
	private TLSCipherSuite(short suite) {
		this.suite = suite;
	}
	public static TLSCipherSuite valueOf(int cs) {
		switch (cs) {
		case 0x0000: return TLS_NULL_WITH_NULL_NULL;
		
		case 0x0001: return TLS_RSA_WITH_NULL_MD5;
		case 0x0002: return TLS_RSA_WITH_NULL_SHA;
		case 0x003B: return TLS_RSA_WITH_NULL_SHA256;
		case 0x0004: return TLS_RSA_WITH_RC4_128_MD5;
		case 0x0005: return TLS_RSA_WITH_RC4_128_SHA;
		case 0x000A: return TLS_RSA_WITH_3DES_EDE_CBC_SHA;
		case 0x002F: return TLS_RSA_WITH_AES_128_CBC_SHA;
		case 0x0035: return TLS_RSA_WITH_AES_256_CBC_SHA;
		case 0x003C: return TLS_RSA_WITH_AES_128_CBC_SHA256;
		case 0x003D: return TLS_RSA_WITH_AES_256_CBC_SHA256;
		
		case 0x000D: return TLS_DH_DSS_WITH_3DES_EDE_CBC_SHA;
		case 0x0010: return TLS_DH_RSA_WITH_3DES_EDE_CBC_SHA;
		case 0x0013: return TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA;
		case 0x0016: return TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA;
		case 0x0018: return TLS_DH_anon_WITH_RC4_128_MD5;
		case 0x001B: return TLS_DH_anon_WITH_3DES_EDE_CBC_SHA;
		case 0x0030: return TLS_DH_DSS_WITH_AES_128_CBC_SHA;
		case 0x0031: return TLS_DH_RSA_WITH_AES_128_CBC_SHA;
		case 0x0032: return TLS_DHE_DSS_WITH_AES_128_CBC_SHA;
		case 0x0033: return TLS_DHE_RSA_WITH_AES_128_CBC_SHA;
		case 0x0034: return TLS_DH_anon_WITH_AES_128_CBC_SHA;
		case 0x0036: return TLS_DH_DSS_WITH_AES_256_CBC_SHA;
		case 0x0037: return TLS_DH_RSA_WITH_AES_256_CBC_SHA;
		case 0x0038: return TLS_DHE_DSS_WITH_AES_256_CBC_SHA;
		case 0x0039: return TLS_DHE_RSA_WITH_AES_256_CBC_SHA;
		case 0x003A: return TLS_DH_anon_WITH_AES_256_CBC_SHA;
		case 0x003E: return TLS_DH_DSS_WITH_AES_128_CBC_SHA256;
		case 0x003F: return TLS_DH_RSA_WITH_AES_128_CBC_SHA256;
		case 0x0040: return TLS_DHE_DSS_WITH_AES_128_CBC_SHA256;
		case 0x0067: return TLS_DHE_RSA_WITH_AES_128_CBC_SHA256;
		case 0x006C: return TLS_DH_anon_WITH_AES_128_CBC_SHA256;
		case 0x0068: return TLS_DH_DSS_WITH_AES_256_CBC_SHA256;
		case 0x0069: return TLS_DH_RSA_WITH_AES_256_CBC_SHA256;
		case 0x006A: return TLS_DHE_DSS_WITH_AES_256_CBC_SHA256;
		case 0x006B: return TLS_DHE_RSA_WITH_AES_256_CBC_SHA256;
		case 0x006D: return TLS_DH_anon_WITH_AES_256_CBC_SHA256;
		case 0x00FF: return TLS_EMPTY_RENEGOTIATION_INFO_SCSV;

		default: return UNKNOWN;
		}
	}
	public static String toString(int cs) {
		TLSCipherSuite tmpval = TLSCipherSuite.valueOf(cs);
		String value = "" + tmpval;
		if(tmpval == TLSCipherSuite.UNKNOWN) {
			String tmpstr = String.format("%04x", cs);
			if(tmpstr.length() > 4)
				tmpstr = tmpstr.substring(tmpstr.length()-4);
			value += "=0x" + tmpstr;
		}
		return value;
	}
}
 