package ru.ispras.telecomtesk.tlsv12.TLSSocket;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;

import ru.ispras.telecomtesk.tlsv12.DebugInfo;

public class TLSSocket 
{
    Socket skt = null;
    BufferedOutputStream out = null;
    DataInputStream in = null;

    boolean isAlive= true;
    
    TLSResponse socketResponse = null;
    
    protected String serverIP;
    protected int serverPort;
    protected long responseTimeout = 10000; // in milliseconds
    
    
    public TLSSocket(String server, int port) {
    	this.serverIP = server;
    	this.serverPort = port;
    	socketResponse = new TLSResponse();
    }

    public TLSSocket(String server, int port, int timeout) {
    	this.serverIP = server;
    	this.serverPort = port;
    	responseTimeout = timeout;
    	socketResponse = new TLSResponse();
    }

	public String getIP() {
		return serverIP;
	}

	public int getPort() {
		return serverPort;
	}

	public long getTimeout() {
		return responseTimeout;
	}

	public void setTimeout(long timeout) throws SocketException {
		responseTimeout = timeout;
		if(skt != null)
			skt.setSoTimeout((int)responseTimeout);
	}

	public void connect() throws UnknownHostException, IOException {
		System.out.println("Connecting " + this.serverIP + " port " + this.serverPort);
		try {
			skt = new Socket(this.serverIP, this.serverPort);
		}
		catch (IOException err){
			System.out.println("Connection failed");
			throw err;
		}
		System.out.println("Connected");
		out = new BufferedOutputStream(new DataOutputStream(skt.getOutputStream()));
		in = new DataInputStream(skt.getInputStream());
		skt.setSoTimeout((int)responseTimeout);
		waitForResponse();
	}
    
	public void disconnect() {
		if (skt != null){
			try {
				skt.close();
			}
			catch (IOException ignored) {
			}
		}
	}
	
    public void sendMessage(byte[] message) throws IOException
    {
    	DebugInfo.printMessage(5, "sendMessage start time: " + System.currentTimeMillis());
    	DebugInfo.printDump(5, message, "TLSSocket::sendMessage");

    	try {
    		out.write(message,0,message.length);
    		out.flush();
    	}
    	catch(IOException ioerr) {
			DebugInfo.printMessage(1,"===Error! TLSSocket::sendMessage: " + ioerr);
    		throw ioerr;
    	}
    	DebugInfo.printMessage(5, "sendMessage wait time: " + System.currentTimeMillis());

    	try {
    		waitForResponse();
    	}
    	catch(IOException ioerr) {
			DebugInfo.printMessage(1,"===Error! TLSSocket::waitForResponse: " + ioerr);
    		throw ioerr;
    	}
    	DebugInfo.printMessage(5, "sendMessage end time: " + System.currentTimeMillis());
    }
    
    
    public boolean isAlive() throws IOException {
        return isAlive;
    }
    
    public void waitForResponse() throws IOException
    {
    	long curtime, starttime;
    	int numbytes;

    	socketResponse.init();
    	starttime = System.currentTimeMillis();
    	DebugInfo.printMessage(5, "waitForResponse: starttime " + starttime + " responseTimeout " + responseTimeout);
    	
    	do {
        	try {
        		numbytes = socketResponse.readbytes(in);
        		if(socketResponse.responseTime == -1)
        			DebugInfo.printMessage(4, "waitForResponse: first " + System.currentTimeMillis());
        	}
        	catch (SocketTimeoutException err) {
        		break;
        	}
    		if(numbytes >0) socketResponse.shiftPosition(numbytes);
    		
    		curtime = System.currentTimeMillis();
    		if(socketResponse.responseTime == -1) {
    			socketResponse.responseTime = curtime - starttime;
    	    	DebugInfo.printMessage(5, "waitForResponse: get first data time " + curtime);
    		}
    		// if(hardtimeout) break;
    	} while ((in.available() > 0) || (curtime - starttime < responseTimeout));

    	DebugInfo.printMessage(5, "waitForResponse: curtime " + System.currentTimeMillis() + " responseTime " + socketResponse.responseTime);
    	socketResponse.printResponse();
    	DebugInfo.printMessage(6, "response: " + socketResponse.lastind + " " + socketResponse.lastpos);
    }
    
   
    public TLSResponse getResponse() {
    	return socketResponse;
    }

      
    public void doPause(double iTimeInSeconds){
        long t0, t1;
        t0=System.currentTimeMillis( );
        t1=System.currentTimeMillis( )+Math.round(iTimeInSeconds*1000);

        do {
	        t0=System.currentTimeMillis( );
	    } while (t0 < t1);
    }
    
    public void finilize() throws IOException {
        out.close();
        in.close();
        skt.close();
    }
   
}