package ru.ispras.telecomtesk.tlsv12.TLSState;

import java.util.ArrayList;
import java.util.Arrays;
import java.security.cert.CertificateFactory;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.io.*;

import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.*;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.TLSExtensions.*;
import ru.ispras.telecomtesk.tlsv12.HandshakeMessages.KeyExchange.*;
import ru.ispras.telecomtesk.tlsv12.TLSSocket.*;
import ru.ispras.telecomtesk.tlsv12.RecordProtocol.*;
import ru.ispras.telecomtesk.tlsv12.TLSAlgorithms.*;
import ru.ispras.telecomtesk.tlsv12.DebugInfo;
import ru.ispras.telecomtesk.tlsv12.TlsKeystore;

public class TLSServerSpec
{
	public TLSScenarioState currentState = TLSScenarioState.DISCONNECTED;
	public TLSScenarioState previousState = TLSScenarioState.DISCONNECTED;
	protected int checkHeader = -1;
	protected boolean isOutOfOrder = false;
	protected String strerr;
	protected Hello last_client_hello = null;
	protected boolean isFixedDH = false;
	public boolean isCertificateVerifyMustSend = false;
	protected boolean isServerKeyExchangeMustRecv = false;
	public boolean isSessionEstablish = false;
	protected boolean isChangeCipher = false;
	public boolean mayNoResponse = false;
	protected ClientCertificateType clientCertType = ClientCertificateType.UNKNOWN;
	public boolean isSessionResume = false;
	public boolean isGetCertificateRequest = false;
	public boolean noRenegotiation = false;
	public boolean secure_renegotiation = false;
	public boolean initial_handshake = true;
	public int waitFatalAlert = -1;
	public int disconnectWithSendAlert = -1;
	public long responseTime = -1;
	public ArrayList sessionMsgList = new ArrayList();
	public int last_client_hello_index = 0;
	public ServerDHParams serverDHparams = null;
	public SignatureAndHashAlgorithm[] sah = null;
	public X509Certificate serverCert = null;
	public Certificate localCert = null;
	public byte[] pre_master_secret = null;
	public byte[] client_verify_data = null;
	public byte[] server_verify_data = null;

	public TLSServerSpec() {
	}
	
	public TLSScenarioState getCurrentState() {
		return currentState;
	}
	
	public void setCurrentState(TLSScenarioState value) {
		this.currentState = value;
	}

	public boolean isHandshakeComplete() {
		if(currentState.connected && 
		   (   (isSessionResume && currentState == TLSScenarioState.AFTER_ClientFinished)
			|| (!isSessionResume && currentState == TLSScenarioState.AFTER_ServerFinished)))
					return true;
		else
			return false;
	}

	public boolean isHandshakeComplete(TLSScenarioState st) {
		if(st.connected && 
		   (   (isSessionResume && st == TLSScenarioState.AFTER_ClientFinished)
			|| (!isSessionResume && st == TLSScenarioState.AFTER_ServerFinished)))
					return true;
		else
			return false;
	}

	public void setClientCertificateType() {
		this.clientCertType = ClientCertificateType.valueOf(TlsKeystore.getSigAlg(localCert));
	}

	public void printFlags(int debugLevel) {
		DebugInfo.printMessage(debugLevel, "TLSServerSpec:");
		DebugInfo.printMessage(debugLevel, "   currentState: " + currentState.name());
		DebugInfo.printMessage(debugLevel, "   previousState: " + previousState.name());
		DebugInfo.printMessage(debugLevel, "   checkHeader: " + checkHeader);
		DebugInfo.printMessage(debugLevel, "   isOutOfOrder: " + isOutOfOrder);
		DebugInfo.printMessage(debugLevel, "   strerr: " + strerr);
		DebugInfo.printMessage(debugLevel, "   isFixedDH: " + isFixedDH);
		DebugInfo.printMessage(debugLevel, "   isCertificateVerifyMustSend: " + isCertificateVerifyMustSend);
		DebugInfo.printMessage(debugLevel, "   isServerKeyExchangeMustRecv: " + isServerKeyExchangeMustRecv);
		DebugInfo.printMessage(debugLevel, "   isGetCertificateRequest: " + isGetCertificateRequest);
		DebugInfo.printMessage(debugLevel, "   noRenegotiation: " + noRenegotiation);
		DebugInfo.printMessage(debugLevel, "   secure_renegotiation: " + secure_renegotiation);
		DebugInfo.printMessage(debugLevel, "   initial_handshake: " + initial_handshake);
		DebugInfo.printMessage(debugLevel, "   isSessionResume: " + isSessionResume);
		DebugInfo.printMessage(debugLevel, "   isSessionEstablish: " + isSessionEstablish);
		DebugInfo.printMessage(debugLevel, "   isChangeCipher: " + isChangeCipher);
		DebugInfo.printMessage(debugLevel, "   mayNoResponse: " + mayNoResponse);
		DebugInfo.printMessage(debugLevel, "   clientCertType: " + clientCertType);
		DebugInfo.printMessage(debugLevel, "   waitFatalAlert: " + waitFatalAlert);
		DebugInfo.printMessage(debugLevel, "   disconnectWithSendAlert: " + disconnectWithSendAlert);
		DebugInfo.printMessage(debugLevel, "   responseTime: " + responseTime);
		DebugInfo.printMessage(debugLevel, "   last_client_hello_index: " + last_client_hello_index);
		DebugInfo.printMessage(debugLevel, "   sessionMsgList size: " + sessionMsgList.size() + "\n");
	}
	
	public void connect(TLSConnection con) {
		DebugInfo.printMessage(1,"connect state");
		checkHeader = -1;
		isOutOfOrder = false;
		last_client_hello = null;
		isFixedDH = false;
		isCertificateVerifyMustSend = false;
		isServerKeyExchangeMustRecv = false;
		isGetCertificateRequest = false;
		noRenegotiation = false;
		secure_renegotiation = false;
		initial_handshake = true;
		isSessionResume = false;
		isSessionEstablish = false;
		isChangeCipher = false;
		mayNoResponse = false;
		clientCertType = ClientCertificateType.UNKNOWN;
		waitFatalAlert = -1;
		disconnectWithSendAlert = -1;
		responseTime = -1;
		previousState = currentState;
		sessionMsgList.clear();
		last_client_hello_index = 0;
		serverDHparams = null;
		sah = null;
		serverCert = null;
		localCert = null;
		pre_master_secret = null;
		client_verify_data = null;
		server_verify_data = null;
		con.init();
		setCurrentState(TLSScenarioState.CONNECTED);
			
		System.out.println("State = " + currentState + "\n");
		return;
	}

	public void disconnect() {
		DebugInfo.printMessage(3,"TLSServerSpec::disconnect");
		previousState = currentState;
		setCurrentState(TLSScenarioState.DISCONNECTED);
		System.out.println("State = " + currentState + "\n\n");
	}
	
	
	public void send(RecordLayer tlsmsg, TLSConnection con) throws Exception {
		Object msg;
		int i;

		DebugInfo.printMessage(3,"TLSServerSpec::send");

		if((!currentState.connected) || (tlsmsg == null))
				return;
		
		DebugInfo.printMessage(3,"TLSServerSpec::send 2");
		printFlags(5);
		DebugInfo.printMessage(6,"Write States\n");
		con.getWriteCurrentState().printDump(6);
		con.getWritePendingState().printDump(6);

		disconnectWithSendAlert = -1;
		
		if(tlsmsg.msglist.size() == 0)
			System.out.println("===Warning! TLSServerSpec::send: Message list is empty");

		System.out.println("send start");
		for(i=0; i < tlsmsg.msglist.size(); i++) {
			msg = tlsmsg.msglist.get(i);
				
			if(msg instanceof HandshakeMessage) {
				System.out.println("HandshakeMessage start");
				if(msg instanceof Hello) {
					System.out.println("Hello start");
					sendHello((Hello)msg,con);
				}
				else if(msg instanceof HelloRequest)
					sendHelloRequest((HelloRequest)msg);
				else if(msg instanceof HMCertificate)
					sendCertificate((HMCertificate)msg);
				else if(msg instanceof CertificateRequest)
					sendCertificateRequest((CertificateRequest)msg);
				else if(msg instanceof ClientKeyExchange)
					sendClientKeyExchange((ClientKeyExchange)msg);
				else if(msg instanceof ServerKeyExchange)
					sendServerKeyExchange((ServerKeyExchange)msg);
				else if(msg instanceof ServerHelloDone)
					sendServerHelloDone((ServerHelloDone)msg);
				else if(msg instanceof CertificateVerify)
					sendCertificateVerify((CertificateVerify)msg);
				else if(msg instanceof Finished)
					sendFinished((Finished)msg, con.getWriteCurrentState());
				else {
					if(tlsmsg.tls_type == TLSContentType.HANDSHAKE)
						sendHandshakeUndefinedType((HandshakeUndefinedType)msg);
					else {
						System.out.println("===Warning! TLSServerSpec::send: Unknown message type: " + tlsmsg.tls_type);
						sendUndefinedType();
					}
				}
			}
			else if(msg instanceof Alert) {
				sendAlert((Alert)msg);
			}
			else if(msg instanceof ChangeCipher) {
				sendChangeCipher((ChangeCipher)msg,con);
			}
			else if(msg instanceof ApplicationData) {
				sendApplicationData((ApplicationData)msg);
			}
			else {
				System.out.println("===Warning! TLSServerSpec::send: Unknown message type: " + tlsmsg.tls_type);
				sendUndefinedType();
			}
				
			sessionMsgList.add(msg);
			if(waitFatalAlert >= 0  ||  currentState == TLSScenarioState.DISCONNECTED) 
				break;
		}
		if(!isChangeCipher)
			con.getWriteCurrentState().incrementSequenceNumber();
		else
			isChangeCipher = false;
		
		System.out.println("send pre comlete\n");
		printFlags(5);
		DebugInfo.printMessage(6,"Write States\n");
		con.getWriteCurrentState().printDump(6);
		con.getWritePendingState().printDump(6);
		
		isOutOfOrder = false;
    }
		
		
	public void send(ArrayList<RecordLayer> tlsmsglst, TLSConnection con) throws Exception {
		Object msg;
		RecordLayer tlsmsg;
		int i;

		DebugInfo.printMessage(3,"TLSServerSpec::send ArrayList");

		if((!currentState.connected) || 
			(tlsmsglst == null) || (tlsmsglst.size() == 0))
				return;

		for(i=0;i<tlsmsglst.size();i++) {
			tlsmsg = tlsmsglst.get(i);
			if(tlsmsg == null)
				return;
		}
		
				
		DebugInfo.printMessage(3,"TLSServerSpec::send ArrayList 2");
		printFlags(5);
		DebugInfo.printMessage(6,"Write States\n");
		con.getWriteCurrentState().printDump(6);
		con.getWritePendingState().printDump(6);
		
		for(i=0; i < tlsmsglst.size(); i++) {
			tlsmsg = tlsmsglst.get(i);

			if(tlsmsg.msglist.size() == 0)
				System.out.println("===Warning! TLSServerSpec::send ArrayList: Message list is empty. TLS type: " + tlsmsg.tls_type + ", index " + i);

			System.out.println("send start");
			for(int j=0; j < tlsmsg.msglist.size(); j++) {
				msg = tlsmsg.msglist.get(j);
				
				if(msg instanceof HandshakeMessage) {
					System.out.println("HandshakeMessage start");
					if(msg instanceof Hello) {
						System.out.println("Hello start");
						sendHello((Hello)msg,con);
					}
					else if(msg instanceof HelloRequest)
						sendHelloRequest((HelloRequest)msg);
					else if(msg instanceof HMCertificate)
						sendCertificate((HMCertificate)msg);
					else if(msg instanceof CertificateRequest)
						sendCertificateRequest((CertificateRequest)msg);
					else if(msg instanceof ClientKeyExchange)
						sendClientKeyExchange((ClientKeyExchange)msg);
					else if(msg instanceof ServerKeyExchange)
						sendServerKeyExchange((ServerKeyExchange)msg);
					else if(msg instanceof ServerHelloDone)
						sendServerHelloDone((ServerHelloDone)msg);
					else if(msg instanceof CertificateVerify)
						sendCertificateVerify((CertificateVerify)msg);
					else if(msg instanceof Finished)
						sendFinished((Finished)msg, con.getWriteCurrentState());
					else {
						if(tlsmsg.tls_type == TLSContentType.HANDSHAKE)
							sendHandshakeUndefinedType((HandshakeUndefinedType)msg);
						else {
							System.out.println("===Warning! TLSServerSpec::send: Unknown message type: " + tlsmsg.tls_type);
							sendUndefinedType();
						}
					}
				}
				else if(msg instanceof Alert) {
					sendAlert((Alert)msg);
				}
				else if(msg instanceof ChangeCipher) {
					sendChangeCipher((ChangeCipher)msg,con);
				}
				else if(msg instanceof ApplicationData) {
					sendApplicationData((ApplicationData)msg);
				}
				else {
					System.out.println("===Warning! TLSServerSpec::send: Unknown message type: " + tlsmsg.tls_type);
					sendUndefinedType();
				}
				
				sessionMsgList.add(msg);
				if(waitFatalAlert >= 0  ||  currentState == TLSScenarioState.DISCONNECTED) 
					break;
			}
			if(!isChangeCipher)
				con.getWriteCurrentState().incrementSequenceNumber();
			else
				isChangeCipher = false;
				
			if(waitFatalAlert >= 0  ||  currentState == TLSScenarioState.DISCONNECTED) 
				break;
		}
		
		System.out.println("send pre comlete\n");
		printFlags(5);
		DebugInfo.printMessage(6,"Write States\n");
		con.getWriteCurrentState().printDump(6);
		con.getWritePendingState().printDump(6);
		
		isOutOfOrder = false;
    }
		
		
	public ArrayList<RecordLayer> receiveTLS(TLSResponse resp, TLSConnection con) throws Exception {	
		System.out.println("\nTLSServerSpec::receiveTLS");
		Object msg;
		int i;
		ArrayList<RecordLayer> response_msgs = null;

		RecordLayer msgrl;
		// cut response to TLS records
		response_msgs = cutResponse(resp, con);
		mayNoResponse = false;
		DebugInfo.printMessage(5,"TLSServerSpec::receiveTLS cutResponse ok");
    	if(disconnectWithSendAlert != -1) {
			DebugInfo.printMessage(5,"TLSServerSpec::receiveTLS disconnectWithSendAlert " + disconnectWithSendAlert);
    		idlingDecrypt(response_msgs, 0, con);
    		return null;
    	}
    	else if(response_msgs.size() == 0)
    		return response_msgs;
    	else if(currentState == TLSScenarioState.DISCONNECTED) {
			DebugInfo.printMessage(1,"===Error! Receive response in state DISCONNECTED");
    		idlingDecrypt(response_msgs, 0, con);
    		return null;
    	}

		previousState = currentState;

		DebugInfo.printMessage(4,"TLSServerSpec::receive: parse start");
		for(i=0; i < response_msgs.size(); i++) {
			msgrl = response_msgs.get(i);

			disconnectWithSendAlert = msgrl.decryptAndParseTLS(con);
            if( disconnectWithSendAlert != -1 ) {
        		DebugInfo.printMessage(1,"decryptAndParseTLS return " + disconnectWithSendAlert);
        		DebugInfo.printMessage(4,"waitFatalAlert " + waitFatalAlert);
                break ;
            }

            if( msgrl.version.minor != 3 && msgrl.tls_type != TLSContentType.ALERT ) {
        		DebugInfo.printMessage(1,"===Error! Bad minor TLS version");
        		DebugInfo.printMessage(4,"waitFatalAlert " + waitFatalAlert);
                break ;
            }

			DebugInfo.printMessage(6,"TLSServerSpec::receive: increment Sequence Number");
			con.getReadCurrentState().incrementSequenceNumber();
		    	
	    	KeyExchangeAlgorithm ke_alg = con.getReadPendingState().getParams().ke_alg;

			DebugInfo.printMessage(4,"TLSServerSpec::receive: check plain message");
			for(int j=0; j < msgrl.msglist.size(); j++) {
				msg = msgrl.msglist.get(j);

				if(msg instanceof HandshakeMessage) {
					if(msg instanceof Hello)
						receiveHello((Hello)msg,con);
					else if(msg instanceof HelloRequest)
						receiveHelloRequest((HelloRequest)msg);
					else if(msg instanceof HMCertificate) {
						try {
							receiveCertificate((HMCertificate)msg,ke_alg);
						}
						catch (Exception err) {}
					}
					else if(msg instanceof CertificateRequest)
						receiveCertificateRequest((CertificateRequest)msg,ke_alg);
					else if(msg instanceof ClientKeyExchange)
						receiveClientKeyExchange((ClientKeyExchange)msg);
					else if(msg instanceof ServerKeyExchange)
						receiveServerKeyExchange((ServerKeyExchange)msg,con.getReadPendingState().getParams());
					else if(msg instanceof ServerHelloDone)
						receiveServerHelloDone((ServerHelloDone)msg);
					else if(msg instanceof CertificateVerify)
						receiveCertificateVerify((CertificateVerify)msg);
					else if(msg instanceof Finished)
						receiveFinished((Finished)msg, con.getReadCurrentState());
					else {
						System.out.println("===Error! TLSServerSpec::receive: Unknown Handshake message type: " + ((HandshakeMessage)msg).msg_type);
						disconnectWithSendAlert = 10; // unexpected_message
						break;
					}
				}
				else if(msg instanceof Alert) {
					receiveAlert((Alert)msg);
				}
				else if(msg instanceof ChangeCipher) {
					receiveChangeCipher((ChangeCipher)msg,con);
				}
				else if(msg instanceof ApplicationData) {
					receiveApplicationData((ApplicationData)msg);
				}
				else {
					System.out.println("===Error! TLSServerSpec::receive: Unknown message type " + msgrl.tls_type);
					disconnectWithSendAlert = 10; // unexpected_message
					break;
				}
					
				sessionMsgList.add(msg);				
				if(disconnectWithSendAlert >= 0  ||  currentState == TLSScenarioState.DISCONNECTED) 
					break;
			}
			if(disconnectWithSendAlert >= 0  ||  currentState == TLSScenarioState.DISCONNECTED) 
				break;
		}

		if(disconnectWithSendAlert >= 0  ||  currentState == TLSScenarioState.DISCONNECTED)
			idlingDecrypt(response_msgs, i+1, con);

		printFlags(5);
		DebugInfo.printMessage(5,"Write States\n");
		con.getWriteCurrentState().printDump(5);
		con.getWritePendingState().printDump(5);
		DebugInfo.printMessage(5,"Read States\n");
		con.getReadCurrentState().printDump(5);
		con.getReadPendingState().printDump(5);
		
		isOutOfOrder = false;
			
		if(   currentState != TLSScenarioState.DISCONNECTED
		   && currentState != TLSScenarioState.AFTER_ServerHelloDone
		   && currentState != TLSScenarioState.AFTER_ServerFinished
		   && disconnectWithSendAlert < 0)
				return null;
			
		return response_msgs;
	}

	ArrayList<RecordLayer> cutResponse(TLSResponse resp, TLSConnection con) throws Exception {
		ArrayList<RecordLayer> resTlsList = null;

		if(resp != null) {
			resTlsList = resp.getTlsArrayCrypto();
			responseTime = resp.responseTime;
			if(resTlsList.size() == 0) {
				DebugInfo.printMessage(3,"TLSServerSpec::cutResponse: Message list is empty");
				if(waitFatalAlert > 0) {
					waitFatalAlert = -1;
					disconnectWithSendAlert = 80;
					DebugInfo.printMessage(3,"Wait fatal alert, but no response");
				}
				else if(   currentState == TLSScenarioState.DISCONNECTED
				   || currentState == TLSScenarioState.AFTER_ClientCertificate
				   || currentState == TLSScenarioState.AFTER_ClientKeyExchange
				   || currentState == TLSScenarioState.AFTER_CertificateVerify
				   || currentState == TLSScenarioState.AFTER_ClientChangeCipherSpec
				   || currentState == TLSScenarioState.AFTER_ServerCloseNotify
				   || (isSessionResume && currentState == TLSScenarioState.AFTER_ClientFinished)
				   || mayNoResponse)
						;
				else if(currentState == TLSScenarioState.AFTER_ClientCloseNotify) {
					disconnectWithSendAlert = -1;
					DebugInfo.printMessage(3,"Wait CloseNotify alert, but no response");
				}
				else {
					disconnectWithSendAlert = 80;
					DebugInfo.printMessage(3,"Unexpected no response. State " + currentState);
				}
			}

	    	if((waitFatalAlert > 0) && (resTlsList.get(0).tls_type != TLSContentType.ALERT)) {
				DebugInfo.printMessage(2,"===Error! TLSServerSpec::cutResponse: Alert message expected");
				idlingDecrypt(resTlsList,0,con);
				waitFatalAlert = -1;
				disconnectWithSendAlert = 10;
				DebugInfo.printMessage(3,"Wait fatal alert, but receive other message");
	    	}
		}
       	return resTlsList;
	}
	
	void sendHello (Hello msg, TLSConnection con) {
			if(msg.msg_type != 1 
			   || currentState.isClosingSession()) {
				isOutOfOrder = true;
				waitFatalAlert = 10;
			}
			else {
				checkHeader = checkHello(msg);
				if(checkHeader >= 0) {
					System.out.println("TLSServerSpec::checkHello: " + strerr);
					waitFatalAlert = checkHeader;
				}
				else if(checkRenegotiationExtension(msg) >= 0) {
					DebugInfo.printMessage(1, "TLSServerSpec::checkRenegotiationExtension failed");
					waitFatalAlert = 40;
				}
			}
			
			previousState = currentState;
			if(msg.msg_type == 1) {
				noRenegotiation = false;
				isSessionResume = false;
				last_client_hello = msg;
				last_client_hello_index = sessionMsgList.size() > 0 ? sessionMsgList.size() : 0;
				
				if(isSessionEstablish)
					setCurrentState(TLSScenarioState.AFTER_ClientHELLOResume);
				else
					setCurrentState(TLSScenarioState.AFTER_ClientHELLO);

				con.initAllPendingStates();
				con.setRandom(msg.rnd, true);
			}
			else if(msg.msg_type == 2)
				setCurrentState(TLSScenarioState.AFTER_ServerHello);
	}


	void sendHelloRequest (HelloRequest msg) {
			isOutOfOrder = true;
			waitFatalAlert = 10;
			mayNoResponse = true;
	}

	void sendServerHelloDone (ServerHelloDone msg) {
			isOutOfOrder = true;
			waitFatalAlert = 10;
			previousState = currentState;
			setCurrentState(TLSScenarioState.AFTER_ServerHelloDone);
	}

	void sendServerKeyExchange (ServerKeyExchange msg) {
			isOutOfOrder = true;
			waitFatalAlert = 10;
			previousState = currentState;
			setCurrentState(TLSScenarioState.AFTER_ServerKeyExchange);
	}

	void sendCertificateRequest (CertificateRequest msg) {
			isOutOfOrder = true;
			waitFatalAlert = 10;
			previousState = currentState;
			setCurrentState(TLSScenarioState.AFTER_CertificateRequest);
	}

	void sendCertificate (HMCertificate msg) {
			if((isGetCertificateRequest == false) || (currentState != TLSScenarioState.AFTER_ServerHelloDone)) {
				isOutOfOrder = true;
				waitFatalAlert = 10;
			}
			else {
				checkHeader = checkCertificate(msg);
				if(checkHeader >= 0) {
					System.out.println("TLSServerSpec::checkCertificate: " + strerr);
					waitFatalAlert = checkHeader;
				}
			}
			previousState = currentState;
			setCurrentState(TLSScenarioState.AFTER_ClientCertificate);
	}

	 void sendClientKeyExchange (ClientKeyExchange msg) {
			
			if( ((isGetCertificateRequest) && (currentState != TLSScenarioState.AFTER_ClientCertificate)) || 
				((!isGetCertificateRequest) && (currentState != TLSScenarioState.AFTER_ServerHelloDone)) ) {
				isOutOfOrder = true;
				waitFatalAlert = 10;
			}
			else {
				checkHeader = checkClientKeyExchange(msg);
				if(checkHeader >= 0) {
					System.out.println("TLSServerSpec::checkClientKeyExchange: " + strerr);
					waitFatalAlert = checkHeader;
				}
			}

			switch(msg.key_alg) {
			case RSA:
				pre_master_secret = new byte[48];
				pre_master_secret[0] = msg.client_version.major;
				pre_master_secret[1] = msg.client_version.minor;
				System.arraycopy(msg.random_bytes, 0, pre_master_secret, 2, 46);
				break;
			case DHE_DSS:
			case DHE_RSA:
			case DH_ANON:
				// set in scenario ClientKeyExchange by "target.pre_master_secret = msg.setDiffieHellmanValue"
			}

			DebugInfo.printDump(5, pre_master_secret, "pre_master_secret");
			previousState = currentState;
			setCurrentState(TLSScenarioState.AFTER_ClientKeyExchange);
	}

	void sendCertificateVerify (CertificateVerify msg) {
			if(currentState != TLSScenarioState.AFTER_ClientKeyExchange
			  || clientCertType == ClientCertificateType.UNKNOWN 
			  || isFixedDH == true) {
					isOutOfOrder = true;
					waitFatalAlert = 10;
			}
			else {
				checkHeader = checkCertificateVerify(msg);
				if(checkHeader >= 0) {
					System.out.println("TLSServerSpec::checkCertificateVerify: " + strerr);
					waitFatalAlert = checkHeader;
				}
			}
			
			previousState = currentState;
			setCurrentState(TLSScenarioState.AFTER_CertificateVerify);
	}

	void sendFinished (Finished msg, TLSState curState) {
			
			if(currentState != TLSScenarioState.AFTER_ClientChangeCipherSpec) {
				isOutOfOrder = true;
				waitFatalAlert = 10;
			}
			else {
				checkHeader = checkFinished(msg, curState, false);
				if(checkHeader >= 0) {
					System.out.println("TLSServerSpec::checkFinished: " + strerr);
					waitFatalAlert = checkHeader;
				}
			}
			
			previousState = currentState;
			setCurrentState(TLSScenarioState.AFTER_ClientFinished);
			if(waitFatalAlert < 0  &&  isSessionResume) {
				isSessionEstablish = true;
			}
				
			client_verify_data = new byte[msg.verify_data.length];
			System.arraycopy(msg.verify_data,0,client_verify_data,0,msg.verify_data.length);
	}

	void sendChangeCipher (ChangeCipher msg, TLSConnection con) {
			
			if(isSessionResume) {
				if(currentState != TLSScenarioState.AFTER_ServerFinished) { 
					isOutOfOrder = true;
					waitFatalAlert = 10;
				}
			}
			else if( (isCertificateVerifyMustSend && (currentState != TLSScenarioState.AFTER_CertificateVerify)) || 
					 (!isCertificateVerifyMustSend && (currentState != TLSScenarioState.AFTER_ClientKeyExchange)) ) {
				isOutOfOrder = true;
				waitFatalAlert = 10;
			}

			
			if(isOutOfOrder == false) {
				checkHeader = checkChangeCipher(msg);
				if(checkHeader >= 0) {
					System.out.println("TLSServerSpec::checkChangeCipher: " + strerr);
					waitFatalAlert = checkHeader;
				}
			}
			
			isChangeCipher = true;
			if(!isSessionResume) {
				try {
					DebugInfo.printMessage(5,"create MasterSecret");
					con.createMasterSecret(pre_master_secret);
				}
				catch (Exception err) {
					DebugInfo.printMessage(5,"===Error! Create MasterSecret failed");
				}
				try {
					DebugInfo.printMessage(5,"create SecretKeys");
					con.createSecretKeys();
				}
				catch (Exception err) {
					DebugInfo.printMessage(5,"===Error! Create SecretKeys failed");
				}
			}
			con.changeState(TLSConnection.StateDirection.WRITE_STATE);
			previousState = currentState;
			setCurrentState(TLSScenarioState.AFTER_ClientChangeCipherSpec);
	}

	void sendAlert (Alert msg) {
			System.out.println("TLSServerSpec::checkAlert: \n" + msg.toString());
			
			checkHeader = checkAlert(msg);
			if(checkHeader >= 0) {
				System.out.println("TLSServerSpec::checkAlert: " + strerr);
				waitFatalAlert = checkHeader;
			}

			previousState = currentState;

			if(msg.isCloseNotifyAlert()) {
				if(currentState == TLSScenarioState.AFTER_ClientCloseNotify) {
					isOutOfOrder = true;
					waitFatalAlert = 10;
				}
				else if(currentState == TLSScenarioState.AFTER_ServerCloseNotify) 
					setCurrentState(TLSScenarioState.DISCONNECTED);
				else
					setCurrentState(TLSScenarioState.AFTER_ClientCloseNotify);
			}
			else if(msg.level == 2) {
				setCurrentState(TLSScenarioState.DISCONNECTED);
			}
			else
				mayNoResponse = true;
	}

	void sendApplicationData (ApplicationData msg) {

			if(  !isSessionEstablish
			  || currentState == TLSScenarioState.AFTER_ClientCloseNotify) {
				isOutOfOrder = true;
				waitFatalAlert = 10;
			}
			
			mayNoResponse = true;
	}

	void sendHandshakeUndefinedType (HandshakeUndefinedType msg) {
			System.out.println("===Warning! TLSServerSpec::send: Unknown Handshake message type: " + ((HandshakeMessage)msg).msg_type);
			System.out.println(msg.toString());
			isOutOfOrder = true;
			waitFatalAlert = 10; // unexpected_message
	}

	void sendUndefinedType () {
			isOutOfOrder = true;
			waitFatalAlert = 10; // unexpected_message
	}

	void receiveHello(Hello msg, TLSConnection con) {
		if(msg.msg_type != 2  
		  || (currentState != TLSScenarioState.AFTER_ClientHELLO 
			&& currentState != TLSScenarioState.AFTER_ClientHELLOResume)) {
			isOutOfOrder = true;
			disconnectWithSendAlert = 10; // unexpected_message
		}
		else {
			checkHeader = checkHello(msg);
			if(checkHeader >= 0) {
				System.out.println("TLSServerSpec::receiveHello:checkHello: " + strerr);
				disconnectWithSendAlert = checkHeader;
			}
			else if(checkRenegotiationExtension(msg) >= 0) {
				DebugInfo.printMessage(1, "TLSServerSpec::checkRenegotiationExtension failed");
				disconnectWithSendAlert = 40;
			}
		}

		previousState = currentState;
		setCurrentState(TLSScenarioState.AFTER_ServerHello);
		if(!isOutOfOrder) // i.e. msg.msg_type == 2
		{
			con.setRandom(msg.rnd, false);
			con.setSessionID(msg.sessionID);
			con.setCompressionAlgorithm(msg.compression_methods[0]);
			con.setAlgorithmParams(TLSCipherSuite.valueOf(msg.cipher_suites[0]));

			if(last_client_hello != null && msg.compareSessionID(last_client_hello.sessionID)) {
				DebugInfo.printMessage(3, "TLSServerSpec::receiveHello: session is resumed");
				int r = setRenegotiateParameters(msg, con);
				if(r != -1) {
					DebugInfo.printMessage(3, "TLSServerSpec::receiveHello:setRenegotiateParameters: failed, error " + r);
					disconnectWithSendAlert = 80;
				}
				isSessionResume = true;
			}
			else {
				switch (con.getReadPendingState().getParams().ke_alg) {
				case DHE_DSS:
				case DHE_RSA:
				case DH_ANON:
					isServerKeyExchangeMustRecv = true;
					break;
				default:
					isServerKeyExchangeMustRecv = false;	
				}
			}
			if(initial_handshake)
				initial_handshake = false;
		}
	}

	void receiveHelloRequest(HelloRequest msg) {
		if(currentState.isClosingSession()) {
			isOutOfOrder = true;
			disconnectWithSendAlert = 10; // unexpected_message
		}
		else {
			checkHeader = checkHelloRequest(msg);
			if(checkHeader >= 0) {
				System.out.println("TLSServerSpec::checkHelloRequest: " + strerr);
				disconnectWithSendAlert = checkHeader;
			}
		}
	}

	void receiveClientKeyExchange(ClientKeyExchange msg) {
		isOutOfOrder = true;
		disconnectWithSendAlert = 10; // unexpected_message
	}

	void receiveCertificateVerify(CertificateVerify msg) {
		isOutOfOrder = true;
		disconnectWithSendAlert = 10; // unexpected_message
	}

	void receiveCertificate(HMCertificate msg, KeyExchangeAlgorithm ke_alg) throws Exception {
		if( ke_alg == KeyExchangeAlgorithm.DH_ANON 
			|| currentState != TLSScenarioState.AFTER_ServerHello ) {
				isOutOfOrder = true;
				disconnectWithSendAlert = 10; // unexpected_message
		}
		else {
			checkHeader = checkCertificate(msg);
			if(checkHeader >= 0) {
				System.out.println("TLSServerSpec::checkCertificate: " + strerr);
			disconnectWithSendAlert = checkHeader;
			}
		}

		try {
			CertificateFactory factory = CertificateFactory.getInstance("X509");
			serverCert = (X509Certificate)factory.generateCertificate( new ByteArrayInputStream(msg.getFirstCertificate()) );
			if(serverCert.getVersion() != 3) {  //  x509v3
				DebugInfo.printMessage(1,"===Error! receiveCertificate: recieved certificate x509 version failed: get " + serverCert.getVersion() + ", need 3(v3)");
				disconnectWithSendAlert = 43;						
			}
		}
		catch (Exception err) {
			DebugInfo.printMessage(1,"===Error! receiveCertificate: create certificate failed: " + err);
			serverCert = null;
			disconnectWithSendAlert = 50;
		}
		previousState = currentState;
		setCurrentState(TLSScenarioState.AFTER_ServerCertificate);
	}

	void receiveServerKeyExchange (ServerKeyExchange msg, TLSSecurityParameters params) {
		if( params.ke_alg == KeyExchangeAlgorithm.RSA || 
			params.ke_alg == KeyExchangeAlgorithm.DH_DSS || 
			params.ke_alg == KeyExchangeAlgorithm.DH_RSA || 
			((params.ke_alg == KeyExchangeAlgorithm.DH_ANON) && (currentState != TLSScenarioState.AFTER_ServerHello)) || 
			((params.ke_alg != KeyExchangeAlgorithm.DH_ANON) && (currentState != TLSScenarioState.AFTER_ServerCertificate)) ) { 
					isOutOfOrder = true;
					disconnectWithSendAlert = 10; // unexpected_message
		}
		else {
			checkHeader = checkServerKeyExchange(msg, params);
			if(checkHeader >= 0) {
				System.out.println("TLSServerSpec::checkServerKeyExchange: " + strerr);
				disconnectWithSendAlert = checkHeader;
			}
		}

		previousState = currentState;
		switch(params.ke_alg) {
		case DH_ANON:
		case DHE_DSS:
		case DHE_RSA:
			serverDHparams = msg.params;
		}
		setCurrentState(TLSScenarioState.AFTER_ServerKeyExchange);
	}

	void receiveCertificateRequest (CertificateRequest msg, KeyExchangeAlgorithm ke_alg) {
		if( ke_alg == KeyExchangeAlgorithm.DH_ANON || 
			((isServerKeyExchangeMustRecv) && (currentState != TLSScenarioState.AFTER_ServerKeyExchange)) || 
			((!isServerKeyExchangeMustRecv) && (currentState != TLSScenarioState.AFTER_ServerCertificate)) ) {	
					isOutOfOrder = true;
					disconnectWithSendAlert = 10; // unexpected_message
		} else {
			checkHeader = checkCertificateRequest(msg);
			if(checkHeader >= 0) {
				System.out.println("TLSServerSpec::checkCertificateRequest: " + strerr);
				disconnectWithSendAlert = checkHeader;
			}
		}

		isGetCertificateRequest = true;
		previousState = currentState;
		sah = msg.supported_signature_algorithms;
		setCurrentState(TLSScenarioState.AFTER_CertificateRequest);
	}

	void receiveServerHelloDone (ServerHelloDone msg) {
		if(isGetCertificateRequest) {
			if(currentState != TLSScenarioState.AFTER_CertificateRequest) {
				isOutOfOrder = true;
				disconnectWithSendAlert = 10; // unexpected_message
			}
		}
		else if( ((isServerKeyExchangeMustRecv) && (currentState != TLSScenarioState.AFTER_ServerKeyExchange)) || 	
			     ((!isServerKeyExchangeMustRecv) && (currentState != TLSScenarioState.AFTER_ServerCertificate)) ) {	
						isOutOfOrder = true;
						disconnectWithSendAlert = 10; // unexpected_message
		}
		
		if(isOutOfOrder == false) {
			checkHeader = checkServerHelloDone(msg);
			if(checkHeader >= 0) {
				System.out.println("TLSServerSpec::checkServerHelloDone: " + strerr);
				disconnectWithSendAlert = checkHeader;
			}
		}

		previousState = currentState;
		setCurrentState(TLSScenarioState.AFTER_ServerHelloDone);
	}

	void receiveFinished (Finished msg, TLSState curState) {
		if(currentState != TLSScenarioState.AFTER_ServerChangeCipherSpec) {
			isOutOfOrder = true;
			disconnectWithSendAlert = 10; // unexpected_message
		} else {
			checkHeader = checkFinished(msg, curState, true);
			if(checkHeader >= 0) {
				System.out.println("TLSServerSpec::checkFinished: " + strerr);
				disconnectWithSendAlert = checkHeader;
			}
		}

		previousState = currentState;
		setCurrentState(TLSScenarioState.AFTER_ServerFinished);
		if(disconnectWithSendAlert < 0  &&  !isSessionResume) {
			isSessionEstablish = true;
			TLSSession newsession = new TLSSession( curState.sessionID,
													curState.params.compression_alg,
													curState.params.cs,
													serverCert,
													curState.params.master_secret
													);
			TLSSessionCache.sessionCache.put(Arrays.hashCode(newsession.id), newsession);
		}

		server_verify_data = new byte[msg.verify_data.length];
		System.arraycopy(msg.verify_data,0,server_verify_data,0,msg.verify_data.length);
	}

	void receiveChangeCipher (ChangeCipher msg, TLSConnection con) {
		if( (currentState != TLSScenarioState.AFTER_ClientFinished) && 
			((currentState != TLSScenarioState.AFTER_ServerHello) || (isSessionResume == false)) ) {	
					isOutOfOrder = true;
					disconnectWithSendAlert = 10; // unexpected_message
		}
		else {
			checkHeader = checkChangeCipher(msg);
			if(checkHeader >= 0) {
				System.out.println("TLSServerSpec::checkChangeCipher: " + strerr);
				disconnectWithSendAlert = checkHeader;
			}
		}

		con.changeState(TLSConnection.StateDirection.READ_STATE);
		previousState = currentState;
		setCurrentState(TLSScenarioState.AFTER_ServerChangeCipherSpec);
	}

	void receiveAlert (Alert msg) {
		if((waitFatalAlert > 0) && (msg.level != 2)) {
			waitFatalAlert = -1;
			disconnectWithSendAlert = 80;
			return;
		}
			
		checkHeader = checkAlert(msg);
		if(checkHeader >= 0) {
			System.out.println("TLSServerSpec::checkAlert: " + strerr);
			disconnectWithSendAlert = checkHeader;
		}
		else {
			if(msg.level == 1) {
				if(msg.description == 0) {
					if(currentState == TLSScenarioState.AFTER_ServerCloseNotify) {
						isOutOfOrder = true;
						disconnectWithSendAlert = 10;							
					} else if(currentState == TLSScenarioState.AFTER_ClientCloseNotify) {
						previousState = currentState;
						setCurrentState(TLSScenarioState.DISCONNECTED);
					} else {
						previousState = currentState;
						setCurrentState(TLSScenarioState.AFTER_ServerCloseNotify);
					}
				}
				else if(msg.description == 100) {
					if( (currentState == TLSScenarioState.AFTER_ClientHELLO
							|| currentState == TLSScenarioState.AFTER_ClientHELLOResume) 
						 && isSessionEstablish) {
						DebugInfo.printMessage(3,"receiveAlert: no renegotiation");
						noRenegotiation = true;
						disconnectWithSendAlert = 0;
					}
					else {
						isOutOfOrder = true;
						disconnectWithSendAlert = 10;							
					}
				}
			}
			else if(msg.level == 2) {
				disconnectWithSendAlert = -1;
				previousState = currentState;
				setCurrentState(TLSScenarioState.DISCONNECTED);
				if(msg.description != waitFatalAlert) {
					DebugInfo.printMessage(1,"===Warning! receiveAlert: wait alert {2," + waitFatalAlert + "}, but received " + msg.toStringShort());
					isOutOfOrder = true;
				}
				waitFatalAlert = -1;
			}
		}
	}

	void receiveApplicationData (ApplicationData msg) {
		try {
			PrintWriter wrt = new PrintWriter(new OutputStreamWriter(new FileOutputStream("responseApplicationData.txt",true)));
			DebugInfo.printMessage(1,"TLSServerSpec::receive: write application data to file");
			wrt.print(new String(msg.data));
			wrt.close();
		} catch (FileNotFoundException ignored) {}
			
		if(  !isSessionEstablish
		  || currentState == TLSScenarioState.AFTER_ServerCloseNotify) {
			isOutOfOrder = true;
			disconnectWithSendAlert = 10; // unexpected_message
		}
	}


	int checkHello(Hello msg) {
		int alert = -1, i, ind;

		if(msg.msg_type == 2 && !msg.version.isTLS12())
					alert = 70;
		else if(msg.rnd.length != 32 
			||  msg.sessionID.length > 32 
			||  msg.cipher_suites.length == 0 
			||  (msg.msg_type == 2 && msg.cipher_suites.length != 1)
			||  msg.compression_methods.length != 1 
			||  msg.compression_methods[0] != 0)
					alert = 47;
		else if( ( TLSCipherSuite.UNKNOWN == TLSCipherSuite.valueOf(msg.cipher_suites[0]) 
				   || TLSCipherSuite.TLS_EMPTY_RENEGOTIATION_INFO_SCSV == TLSCipherSuite.valueOf(msg.cipher_suites[0]))
				 && msg.cipher_suites.length == 1)
			alert = 40;
		else if(msg.checkDuplicateExtensions())
				alert = 50;
		else if(msg.msg_type == 2) {
			if(last_client_hello == null)
				alert = 10;
			else {
				if(last_client_hello.ntoh(last_client_hello.rnd) > last_client_hello.ntoh(msg.rnd)) 
					// uint32 gmt_unix_time of Random structure
					System.out.println("===Warning! TLSServerSpec::checkHello: ServerHello.time < ClientHello.time");

				for(i=0; i<last_client_hello.cipher_suites.length; i++)
					if(msg.cipher_suites[0] == last_client_hello.cipher_suites[i])
						break;
				if(i == last_client_hello.cipher_suites.length) 
					return 47;
				
				HelloExtension curext;
				for(i=0; i<msg.extensions.size(); i++) {
					curext = (HelloExtension)msg.extensions.get(i);
					if(curext.type == (short)0xFF01)
						continue;
					ind = last_client_hello.indexOfExtension(curext.type);
					if(ind == -1 || curext.type == 13) {
						alert = 110;
						break;
					}
				}
			}
		}

		DebugInfo.printMessage(5, "TLSServerSpec::checkHello: return " + alert);
		return alert;
	}

	int checkRenegotiationExtension(Hello msg) {
		int res = -1, i, ind;

		DebugInfo.printMessage(4, "TLSServerSpec::checkRenegotiationExtension start");
        RenegotiationExtension rn_ext = msg.getRenegotiationExtension();

        if(msg.msg_type == 1) {
			if(initial_handshake) {
				if(rn_ext != null  &&  !rn_ext.isEmpty())
					res = 1;
			}
			else {
				if(msg.checkSCSV())
					res = 2;
				else {
                    if(secure_renegotiation == false  &&  rn_ext != null)
						res = 3;
					else if(secure_renegotiation == true) {
						if(rn_ext == null  ||  !rn_ext.checkData(client_verify_data))
							res = 4;
					}
				}
			}
		}
		else { // msg.msg_type == 2
			if(initial_handshake) {
				if(rn_ext != null) {
					if(last_client_hello != null
					  && !last_client_hello.checkSCSV()
					  && last_client_hello.getRenegotiationExtension() == null)
						res = 1;
					else if(!rn_ext.isEmpty())
						res = 2;
					else
						secure_renegotiation = true;
				}
			}
			else {
                if(secure_renegotiation == false  &&  rn_ext != null)
					res = 3;
				else if(secure_renegotiation == true) {
					if(rn_ext == null  ||  !rn_ext.checkData(client_verify_data,server_verify_data))
						res = 4;
				}
			}
		}

		DebugInfo.printMessage(4, "TLSServerSpec::checkRenegotiationExtension: return " + res);
		return res;
	}

	int checkHelloRequest(HelloRequest msg) {
		return -1;
	}

	int checkCertificate(HMCertificate msg) {
		int res = -1;
		
		if(msg.certificate_list != null) 
			for(int i=0;i < msg.certificate_list.length;i++) {
				if(msg.certificate_list[i] != null  &&  msg.certificate_list[i].length == 0)
				res = 10;
			}
		else
			res = 10;
		
		return res;
	}

	int checkCertificateRequest(CertificateRequest msg) {
		int alert = -1;
		int i;
		
		if(  msg.certificate_types.length == 0 
		  || msg.certificate_types.length > 255 // 1..2^8-1
		  || msg.supported_signature_algorithms.length == 0
		  || msg.supported_signature_algorithms.length > 65535) // 2..2^16-1  see http://www.rfc-editor.org/errata_search.php?rfc=5246
				alert = 47;
		else {
			for(i=0; i<msg.certificate_types.length; i++) {
				if(msg.certificate_types[i] == ClientCertificateType.UNKNOWN.getValue())
					return 47;
			}
			for(i=0; i<msg.supported_signature_algorithms.length; i++) {
				if(msg.supported_signature_algorithms[i].isUnsupportingAlgorithm())
					return 47;
			}
			for(i=0; i<msg.certificate_authorities.length; i++) {
				if( msg.certificate_authorities[i].length() == 0 
				 || msg.certificate_authorities[i].length() > 65535) // 2^16-1
					return 47;
			}
		}
		
		return alert;
	}

	int checkClientKeyExchange(ClientKeyExchange msg) {
		//  RSA  PreMasterSecret  EncryptedPreMasterSecret
		//  :  isFixedDh,   ,  - dh_Yc 
		return -1;
	}

	int checkServerKeyExchange(ServerKeyExchange msg, TLSSecurityParameters params) {
                if(msg.key_alg != params.ke_alg)
			return 1;
		switch(params.ke_alg) {
		case DH_ANON:
		case DHE_DSS:
		case DHE_RSA:
			if(msg.params == null)
				return 2;
			if(msg.params.isDHempty())
				return 3;
			if(false == msg.validateDS(params, serverCert.getPublicKey()))
				return 4;
		}
		return -1;
	}

	int checkServerHelloDone(ServerHelloDone msg) {
		return -1;
	}

	int checkCertificateVerify(CertificateVerify msg) {
		return -1;
	}

	int checkFinished(Finished msg, TLSState currentState, boolean isReceivedFromServer) {
		int res = -1;

		DebugInfo.printMessage(4,"checkFinished: sessionMsgList " + sessionMsgList.size() + " " + last_client_hello_index);
		if((msg != null) && (msg.verify_data == null)) {
			DebugInfo.printMessage(1,"===Warning! checkFinished: verifyData not set");
			try {
				msg.calcVerifyData( currentState.getParams(), 
									sessionMsgList, 
									last_client_hello_index, 
									sessionMsgList.size() - last_client_hello_index);
			}
			catch (Exception err) {
				DebugInfo.printMessage(1,"===Error! checkFinished: calcVerifyData failed: " + err);
				return 51;
			}
			return res;
		}

		Finished testMsg = new Finished(isReceivedFromServer);
		try {
			testMsg.calcVerifyData( currentState.getParams(), 
								sessionMsgList, 
								last_client_hello_index, 
								sessionMsgList.size() - last_client_hello_index);
		}
		catch (Exception err) {
			DebugInfo.printMessage(1,"===Error! checkFinished: calcVerifyData failed: " + err);
			return 51;
		}

		if(testMsg.verify_data.length != msg.verify_data.length) {
			DebugInfo.printMessage(1,"===Error! checkFinished: compare data length failed: " + testMsg.verify_data.length + " " + msg.verify_data.length);
			res = 51;
		}
		else {
			for(int i=0; i < msg.verify_data.length; i++)
				if(testMsg.verify_data[i] != msg.verify_data[i]) {
					DebugInfo.printMessage(1,"===Error! checkFinished: compare data failed: ");
					DebugInfo.printDump(1,msg.verify_data,"===Error! checkFinished: get: ");
					DebugInfo.printDump(1,testMsg.verify_data,"===Error! checkFinished: need: ");
					res = 51;
					break;
				}
		}
		return res;
	}

	int checkAlert(Alert msg) {
		return (msg.level == 1 || msg.level == 2) ? -1 : 10;
	}

	int checkChangeCipher(ChangeCipher msg) {
		return (msg.flag == 1) ? -1 : 10;
	}

	public void sendApplicationData(RecordLayer tlsmsg, TLSConnection con) throws Exception {
		DebugInfo.printMessage(3,"TLSServerSpec::sendApplicationData");
			
		boolean res = true;
			
		if((!currentState.connected) ||	(tlsmsg == null))
				return;

		if(tlsmsg.msglist.size() == 0)
			System.out.println("===Warning! TLSServerSpec::sendApplicationData: Message list is empty");

		for(int i=0; i < tlsmsg.msglist.size(); i++) {
			if(!(tlsmsg.msglist.get(i) instanceof ApplicationData)) {
				res = false;
				break;
			}
		}
			
		if(  !isSessionEstablish
				  || currentState == TLSScenarioState.AFTER_ClientCloseNotify) {
			isOutOfOrder = true;
			disconnectWithSendAlert = 10; // unexpected_message
		}

		if(res)
			con.getWriteCurrentState().incrementSequenceNumber();
			
		isOutOfOrder = false;

		return;
	}
		
	public ArrayList<RecordLayer> receiveApplicationData(TLSResponse resp, TLSConnection con) throws Exception {
			DebugInfo.printMessage(3,"TLSServerSpec::receiveApplicationData");

			if((resp == null) || (resp.err != 0))
				return null;
		
			Object msg;
			int i = 0;
			RecordLayer msgrl;
			// cut response to TLS records
			ArrayList<RecordLayer> response_msgs = resp.getTlsArrayCrypto();
	    	
			if(response_msgs.size() == 0) {
				DebugInfo.printMessage(3,"TLSServerSpec::receive: Message list is empty");
					return response_msgs;
			}

			if(  !isSessionEstablish
					  || currentState == TLSScenarioState.AFTER_ServerCloseNotify) {
				isOutOfOrder = true;
				disconnectWithSendAlert = 10; // unexpected_message
			}
			
			DebugInfo.printMessage(4,"TLSServerSpec::receive: parse start");
			for(i=0; i < response_msgs.size(); i++) {
				if(disconnectWithSendAlert >= 0) {
					i = -1; // used below for idlingDecrypt(response_msgs, i+1, con)
					break;				
				}
				msgrl = response_msgs.get(i);
				if(i==0 && (waitFatalAlert > 0) && (msgrl.tls_type != TLSContentType.ALERT)) {
					DebugInfo.printMessage(2,"===Error! TLSServerSpec::receive: Alert message expected");
					waitFatalAlert = -1;
					disconnectWithSendAlert = 0;
					i = -1; // used below for idlingDecrypt(response_msgs, i+1, con)
					break;
				}
			
				disconnectWithSendAlert = msgrl.decryptAndParseTLS(con);
				if(disconnectWithSendAlert != -1)
					break;
	    	
				DebugInfo.printMessage(6,"TLSServerSpec::receive: increment Sequence Number");
				con.getReadCurrentState().incrementSequenceNumber();
		    	
				DebugInfo.printMessage(4,"TLSServerSpec::receive: check plain message");
				for(int j=0; j < msgrl.msglist.size(); j++) {
					msg = msgrl.msglist.get(j);

					if(msg instanceof Alert) {
						Alert alr = (Alert)msg;
						if((waitFatalAlert > 0) && (alr.level != 2)) {
							waitFatalAlert = -1;
							disconnectWithSendAlert = 80;
							break;
						}
						
						checkHeader = checkAlert(alr);
						if(checkHeader >= 0) {
							System.out.println("TLSServerSpec::checkAlert: " + strerr);
							disconnectWithSendAlert = checkHeader;
							break;
						}
						else {
							if(alr.level == 1) {
								System.out.println(alr.toString());
								if(alr.description == 0) {
									if(currentState == TLSScenarioState.AFTER_ServerCloseNotify) {
										isOutOfOrder = true;
										disconnectWithSendAlert = 10;
										break;
									}
									else if(currentState == TLSScenarioState.AFTER_ClientCloseNotify)
										setCurrentState(TLSScenarioState.DISCONNECTED);
									else
										setCurrentState(TLSScenarioState.AFTER_ServerCloseNotify);
								}
							}
							else if(alr.level == 2) {
								disconnectWithSendAlert = -1;
								setCurrentState(TLSScenarioState.DISCONNECTED);
								if(alr.description != waitFatalAlert) {
									isOutOfOrder = true;
									waitFatalAlert = -1;
									break;
								}
								waitFatalAlert = -1;
							}
						}
					}
					else if(msg instanceof ApplicationData) {
						try {
							PrintWriter wrt = new PrintWriter(new OutputStreamWriter(new FileOutputStream("responseApplicationData.txt",true)));
							DebugInfo.printMessage(6,"TLSServerSpec::receive: write application data to file");
							wrt.print(new String(((ApplicationData)msg).data));
							wrt.close();
						}
						catch (FileNotFoundException ignored) {}
					}
					else {
						System.out.println("===Error! TLSServerSpec::receive: Unknown message type");
						disconnectWithSendAlert = 10; // unexpected_message
						break;
					}
					
				}
				if(disconnectWithSendAlert >= 0 || isOutOfOrder)
					break;
			}

			if(disconnectWithSendAlert >= 0 || isOutOfOrder)
				idlingDecrypt(response_msgs, i+1, con);

			return response_msgs;
	}

	void idlingDecrypt(ArrayList<RecordLayer> msglst, int ind, TLSConnection con) {
		RecordLayer msgrl;
		Object msg;
		
		DebugInfo.printMessage(3,"TLSServerSpec::idlingDecrypt, message index " + ind);

		if(msglst == null  ||  con == null) return;

		for(int i=ind; i < msglst.size(); i++) {
			msgrl = msglst.get(i);
			if(-1 != msgrl.decryptAndParseTLS(con))
				return;
			
			DebugInfo.printMessage(6,"TLSServerSpec::idlingDecrypt: increment Sequence Number");
			con.getReadCurrentState().incrementSequenceNumber();

			for(int j=0; j < msgrl.msglist.size(); j++) {
				msg = msgrl.msglist.get(j);

				if(msg instanceof HandshakeMessage) {
					if(msg instanceof Hello) {
						Hello hl_msg = (Hello)msg;
						if(hl_msg.msg_type != 2)
							continue;
						if(last_client_hello != null && hl_msg.compareSessionID(last_client_hello.sessionID))
							isSessionResume = true;
						con.setRandom(hl_msg.rnd, false);
						con.setSessionID(hl_msg.sessionID);
						con.setCompressionAlgorithm(hl_msg.compression_methods[0]);
						con.setAlgorithmParams(TLSCipherSuite.valueOf(hl_msg.cipher_suites[0]));
						switch (con.getReadPendingState().getParams().ke_alg) {
							case DHE_DSS:
							case DHE_RSA:
							case DH_ANON:
								isServerKeyExchangeMustRecv = true;
								break;
							default:
								isServerKeyExchangeMustRecv = false;	
						}
					}
					else if(msg instanceof HMCertificate) {
						HMCertificate cert_msg = (HMCertificate)msg;
						try {
							CertificateFactory factory = CertificateFactory.getInstance("X509");
							serverCert = (X509Certificate)factory.generateCertificate( new ByteArrayInputStream(cert_msg.getFirstCertificate()) );
						}
						catch (Exception err) {
							DebugInfo.printMessage(1,"===Error! idlingDecrypt: create certificate failed: " + err);
							serverCert = null;
						}
					}
				}
				else if(msg instanceof ChangeCipher) {
					con.changeState(TLSConnection.StateDirection.READ_STATE);
					setCurrentState(TLSScenarioState.AFTER_ServerChangeCipherSpec);
				}
			}
		}
			
		DebugInfo.printMessage(3,"TLSServerSpec::idlingDecrypt return");
		return;
	}

	private int setRenegotiateParameters(Hello msg, TLSConnection current_connection) {
		int res = -1;
		TLSSecurityParameters cur_par, pend_par;
		
		DebugInfo.printMessage(4,"TLSServerSpec::setRenegotiateParameters");

		if(msg == null  ||  current_connection == null)
			res = 1;
		else if(msg.msg_type != 2)
			res = 2;
		else {
			TLSSession session = TLSSessionCache.sessionCache.get(Arrays.hashCode(msg.sessionID));
			if(session == null)
				res = 3;
			else if(msg.cipher_suites[0] != session.cipher_spec.getValue())
				res = 4;
			else if(msg.compression_methods[0] != session.compression_method)
				res = 5;
			/* Most current TLS extensions are relevant only
			   when a session is initiated: when an older session is resumed, the
			   server does not process these extensions in Client Hello, and does
			   not include them in Server Hello. */
			//else if(!checkReceivedExtensions(msg.extensions))
			//	res = 6;
			else { // create parameters
				try {
					DebugInfo.printMessage(5,"set MasterSecret");
					current_connection.setMasterSecret(session.master_secret);
				}
				catch (Exception err) {
					DebugInfo.printMessage(5,"===Error! Set MasterSecret failed");
					res = 7;
				}
				if(res < 0) {
					try {
						DebugInfo.printMessage(5,"create SecretKeys");
						current_connection.createSecretKeys();
					}
					catch (Exception err) {
						DebugInfo.printMessage(5,"===Error! Create SecretKeys failed");
						res = 8;
					}
				}
			}
		}
		return res;
	}
	
	public byte[] DisconnectWithAlert(TLSConnection current_connection) {
		byte[] msg = null;
		RecordLayer tlsmsg = new RecordLayer();
		tlsmsg.msglist = new ArrayList();
		try {
			isOutOfOrder = false;
			if(disconnectWithSendAlert >= 0  &&  currentState != TLSScenarioState.DISCONNECTED) {
				System.out.println("\nSend closing alert");
				Alert al = new Alert(2,disconnectWithSendAlert);
				if(disconnectWithSendAlert == 0)
					al.level = 1;
				tlsmsg.msglist.clear();
				tlsmsg.msglist.add(al);
				tlsmsg.setContent(al,null,current_connection);
				disconnectWithSendAlert = -1;
				msg = tlsmsg.toByteArray();
			}
		} catch (Exception ignored) {				
		} finally {
			System.out.println("Closing session");
			disconnect();	
		}
		return msg;
	}
}
